//package BayesNetwork;

//import GenExceptions.*;

import java.util.*;

abstract public class BayesNetwork {

	abstract public BayesNetworkNode createNode(String[] values, double[] prob) throws ProbabilityException;
	// requires: values and prob must not be null
	// effects:  Creates and returns a BayesNetworkNode which is added to this BayesNetwork.  The node is
	//           initialized to have the values in values[] with initial probabilities contained in
	//			 probs[], where each entry in probs[] corresponds to the value of equal index in values[].
	//			 A ProbabilityException occurs if the size of values and probs is not equal, if a value
	//			 appears more than once in values[], or if probs[] do not sum close to 1.0

	abstract public Enumeration getNodes();
	// effects:  Returns an Enumeration of BayesNetworkNodes contained in the network.

	abstract public void removeNode(BayesNetworkNode node);
	// requires: node is not null and was created from the createNode method of this network.
	// effects:  Removes node and all of its arcs from the network.  If node has already be removed, nothing is done.

	abstract public void createArc(BayesNetworkNode source, BayesNetworkNode dest) throws InvalidArcException;
	// requires: source and dest are not null, and were created by the network.
	// effects: Creates an arc from source to dest in the network.  This operation reinitializes the probabilty
	//			table of the dest node, using a uniform distribution for its values conditioned on parent values.
	//			If either node is no longer in the network, nothing is done.
	//			If the arc would create a loop, an InvalidArcExcpetion is thrown and the network is unchanged.

	abstract public void removeArc(BayesNetworkNode source, BayesNetworkNode dest);
	// requires: source and dest are not null, and were created by the network.
	// effects:  Removes the arc from source to dest if one exists.  This operation reinitializes the probability
	//			 table of the dest node, using a uniform distribution for its values conditioned on parent values.
	//			 If either node is no longer in the network, nothing is done.

	abstract public double[] inferenceOnNode(BayesNetworkNode node) throws ProbabilityException;
	// requires: node is not null and was created by this network.
	// effects:  Returns a double array containing the current infernence probabilities on this node,
	//			 where each entry in the double array corresponds to the values from node.getValues array.
	//			 If this node has been removed, then null is returned.
	//			 If there are inconsistancies in the network, a ProbabilityException is thrown.
}
