//package BayesNetwork;

//import GenExceptions.*;

import java.util.*;

abstract public class BayesNetworkNode {

	abstract public int getUniqueID();
	// effects: get the unique id of this node

	abstract public String[] getValues();
	// effects: returns the values this node can take on.
	//  They are returned in the same order as they were for this nodes creation

	abstract public double[] getProb(BayesNetworkNode[] parentNodes, String[] parentValues) 
		throws ProbabilityException, InvalidValueException;
	// effects: Returns the probs of each values (the order of probs correspond to the order of
	//			values in getValues()) when the parent nodes have the parentValues (where each
	//			parent values corresponds with the parent with the same array position in parentNodes).
	//			If this node has no parents, parentNodes and parentValues should be set to null.
	//			A ProbabilityException occurs if all parents nodes are not given once in the parentNodes,or
	//          if not enough or too many parent values are given.
	//			An invalidValueException occurs if a parentValue is invalid for its parent

	abstract public void changeProbFunction(BayesNetworkNode[] parentNodes, String[] parentValues, 
						 double[] probsForValues) throws ProbabilityException, InvalidValueException;
	// effects: For the parentNodes and their values given in parentValues (where a parent's value is at
	//			the same array index in the parentValues array), this node's value probs are set by
	//			probsForValues, where each value in this array refers to the value at this array index
	//			returned by getValues().
	//			If this node has no parents, parentNodes and parentValues should be set to null.
	//			A probabilityException is thrown if all parent nodes are not given once, if the
	//			the number of parent values is wrong, or if the wrong number of probs are given for this
	//			nodes values.
	//			An invalidValueException is thrown if a parent's value is invalid.
	//			The node is left unchanged if an exception occurs

	abstract public void makeObservation(String valueObserved) throws InvalidValueException, ProbabilityException;
	// effects: Set this nodes observed value to valueObserved
	//			An invalidValueException is thrown if the value is not valid.
	//			An exception leaves this node unchanged.

	abstract public void clearObservation();
	// effects: clear any obseravtion on this node.

	abstract public String getObservation();
	// effects: returns this nodes observed value.  It returns null if no observation has been made

	abstract public Enumeration getChildren();
	// effects: returns an enumeration of this nodes children nodes

	abstract public Enumeration getParents();
	// effects: returns an enumeration of this nodes parent nodes
}



