//package Disease;

//import Person.*;
//import BayesNetwork.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.LineBorder;

public class DiseaseDialog extends JDialog {

	private Disease newDisease;

	private final String simpleDiseaseTitle = "Single diallelic locus disease";
	private boolean bAllelesSet;

	public DiseaseDialog(JFrame frame, Disease d) {
		// requires:  frame is not null
		// effects:  Creates a dialog for the current disease d.  If null, creates a dialog for a new disease.

		super(frame, "Edit Pedigree Disease", true);
		this.setResizable(false);

		newDisease = null;

		JPanel simpleDiseasePanel = newSimpleDiseasePanel(d);
		setContentPane(simpleDiseasePanel);
		updateDialogDisplay();
	}

	public Disease getNewDisease() {
		// effects:  Returns a disease if the user changed the input disease or if he created a new one.
		//			 Returns null if the user canceled the dialog box, making no changes
		return newDisease;
	}

	private JPanel newSimpleDiseasePanel(Disease d) {
		bAllelesSet = false;

		JPanel allelePanel = new JPanel();
		final JRadioButton dom_recAutoButton = new JRadioButton("dominant/recessive alleles", true);
		final JRadioButton co_domAutoButton = new JRadioButton("codominant alleles");
		final JRadioButton dom_recXlinkedButton = new JRadioButton("dominant/recessive alleles", true);
		final JRadioButton co_domXlinkedButton = new JRadioButton("codominant alleles");
		final JPanel domrecPhenPanel = new JPanel();		
		final JPanel codomPhenPanel = new JPanel();
		final JPanel noPhenPanel = new JPanel();

		final String sdomAlleleType = new String("Dominant Allele");
		final String srecAlleleType = new String("Recesssive Allele");
		final JLabel Allele1Label = new JLabel(sdomAlleleType);
		final JLabel Allele2Label = new JLabel(srecAlleleType);
		final String sAllele1Type = new String("Allele 1");
		final String sAllele2Type = new String("Allele 2");

		final JPanel rightNamePanel = new JPanel();
		final JPanel leftNamePanel = new JPanel();
		final JLabel firstcodomAllelLabel = new JLabel();
		final JLabel thirdcodomAllelLabel = new JLabel();
		final JLabel secondcodomAllelLabel = new JLabel();
		final JTextField domPhen = new JTextField(15);
		final JTextField recPhen = new JTextField(15);
		final JTextField firstcodomAllele = new JTextField(15);
		final JTextField secondcodomAllele = new JTextField(15);
		final JTextField thirdcodomAllele = new JTextField(15);
		final JLabel genotype1Label = new JLabel();
		final JLabel genotype2Label = new JLabel();
		final JLabel genotype3Label = new JLabel();
		final JPanel theBottomPanel = new JPanel();
		final JPanel domrecProbPanel = new JPanel();	
		final JLabel mgenotype1Label = new JLabel();
		final JLabel mgenotype2Label = new JLabel();
		final JPanel maleGenotypeProbPanel = new JPanel();
		final JPanel phenotypePanel = new JPanel();


		final JRadioButton autosomalButton = new JRadioButton("Autosomal", true);
		autosomalButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				dom_recXlinkedButton.doClick();
                dom_recXlinkedButton.setEnabled(false);
				co_domXlinkedButton.setEnabled(false);
				dom_recAutoButton.setEnabled(true);
				co_domAutoButton.setEnabled(true);				
            }
        });


		final JRadioButton xlinkedButton = new JRadioButton("X-linked");
		xlinkedButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				dom_recAutoButton.doClick();
                dom_recXlinkedButton.setEnabled(true);
				co_domXlinkedButton.setEnabled(true);
				dom_recAutoButton.setEnabled(false);
				co_domAutoButton.setEnabled(false);				
            }
        });

		ButtonGroup locusTypeGroup = new ButtonGroup();
		locusTypeGroup.add(autosomalButton);
		locusTypeGroup.add(xlinkedButton);

		JPanel fillerPanel1 = new JPanel();
		fillerPanel1.setPreferredSize(new Dimension(20, 20));
		JPanel fillerPanel2 = new JPanel();
		fillerPanel2.setPreferredSize(new Dimension(20, 20));

		
		ActionListener dom_recListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				Allele1Label.setText(sdomAlleleType);
				Allele2Label.setText(srecAlleleType);
				updateDialogDisplay();
            }
        };
		ActionListener co_domListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				Allele1Label.setText(sAllele1Type);
				Allele2Label.setText(sAllele2Type);
				updateDialogDisplay();
            }
        };

		dom_recAutoButton.addActionListener(dom_recListener);
		co_domAutoButton.addActionListener(co_domListener);
		dom_recXlinkedButton.addActionListener(dom_recListener);
		co_domXlinkedButton.addActionListener(co_domListener);

		ButtonGroup autoAlleleGroup = new ButtonGroup();
		autoAlleleGroup.add(dom_recAutoButton);
		autoAlleleGroup.add(co_domAutoButton);
		JPanel topAllelePanel = new JPanel();
		topAllelePanel.setLayout(new BorderLayout());
		JPanel topRightPanel = new JPanel();
		topRightPanel.setLayout(new BorderLayout());
		topRightPanel.add(dom_recAutoButton, BorderLayout.NORTH);
		topRightPanel.add(co_domAutoButton, BorderLayout.CENTER);
		topAllelePanel.add(autosomalButton, BorderLayout.NORTH);
		topAllelePanel.add(fillerPanel1, BorderLayout.WEST);
		topAllelePanel.add(topRightPanel, BorderLayout.CENTER);

		
		ButtonGroup xlinkedAlleleGroup = new ButtonGroup();
		xlinkedAlleleGroup.add(dom_recXlinkedButton);
		xlinkedAlleleGroup.add(co_domXlinkedButton);
		JPanel bottomAllelePanel = new JPanel();
		bottomAllelePanel.setLayout(new BorderLayout());
		JPanel bottomRightPanel = new JPanel();
		bottomRightPanel.setLayout(new BorderLayout());
		bottomRightPanel.add(dom_recXlinkedButton, BorderLayout.NORTH);
		bottomRightPanel.add(co_domXlinkedButton, BorderLayout.CENTER);
		bottomAllelePanel.add(xlinkedButton, BorderLayout.NORTH);
		bottomAllelePanel.add(fillerPanel2, BorderLayout.WEST);
		bottomAllelePanel.add(bottomRightPanel, BorderLayout.CENTER);

		allelePanel.setLayout(new BorderLayout());
		allelePanel.add(topAllelePanel, BorderLayout.NORTH);
		allelePanel.add(bottomAllelePanel, BorderLayout.SOUTH);	

		
		
		final JTextField domInputTextField = new JTextField(3);
		JPanel p1 = new JPanel();
		p1.setPreferredSize(new Dimension(10,10));			
		final JTextField recInputTextField = new JTextField(3);
		JPanel p2 = new JPanel();
		p2.setPreferredSize(new Dimension(10,10));
		JPanel p3 = new JPanel();
		p3.setPreferredSize(new Dimension(10,10));
		JPanel allelePairPanel = new JPanel();
		allelePairPanel.add(Allele1Label);
		allelePairPanel.add(p1);
		allelePairPanel.add(domInputTextField);
		allelePairPanel.add(p3);
		allelePairPanel.add(Allele2Label);
		allelePairPanel.add(p2);
		allelePairPanel.add(recInputTextField);



		// create the button for setting names
		final JButton setAlleleButton = new JButton("Set Alleles");
		setAlleleButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				String sError;
				// make sure that alleles have been entered
				String Allele1 = domInputTextField.getText().trim();
				String Allele2 = recInputTextField.getText().trim();
				if (Allele1.equals("") || Allele2.equals("")) {
					errorMessage("Please enter alleles");
					return;
				}
				if (Allele1.equals(Allele2)) {
					errorMessage("Allele values must be different");
					return;
				}
				// we are all set
				autosomalButton.setEnabled(false);
				xlinkedButton.setEnabled(false);
				dom_recAutoButton.setEnabled(false);
				co_domAutoButton.setEnabled(false);
				dom_recXlinkedButton.setEnabled(false);
				co_domXlinkedButton.setEnabled(false);
				setAlleleButton.setEnabled(false);
				domInputTextField.setEditable(false);
				recInputTextField.setEditable(false);
				phenotypePanel.remove(noPhenPanel);
				if ((dom_recAutoButton.isSelected() && autosomalButton.isSelected())|| 
					(dom_recXlinkedButton.isSelected() && xlinkedButton.isSelected()))
					phenotypePanel.add(domrecPhenPanel, BorderLayout.NORTH);
				else {
					firstcodomAllelLabel.setText(Allele1 + "-" + Allele1);
					thirdcodomAllelLabel.setText(Allele2 + "-" + Allele2);
					secondcodomAllelLabel.setText(Allele1 + "-" + Allele2);
					phenotypePanel.add(codomPhenPanel, BorderLayout.NORTH);
				}

				// add the prob Panels
				genotype1Label.setText(Allele1 + "-" + Allele1);
				genotype2Label.setText(Allele1 + "-" + Allele2);
				genotype3Label.setText(Allele2 + "-" + Allele2);
				theBottomPanel.add(domrecProbPanel, BorderLayout.NORTH);
				if (xlinkedButton.isSelected()) {
					mgenotype1Label.setText(Allele1);
					mgenotype2Label.setText(Allele2);
					theBottomPanel.add(maleGenotypeProbPanel, BorderLayout.CENTER);
				}

				updateDialogDisplay();
				bAllelesSet = true;
			}
        });
		final JButton resetAlleleButton = new JButton("Reset Alleles");
		resetAlleleButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				if (!bAllelesSet)
					return;

				autosomalButton.setEnabled(true);
				xlinkedButton.setEnabled(true);
				dom_recAutoButton.setEnabled(true);
				co_domAutoButton.setEnabled(true);
				setAlleleButton.setEnabled(true);
				domInputTextField.setEditable(true);
				recInputTextField.setEditable(true);

				if (autosomalButton.isSelected())
					autosomalButton.doClick();
				else
					xlinkedButton.doClick();
				if ((dom_recAutoButton.isSelected() && autosomalButton.isSelected())|| 
					(dom_recXlinkedButton.isSelected() && xlinkedButton.isSelected())) {
					phenotypePanel.remove(domrecPhenPanel);
				}
				else {
					phenotypePanel.remove(codomPhenPanel);
				}
				phenotypePanel.add(noPhenPanel, BorderLayout.NORTH);

				// now remove prob stuff
				theBottomPanel.remove(domrecProbPanel);
				if (xlinkedButton.isSelected())
					theBottomPanel.remove(maleGenotypeProbPanel);

				updateDialogDisplay();

				bAllelesSet = false;
			}
		});

		
		JPanel alleleButtonPanel = new JPanel();
		alleleButtonPanel.add(setAlleleButton, BorderLayout.WEST);
		alleleButtonPanel.add(resetAlleleButton, BorderLayout.EAST);
		JPanel alleleNamePanel = new JPanel();
		alleleNamePanel.setLayout(new BorderLayout());
		alleleNamePanel.add(allelePairPanel, BorderLayout.NORTH);
		alleleNamePanel.add(alleleButtonPanel, BorderLayout.SOUTH);


		noPhenPanel.setLayout(new BorderLayout());
		noPhenPanel.add(new JLabel("Alleles must be set before phenotypes can be chosen.", JLabel.CENTER), BorderLayout.CENTER);


		// dom-rec Phenoptye panel with pheno fields
		JLabel domPhenTitle = new JLabel("Dominant and recessive phenotypes", JLabel.CENTER);
		JPanel domrecPhenCenterPanel = new JPanel();
		JPanel domPhenPanel = new JPanel();
		domPhenPanel.add(new JLabel("Dominant Phenotype"));
		domPhenPanel.add(domPhen);
		JPanel recPhenPanel = new JPanel();
		recPhenPanel.add(new JLabel("Recessive Phenotype"));
		recPhenPanel.add(recPhen);
		domrecPhenCenterPanel.setLayout(new BorderLayout());
		domrecPhenCenterPanel.add(domPhenPanel, BorderLayout.NORTH);
		domrecPhenCenterPanel.add(recPhenPanel, BorderLayout.SOUTH);
		domrecPhenPanel.setLayout(new BorderLayout());
		domrecPhenPanel.add(domPhenTitle, BorderLayout.NORTH);
		domrecPhenPanel.add(domrecPhenCenterPanel, BorderLayout.CENTER);

		JLabel codomPhenTitle = new JLabel("Codominant phenotypes", JLabel.CENTER);
		JPanel codomCenterPhenPanel = new JPanel();
		JPanel firstcodomPhenPanel = new JPanel();
		firstcodomPhenPanel.add(firstcodomAllelLabel);
		firstcodomPhenPanel.add(firstcodomAllele);
		JPanel secondcodomPhenPanel = new JPanel();
		secondcodomPhenPanel.add(secondcodomAllelLabel);
		secondcodomPhenPanel.add(secondcodomAllele);
		JPanel thirdcodomPhenPanel = new JPanel();
		thirdcodomPhenPanel.add(thirdcodomAllelLabel);
		thirdcodomPhenPanel.add(thirdcodomAllele);
		codomCenterPhenPanel.setLayout(new BorderLayout());
		codomCenterPhenPanel.add(firstcodomPhenPanel, BorderLayout.NORTH);
		codomCenterPhenPanel.add(secondcodomPhenPanel, BorderLayout.CENTER);
		codomCenterPhenPanel.add(thirdcodomPhenPanel, BorderLayout.SOUTH);
		codomPhenPanel.setLayout(new BorderLayout());
		codomPhenPanel.add(codomPhenTitle, BorderLayout.NORTH);
		codomPhenPanel.add(codomCenterPhenPanel, BorderLayout.CENTER);

		// the probs Panels
		domrecProbPanel.setLayout(new BorderLayout());
		domrecProbPanel.add(new JLabel("Genotype Probabilities", JLabel.CENTER), BorderLayout.NORTH);
		// create the three mini-panels for each genotype
		JPanel geno1probPanel = new JPanel();
		geno1probPanel.setLayout(new BorderLayout());
		final JTextField geno1Prob = new JTextField(5);
		geno1probPanel.add(genotype1Label, BorderLayout.WEST);
		geno1probPanel.add(geno1Prob, BorderLayout.EAST);
		JPanel geno2probPanel = new JPanel();
		geno2probPanel.setLayout(new BorderLayout());
		final JTextField geno2Prob = new JTextField(5);
		geno2probPanel.add(genotype2Label, BorderLayout.WEST);
		geno2probPanel.add(geno2Prob, BorderLayout.EAST);
		JPanel geno3probPanel = new JPanel();
		geno3probPanel.setLayout(new BorderLayout());
		final JTextField geno3Prob = new JTextField(5);
		geno3probPanel.add(genotype3Label, BorderLayout.WEST);
		geno3probPanel.add(geno3Prob, BorderLayout.EAST);
		// add mini-panels to domrecprobpanel
		JPanel femGenotypesPanel = new JPanel();
		femGenotypesPanel.add(geno1probPanel);
		femGenotypesPanel.add(geno2probPanel);
		femGenotypesPanel.add(geno3probPanel);
		domrecProbPanel.add(femGenotypesPanel, BorderLayout.CENTER);
		// create the male genotype porb panel for xlinked diseases
		JPanel mgeno1probPanel = new JPanel();
		mgeno1probPanel.setLayout(new BorderLayout());
		final JTextField mgeno1Prob = new JTextField(5);
		mgeno1probPanel.add(mgenotype1Label, BorderLayout.WEST);
		mgeno1probPanel.add(mgeno1Prob, BorderLayout.EAST);
		JPanel mgeno2probPanel = new JPanel();
		mgeno2probPanel.setLayout(new BorderLayout());
		final JTextField mgeno2Prob = new JTextField(5);
		mgeno2probPanel.add(mgenotype2Label, BorderLayout.WEST);
		mgeno2probPanel.add(mgeno2Prob, BorderLayout.EAST);
		maleGenotypeProbPanel.add(mgeno1probPanel);
		maleGenotypeProbPanel.add(mgeno2probPanel);

		
		JButton OKButton = new JButton("OK");
		OKButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!bAllelesSet) {
					errorMessage("Alleles must be set.");
					return;
				}
				// verify that phenotypes have been entered
				String sdomPhen = null;
				String srecPhen = null;
				String sFirstPhen = null;
				String sSecondPhen = null;
				String sThirdPhen = null;

				if ((dom_recAutoButton.isSelected() && autosomalButton.isSelected())|| 
					(dom_recXlinkedButton.isSelected() && xlinkedButton.isSelected())) {
					sdomPhen = domPhen.getText().trim();
					srecPhen = recPhen.getText().trim();
					if (sdomPhen.equals("") || srecPhen.equals("")) {
						errorMessage("Phenotypes must be chosen");
						return;
					}
					if (sdomPhen.equals(srecPhen)) {
						errorMessage("Phenotypes must be different");
						return;
					}
				}
				else {
					sFirstPhen = firstcodomAllele.getText().trim();
					sSecondPhen = secondcodomAllele.getText().trim();
					sThirdPhen = thirdcodomAllele.getText().trim();
					if (sFirstPhen.equals("") || sSecondPhen.equals("") || sThirdPhen.equals("")) {
						errorMessage("Phenotyes must be chosen");
						return;
					}
					if (sFirstPhen.equals(sThirdPhen) || sFirstPhen.equals(sSecondPhen) || sSecondPhen.equals(sThirdPhen)) {
						errorMessage("Phenotypes must be different");
						return;
					}
					
				}
				// we are all set, create the disease
				String[] sAlleles = new String[2];
				sAlleles[0] = domInputTextField.getText().trim();
				sAlleles[1] = recInputTextField.getText().trim();
				String[] sPhenotypes = new String[3];
				if (sdomPhen != null) {  // this is a dom-rec disease
					sPhenotypes[0] = sPhenotypes[1] = sdomPhen;
					sPhenotypes[2] = srecPhen;
				}
				else {
					sPhenotypes[0] = sFirstPhen;
					sPhenotypes[1] = sSecondPhen;
					sPhenotypes[2] = sThirdPhen;
				}

				double[] fProbs = new double[3];
				double[] mProbs = new double[2];
				try { 
					fProbs[0] = Double.valueOf(geno1Prob.getText()).doubleValue();
					fProbs[1] = Double.valueOf(geno2Prob.getText()).doubleValue();
					fProbs[2] = Double.valueOf(geno3Prob.getText()).doubleValue();
					
					if (xlinkedButton.isSelected()) {
						mProbs[0] = Double.valueOf(mgeno1Prob.getText()).doubleValue();
						mProbs[1] = Double.valueOf(mgeno2Prob.getText()).doubleValue();
					}
				}
				catch(NumberFormatException nfe) {
					errorMessage("Valid probabilities must be entered for the genotypes.");
					return;
				}

				if (autosomalButton.isSelected()) {
					// create a autosomal disease
					try {
						AutosomalDisease AD = new AutosomalDisease("AD", sAlleles, sPhenotypes, fProbs);
						setDisease(AD);
						endDialog();
						return;
					}
					catch(InvalidDiseaseException dfe) {
						// would only occur if probs don't sum to 1.0
						errorMessage("Genotype probabilities must sum to 1.");
						return;
					}
				}
				else {
					// create an xlinked disease
					try {
						XlinkedDisease XD = new XlinkedDisease("XD", sAlleles, sPhenotypes, fProbs, mProbs);
						setDisease(XD);
						endDialog();
						return;
					}
					catch(InvalidDiseaseException dfe) {
						// would only occur if probs don't sum to 1.0
						errorMessage("Genotype probabilities must sum to 1.");
						return;
					}
				}				
			}
		});

		JButton CancelButton = new JButton("Cancel");
		CancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				endDialog();
			}
		});

		// panel to hold the buttons
		JPanel bottomButtonsPanel = new JPanel();
		bottomButtonsPanel.add(OKButton, BorderLayout.WEST);
		bottomButtonsPanel.add(CancelButton, BorderLayout.EAST);

		// create a bottom panel with probs and buttons
		
		theBottomPanel.setLayout(new BorderLayout());
		// don't stick in the probs Panels - done by setAllele button
		theBottomPanel.add(bottomButtonsPanel, BorderLayout.SOUTH);

		JPanel simpleDiseasePanel = new JPanel();
		simpleDiseasePanel.setLayout(new BorderLayout());
		JPanel sDPCenterPanel = new JPanel();
		sDPCenterPanel.setLayout(new BorderLayout());
		sDPCenterPanel.add(allelePanel, BorderLayout.NORTH);
		sDPCenterPanel.add(alleleNamePanel, BorderLayout.SOUTH);
		sDPCenterPanel.setBorder(LineBorder.createBlackLineBorder());
		simpleDiseasePanel.add(sDPCenterPanel, BorderLayout.NORTH);
		phenotypePanel.setBorder(LineBorder.createBlackLineBorder());
		phenotypePanel.setLayout(new BorderLayout());
		phenotypePanel.add(noPhenPanel, BorderLayout.NORTH);
		phenotypePanel.add(theBottomPanel, BorderLayout.SOUTH);
		simpleDiseasePanel.add(phenotypePanel, BorderLayout.SOUTH);

		

		if (d != null) {
			try {
				AutosomalDisease autoD = (AutosomalDisease)d;
				// autosomal is default-
				autosomalButton.doClick();
				// set the alleles
				String[] sAlleles = autoD.getAlleles();
				domInputTextField.setText(sAlleles[0]);
				recInputTextField.setText(sAlleles[1]);
				// set the phenotypes and the codom/domrec
				String[] sPhenotypes = autoD.getPhenotypes();
				if (sPhenotypes.length == 3) {// this is co-dom
					co_domAutoButton.doClick();
					firstcodomAllele.setText(sPhenotypes[0]);
					secondcodomAllele.setText(sPhenotypes[1]);
					thirdcodomAllele.setText(sPhenotypes[2]);
				}
				else {
					domPhen.setText(sPhenotypes[0]);
					recPhen.setText(sPhenotypes[1]);
				}

				setAlleleButton.doClick();
				// set the probs
				double[] dGenoProbs = autoD.getGenoProbs();
				geno1Prob.setText(Double.toString(dGenoProbs[0]));
				geno2Prob.setText(Double.toString(dGenoProbs[1]));
				geno3Prob.setText(Double.toString(dGenoProbs[2]));
			}
			catch(ClassCastException e) {
				// do nothing
			}

			try {
				XlinkedDisease xD = (XlinkedDisease)d;
				xlinkedButton.doClick();
				// set the alleles
				String[] sAlleles = xD.getAlleles();
				domInputTextField.setText(sAlleles[0]);
				recInputTextField.setText(sAlleles[1]);
				// set the phenotypes and the codom/domrec
				String[] sPhenotypes = xD.getPhenotypes();
				if (sPhenotypes.length == 3) {// this is co-dom
					co_domXlinkedButton.doClick();
					firstcodomAllele.setText(sPhenotypes[0]);
					secondcodomAllele.setText(sPhenotypes[1]);
					thirdcodomAllele.setText(sPhenotypes[2]);
				}
				else {
					domPhen.setText(sPhenotypes[0]);
					recPhen.setText(sPhenotypes[1]);
				}

				setAlleleButton.doClick();

				// set the probs
				double[] dmGenoProbs = xD.getMaleGenoProbs();
				double[] dfGenoProbs = xD.getFemaleGenoProbs();
				geno1Prob.setText(Double.toString(dfGenoProbs[0]));
				geno2Prob.setText(Double.toString(dfGenoProbs[1]));
				geno3Prob.setText(Double.toString(dfGenoProbs[2]));
				mgeno1Prob.setText(Double.toString(dmGenoProbs[0]));
				mgeno2Prob.setText(Double.toString(dmGenoProbs[1]));
			}
			catch(ClassCastException e) {
				// do nothing
			}
		}
		else 
			autosomalButton.doClick();

		return simpleDiseasePanel; 
	}

	private void endDialog() {
		this.setVisible(false);
	}

	private void setDisease(Disease d) {
		newDisease = d;
	}

	private void updateDialogDisplay() {
		this.validate();
		this.pack();
	}

	private void errorMessage(String eMessage) {
		JOptionPane.showMessageDialog(this, eMessage,
					"Disease Edit Error", JOptionPane.INFORMATION_MESSAGE);
	}
}
