//package BayesNetwork;

//import GenExceptions.*;

import java.util.*;
import java.awt.*;
import InferenceGraphs.*;
import BayesianNetworks.*;
import QuasiBayesianInferences.*;

public class GNUBayesNetNode extends BayesNetworkNode{

	InferenceGraphNode ign;
	int IDNumber;
	GNUJavaBayesNet GNUnet;

	public GNUBayesNetNode(int ID, String[] values, double[] prob, 
		                InferenceGraphNode igNode, GNUJavaBayesNet GNUnet) throws ProbabilityException{
		ign=igNode;
		IDNumber = ID;
		this.GNUnet = GNUnet;

		//init the ign

		System.out.println("*");
		if (values.length != prob.length)
			throw new ProbabilityException();

		System.out.println("*");
		// make sure all strings are different ...
		for(int x = 0; x < values.length; x++) 
			for(int y = x+1; y < values.length; y++) {
			if (values[x].equals(values[y]))
				throw new ProbabilityException();
		}
		GNUnet.getIG().change_values(ign, values);
		System.out.println("*");

		// now make probs sum to 1;  since these are floats, I have to make sure sum is close to zero
		double dSum = 0;
		for(int x = 0; x < prob.length; x++) {
			dSum += prob[x];
			System.out.println(prob[x]);
		}

		System.out.println("sum is " + dSum);

		// the sum has to be within 0.1 of 1
		if (java.lang.Math.abs(dSum - 1.0) > .1)
			throw new ProbabilityException();
		System.out.println("*");

		// set the probs
		String[][] variable_value_pairs = new String[1][2];
        variable_value_pairs[0][0] = ign.get_name();
		for (int x=0; x < prob.length; x++) {
			variable_value_pairs[0][1] = values[x];
			ign.set_function_value(variable_value_pairs, prob[x], 0);
		}
	}

	public int getUniqueID() {
		return IDNumber;
	}

	public String[] getValues() {
		if (ign != null)
			return (String[])ign.get_values().clone();
		else
			return null;
	}

	public double[] getProb(BayesNetworkNode[] bnnparentNodes, String[] parentValues) 
		throws ProbabilityException, InvalidValueException {
		if (ign == null)
			return null;

		Vector vParents = ign.get_parents();
		Vector vParentsInInput = new Vector(); // I will use this to store input parents and look for dups
		int numParents = vParents.size();

		if (numParents == 0) {
			if ((bnnparentNodes != null) || (parentValues != null))
				throw new ProbabilityException();
		}
		else {
			if ((bnnparentNodes == null) || (parentValues == null))
				throw new ProbabilityException();

			if ((bnnparentNodes.length != parentValues.length) || (bnnparentNodes.length != numParents))
				throw new ProbabilityException();
		}

		BayesNet bn= GNUnet.getIG().get_bayes_net();
		String[][] variable_value_pairs = new String[numParents + 1][2];
		
        for (int k=0; k<numParents; k++) {
			GNUBayesNetNode gnuBayesNode = (GNUBayesNetNode)bnnparentNodes[k];
			if ((gnuBayesNode.getIGN() == null) || (!vParents.contains(gnuBayesNode.getIGN())) ||
				(vParentsInInput.contains(gnuBayesNode)))
				throw new ProbabilityException();

			vParentsInInput.addElement(gnuBayesNode);
			variable_value_pairs[k][0] = gnuBayesNode.getIGN().get_name();
            variable_value_pairs[k][1] = parentValues[k];
			// need to make sure this is a valid value for this parent
			int index = bn.index_of_variable(variable_value_pairs[k][0]);
			if (bn.get_probability_variable(index).index_of_value(parentValues[k]) == BayesNet.INVALID_INDEX)
				throw new InvalidValueException();
		}

		variable_value_pairs[numParents][0] = ign.get_name();

		double[] returnProbs = new double[ign.get_number_values()];
		String[] ignValues = ign.get_values();
		for(int i=0; i< ignValues.length; i++) {
			variable_value_pairs[numParents][1] = ignValues[i];
			returnProbs[i] = ign.get_function_value(variable_value_pairs, 0);
		}

		return returnProbs;
	}

	public void changeProbFunction(BayesNetworkNode[] bnnparentNodes, String[] parentValues, 
		double[] probsForValues) throws ProbabilityException, InvalidValueException{
		if (ign == null)
			return;
		
		Vector vParents = ign.get_parents();
		Vector vParentsInInput = new Vector(); // I will use this to store input parents and look for dups
		int numParents = vParents.size();

		if (numParents == 0) {
			if ((bnnparentNodes != null) || (parentValues != null))
				throw new ProbabilityException();
		}
		else {
			if ((bnnparentNodes == null) || (parentValues == null))
				throw new ProbabilityException();

			if ((bnnparentNodes.length != parentValues.length) || (bnnparentNodes.length != numParents))
				throw new ProbabilityException();
		}

		if (probsForValues.length != ign.get_number_values())
			throw new ProbabilityException();

		// now make probs sum to 1;  since these are floats, I have to make sure sum is close to zero
		double dSum = 0;
		for(int x = 0; x < probsForValues.length; x++)
			dSum += probsForValues[x];

		// the sum has to be within 0.1 of 1
		if (java.lang.Math.abs(dSum - 1.0) > .1)
			throw new ProbabilityException();

		BayesNet bn= GNUnet.getIG().get_bayes_net();
		String[][] variable_value_pairs = new String[numParents + 1][2];
		
        for (int k=0; k<numParents; k++) {
			GNUBayesNetNode gnuBayesNode = (GNUBayesNetNode)bnnparentNodes[k];
			if ((gnuBayesNode.getIGN() == null) || (!vParents.contains(gnuBayesNode.getIGN())) ||
				(vParentsInInput.contains(gnuBayesNode)))
				throw new ProbabilityException();

			vParentsInInput.addElement(gnuBayesNode);
			variable_value_pairs[k][0] = gnuBayesNode.getIGN().get_name();
            variable_value_pairs[k][1] = parentValues[k];
			// need to make sure this is a valid value for this parent
			int index = bn.index_of_variable(variable_value_pairs[k][0]);
			if (bn.get_probability_variable(index).index_of_value(parentValues[k]) == BayesNet.INVALID_INDEX)
				throw new InvalidValueException();
		}
		variable_value_pairs[numParents][0] = ign.get_name();

		String[] ignValues = ign.get_values();
		for(int i=0; i< probsForValues.length; i++) {
			variable_value_pairs[numParents][1] = ignValues[i];
			ign.set_function_value(variable_value_pairs, probsForValues[i], 0);
		}

	}

	public void makeObservation(String valueObserved) throws InvalidValueException, ProbabilityException {
		//make sure it is a valid variable
		// ign.set_observation_value will do nothing for an invalid value
		if (ign ==null)
			return;

		String[] values = ign.get_values();
		int x;
		for(x=0; x< values.length; x++) {
			if (values[x].equals(valueObserved))
				break;
		}
		if (x == values.length)
			throw new InvalidValueException();
		// temporarily set variable to determine if the variable is valid
	       
		ign.set_observation_value(valueObserved);
		int iIndex = ign.get_observed_value();
		ign.clear_observation();
		// System.out.println("index: " +iIndex);
		QBInference tempqbi = new QBInference(GNUnet.getIG().get_bayes_net(), false);
		// GNUnet.getIG().get_bayes_net().print(System.out);
		// tempqbi.print();
		// System.out.println(ign.get_name());
		tempqbi.inference(ign.get_name());
		// tempqbi.print();
	       double[] probs = tempqbi.get_result().get_values();
	       if(probs[iIndex] < Double.MIN_VALUE){
		 throw new ProbabilityException();
	       }
	       // We've made sure the value is valid, now set it.
	       ign.set_observation_value(valueObserved);
	       // System.out.println("probs index value: "+probs[iIndex]);
	}

	public void clearObservation() {
		if (ign == null)
			return;
		ign.clear_observation();
	}

	public String getObservation() {
		if (ign ==null)
			return null;
		if (!ign.is_observed())
			return null;
		int iIndex = ign.get_observed_value();

		BayesNet bn= GNUnet.getIG().get_bayes_net();
		ProbabilityVariable pv =bn.get_probability_variable(bn.index_of_variable(ign.get_name()));
		return pv.get_value(iIndex);
	}

	public Enumeration getParents() {
		if (ign == null)
			return (new Vector()).elements();  // empty enum

		Vector vIGNParents = ign.get_parents();
		Vector vGNUBayesNodes = new Vector();
		for(int x = 0; x < vIGNParents.size(); x++)
			vGNUBayesNodes.addElement(GNUnet.getGNUBayesNetNode((InferenceGraphNode)vIGNParents.elementAt(x)));

		return vGNUBayesNodes.elements();
	}

	public Enumeration getChildren() {
		if (ign == null)
			return (new Vector()).elements();  // empty enum

		Vector vIGNChildren = ign.get_children();
		Vector vGNUBayesNodes = new Vector();
		for(int x = 0; x < vIGNChildren.size(); x++)
			vGNUBayesNodes.addElement(GNUnet.getGNUBayesNetNode((InferenceGraphNode)vIGNChildren.elementAt(x)));

		return vGNUBayesNodes.elements();
	}

	public InferenceGraphNode getIGN() {
		return ign;
	}

	// this is called GNUBayesNet to inform the node that it has been removed
	public void remove() {
		ign = null;
		GNUnet = null;
	}
}
