//package BayesNetwork;

//import GenExceptions.*;

import java.util.*;
import java.awt.*;
import InferenceGraphs.*;
import BayesianNetworks.*;
import QuasiBayesianInferences.*;

public class GNUJavaBayesNet extends BayesNetwork {

	Vector vBayesNodes;  // this is all the GNUBayesNetNodes in this net
						// I need to keep track to implement getChildren and getParents for nodes
	InferenceGraph ig;
	QBInference qbi;
	int nextID;
	
	// constructor
	public GNUJavaBayesNet() {
		ig = new InferenceGraph();
		qbi = null;
		vBayesNodes = new Vector();
		nextID = Integer.MIN_VALUE;
	}

	public BayesNetworkNode createNode(String[] values, double[] prob) throws ProbabilityException {
		// I will add this node at 50,50 so I can get it back.
		// the only way to get a pointer to this node is to get all nodes and see which one
		// is at 50, 50.  Only have to do this once, since same IGN is used the whole time
		ig.create_node(50, 50);
		// not too sure what this does, but it should be done when adding a node for IG
		ig.reset_marginal();
		// old inference is now invalid
		qbi = null;

		// get the ig node that was added
		Enumeration enumNodes = ig.elements();
		InferenceGraphNode ignNewNode = null;
		while(enumNodes.hasMoreElements()) {
			InferenceGraphNode ign = (InferenceGraphNode)enumNodes.nextElement();
			if ((ign.get_pos_x()==50) && (ign.get_pos_y()==50))
				ignNewNode = ign;
		}
		
		//set the position of the node back so that next time we create a node we don't
		// pick this one again.
		ig.set_pos(ignNewNode, new Point(0,0));

		
		// now set the properties of the node
		GNUBayesNetNode GNUnode;
		try {
			GNUnode = new GNUBayesNetNode(nextID++, values, prob, ignNewNode, this);
		}
		catch(ProbabilityException e) {
			// the paramters for this node are wrong, so lets delete it
			ig.delete_node(ignNewNode); // Delete node.
			throw e;
		}

		// add this our list of nodes
		vBayesNodes.addElement(GNUnode);
		return GNUnode;
	}

	public Enumeration getNodes() {
		return vBayesNodes.elements();
	}

	public void removeNode(BayesNetworkNode bnnNode) {
		GNUBayesNetNode node = (GNUBayesNetNode)bnnNode;

		InferenceGraphNode ign = node.getIGN();
		if (ign == null)
			return;
		// what about deleteing old arcs which are now invalid?
		// -- this is done by delete_node
		ig.delete_node(node.getIGN()); // Delete node.
        ig.reset_marginal();
		// old inference is now invalid
		qbi = null;


		// removes this one from our list of nodes
		vBayesNodes.removeElement(node);
		node.remove();
	}

	// create an arc from source to dest, throws an exception is a loop is created
	public void createArc(BayesNetworkNode bnnSource, BayesNetworkNode bnnDest) throws InvalidArcException {
		GNUBayesNetNode source = (GNUBayesNetNode)bnnSource;
		GNUBayesNetNode dest = (GNUBayesNetNode)bnnDest;

		if ((source.getIGN() == null) || (dest.getIGN() == null))
			return;
		// make sure that we are not creating a self loop
		if (source.equals(dest))
			throw new InvalidArcException();
		// make sure we are not creating a loop
		if (ig.hasCycle(source.getIGN(), dest.getIGN()))
			throw new InvalidArcException();

		// not too sure what this reset does ...
		boolean flag_created = ig.create_arc(source.getIGN(), dest.getIGN());
        if (flag_created == true)  {
            ig.reset_marginal();
			// old inference is now invalid
			qbi = null;
		}
	}


	//this returns the BayesNode for the InferenceGraphNode - used for the getParents() and getChildren()
	// in the GNUBayesNetNode object - returns null if the ign is not in this Bayes net
	public GNUBayesNetNode getGNUBayesNetNode(InferenceGraphNode ign) {
		GNUBayesNetNode theNode = null;
		for(int x=0; x < vBayesNodes.size(); x++) {
			theNode = (GNUBayesNetNode)vBayesNodes.elementAt(x);
			if (ign.equals(theNode.getIGN()))
				return theNode;
		}

		return null;
	}

   public void removeArc(BayesNetworkNode bnnSource, BayesNetworkNode bnnDest) {
	   GNUBayesNetNode source = (GNUBayesNetNode)bnnSource;
	   GNUBayesNetNode dest = (GNUBayesNetNode)bnnDest;

	   if ((source.getIGN() == null) || (dest.getIGN() == null))
			return;

	   ig.delete_arc(source.getIGN(), dest.getIGN());
       ig.reset_marginal();
	   // old inference is now invalid
	   qbi = null;
   }

	// get an array of the probabilities for the variables of this node
   public double[] inferenceOnNode(BayesNetworkNode bnnNode) throws ProbabilityException {
	   GNUBayesNetNode node = (GNUBayesNetNode) bnnNode;
	   

	   InferenceGraphNode ign = node.getIGN();
	   if (ign == null)
			return null;

	   System.out.println(".");
	   if (qbi == null) {
		   System.out.println("qbi null");
		   // need to check for inconsistancies - I will do this by undoing all observations, and
		   // then putting them back, making sure all of them are probable
		   Vector vNodesWithObs = new Vector();
		   Vector vOldObservations = new Vector();
		   Vector vOldObsIndex = new Vector();  // this is for performance reasons
		   for(int x = 0; x < vBayesNodes.size(); x++) {
			   GNUBayesNetNode bnn = (GNUBayesNetNode)vBayesNodes.elementAt(x);
			   InferenceGraphNode tempign = bnn.getIGN();
			   if (tempign.is_observed()) {
				   int iIndex = tempign.get_observed_value();
				   vNodesWithObs.addElement(bnn);
				   BayesNet bn= ig.get_bayes_net();
				   ProbabilityVariable pv =bn.get_probability_variable(bn.index_of_variable(tempign.get_name()));
				   vOldObservations.addElement(pv.get_value(iIndex));
				   vOldObsIndex.addElement(new Integer(iIndex));
				   bnn.clearObservation();
			   }
		   }

		   System.out.println(".");
		   // now add each one back and make sure it is probable
		   boolean bErrorFound = false;
		   int x;
		   for(x = 0; (x < vNodesWithObs.size()) && !bErrorFound; x++) {
				QBInference tempqbi = new QBInference( ig.get_bayes_net(), false);
				// Debugging code
				// ig.get_bayes_net().print(System.out);
				// tempqbi.print();
				GNUBayesNetNode bnn = (GNUBayesNetNode)vNodesWithObs.elementAt(x);
				System.out.println("*");
				// System.out.println(bnn.getIGN().get_name());
				tempqbi.inference(bnn.getIGN().get_name());
				// tempqbi.print();
				System.out.println("*");
				ProbabilityFunction pf = tempqbi.get_result();
				System.out.println("*");
				double[] probs = pf.get_values();
				int index = ((Integer)vOldObsIndex.elementAt(x)).intValue();
				// System.out.println("probs index value: "+probs[index]);
				// System.out.println("index: " +index);
		   		if (probs[index] < Double.MIN_VALUE)  // is it zero?
					bErrorFound = true;
				System.out.println("*");
				try {
					bnn.makeObservation((String)vOldObservations.elementAt(x));
				}
				catch(InvalidValueException e) {
					System.err.println("Unexpected value exception in inferenceOnNode in BayesNet");
				}
		   }
		   System.out.println(".");
		   if (bErrorFound) {  // put things back in order
			   for(int y = x; y < vNodesWithObs.size(); y++) {
				   GNUBayesNetNode bnn = (GNUBayesNetNode)vNodesWithObs.elementAt(y);
				   try {
					bnn.makeObservation((String)vOldObservations.elementAt(y));
				   }
				   catch(InvalidValueException e) {
					System.err.println("Unexpected value exception in inferenceOnNode in BayesNet");
				   }
			   }
			   throw new ProbabilityException();
		   }
		   System.out.println(".");
		   qbi = new QBInference( ig.get_bayes_net(), false);
		   }

	   System.out.println("-");
	   qbi.inference(node.getIGN().get_name());
	   ProbabilityFunction pf = qbi.get_result();
	   return (double[])pf.get_values().clone();
   }

   public InferenceGraph getIG() {
	   return ig;
   }

}


