//package PedigreeCanvas;

//import Disease.*;
//import GeneticScreen.*;
//import PedigreeCanvas.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.beans.*; //Property change stuff
import java.net.URL;
import netscape.security.*;

public class GeneticProgramApplet extends JApplet {


	// Local State
	private JFrame frame = null;
	private boolean bIsApplet;
	private boolean bHasPrivilege;
	

	private PedigreeCanvas pedCanvas;
	private File pedigreeFile;
	private FileDialog fd;
	

	JMenu diseaseMenu, drawMenu;
	JMenuItem miOpenPedigree, miSavePedigree, miSaveAsPedigree, miClosePedigree;
	JMenuItem miAddGSD, miRemoveGSD;
	JToolBar drawToolBar;

	// the cursors
	private Cursor pointerCursor, maleCursor, femaleCursor, babyCursor, marriageCursor, childCursor, deleteCursor;
	
	private JTextArea debug;

  public void main(String argv[]) {
    //Changed to try and make the applet work
	this.construct(false); // create it as an application
  }
  
  public void init() {	
		construct(true);		
    }

	public void start() {
		// setup up frame window and also load pedigree class so that 
		// there is no delay when the pedigree canvas is loaded by the user
	
		//PedigreeCanvas pcTemp = new PedigreeCanvas(frame, new NewAutosomalDisease());
		

		frame.setVisible(true);
		//getContentPane().removeAll();
		//getContentPane().add(new JLabel("Finshed Loading Genetic Pedigree Applet."), BorderLayout.CENTER);
	}

	public void stop () {
		System.err.println("********* Applet.stop was executed **********");
		debug.setText("********* Applet.stop was executed **********");
	}

	private void construct(boolean bIsApplet) {
		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		}
		catch(Exception e) {
			System.err.println("LookAndFeel error");
		}

		this.bIsApplet = bIsApplet;
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(new JLabel("Loading Genetic Pedigree Applet..."), BorderLayout.CENTER);
		debug = new JTextArea();
		getContentPane().add(debug, BorderLayout.SOUTH);

		// if this is an applet, we need to ask user for file access rights
		if (bIsApplet) {
			try {
			 PrivilegeManager.enablePrivilege("UniversalFileAccess");
			 bHasPrivilege = true;
			} 
			catch (netscape.security.ForbiddenTargetException e) {
			 bHasPrivilege = false;
			} 
			catch (Exception e) {
			 bHasPrivilege = false;
			}
		}
		else
			bHasPrivilege = true;

		frame = new JFrame("Genetic Program");
		initFrame();
		
		// start the program with no open pedigree
		pedCanvas = null;
		pedigreeFile = null;

		// disable menus since no pedigree is open
		enablePedigreeMenus(false);		

		// create the fileDialog
		if (bHasPrivilege)
			fd= new FileDialog(frame);
		
		// since only applets call start, and start does some necessary init
		if (!bIsApplet)
			start();
	}

	private void initFrame() {
		// setup the frame
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                pedigreeExit();
            }
        });
		frame.setSize(new Dimension(400, 400));


		// create the menu bars and items
		JMenuBar menuBar;
		JMenu menu;
		JMenuItem menuItem;


		//Create the menu bar.
		menuBar = new JMenuBar();
		frame.setJMenuBar(menuBar);

		//Build the pedigree menu.
		menu = new JMenu("Pedigree");
		menuBar.add(menu);
		menuItem = new JMenuItem("New Pedigree");
		menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                newPedigree();
            }
        });
		menu.add(menuItem);
		miOpenPedigree = new JMenuItem("Open Pedigree");
		miOpenPedigree.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                openPedigree();
            }
        });
		menu.add(miOpenPedigree);
		miSavePedigree = new JMenuItem("Save Pedigree");
		miSavePedigree.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                savePedigree();
            }
        });
		menu.add(miSavePedigree);
		miSaveAsPedigree = new JMenuItem("Save As Pedigree");
		miSaveAsPedigree.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                saveAsPedigree();
            }
        });
		menu.add(miSaveAsPedigree);
		miClosePedigree = new JMenuItem("Close Pedigree");
		miClosePedigree.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closePedigree();
            }
        });
		menu.add(miClosePedigree);
		menuItem = new JMenuItem("Exit");
		menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedigreeExit();
            }
        });
		menu.add(menuItem);

		//Build Diseases menu in the menu bar.
		diseaseMenu = new JMenu("Disease");
		menuBar.add(diseaseMenu);
		JMenuItem miCreateDisease = new JMenuItem("Create a Disease");
		miCreateDisease.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                createDisease();
            }
        });
		diseaseMenu.add(miCreateDisease);
		JMenuItem miEditDisease = new JMenuItem("Edit a Disease");
		miEditDisease.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                editDisease();
            }
        });
		diseaseMenu.add(miEditDisease);

		//Build the Genetic Screen menu
		JMenu gsMenu = new JMenu("Genetic Screen");
		menuBar.add(gsMenu);
		JMenuItem miCreateGS = new JMenuItem("Create genetic screen");
		miCreateGS.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				createGSD();
			}
		});
		gsMenu.add(miCreateGS);
		JMenuItem miEditGS = new JMenuItem("Edit genetic screen");
		miEditGS.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				editGSD();
			}
		});
		gsMenu.add(miEditGS);
		miAddGSD = new JMenuItem("Add genetic screen to pedigree");
		miAddGSD.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addGSD();
			}
		});
		gsMenu.add(miAddGSD);
		miRemoveGSD = new JMenuItem("Remove genetic screen to pedigree");
		miRemoveGSD.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				pedCanvas.removeGeneticScreenDescriptor();
			}
		});
		gsMenu.add(miRemoveGSD);



		//Build Draw menu in the menu bar.
		//a group of radio button menu items
		drawMenu = new JMenu("Draw");
		menuBar.add(drawMenu);
		
		//create the toolbar and menu
		createDrawMenuAndToolBar();
	}


	private void createDrawMenuAndToolBar() {
		drawToolBar = new JToolBar();

		// Pointer menu item
		ButtonGroup group = new ButtonGroup();
		final JRadioButtonMenuItem rbPointerMenuItem = new JRadioButtonMenuItem("Pointer");
		rbPointerMenuItem.setSelected(true);
		rbPointerMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Pointer");
            }
        });
		group.add(rbPointerMenuItem);
		drawMenu.add(rbPointerMenuItem);
		// Pointer botton
		JButton button;
		ImageIcon imgIcon;
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/pointer.GIF"));
		else
			imgIcon = new ImageIcon("images/pointer.GIF");
		button= new JButton(imgIcon);
        button.setToolTipText("Pointer");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Pointer");
				rbPointerMenuItem.setSelected(true);
            }
        });
        drawToolBar.add(button);

		// Male menu item
		final JRadioButtonMenuItem rbMaleMenuItem = new JRadioButtonMenuItem("Male");
		rbMaleMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Male");
            }
        });
		group.add(rbMaleMenuItem);
		drawMenu.add(rbMaleMenuItem);
		// Male button
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/male.GIF"));
		else
			imgIcon = new ImageIcon("images/male.GIF");
		button= new JButton(imgIcon);
		button.setToolTipText("Male");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Male");
				rbMaleMenuItem.setSelected(true);
            }
        });
        drawToolBar.add(button);

		// female menu item
		final JRadioButtonMenuItem rbFemaleMenuItem = new JRadioButtonMenuItem("Female");
		rbFemaleMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Female");
            }
        });
		group.add(rbFemaleMenuItem);
		drawMenu.add(rbFemaleMenuItem);
		// Female button
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/female.GIF"));
		else
			imgIcon = new ImageIcon("images/female.GIF");
		button= new JButton(imgIcon);
		button= new JButton(imgIcon);
        button.setToolTipText("Female");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Female");
				rbFemaleMenuItem.setSelected(true);
            }
        });
        drawToolBar.add(button);

		// baby menu item
		final JRadioButtonMenuItem rbBabyMenuItem = new JRadioButtonMenuItem("Baby");
		rbBabyMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Baby");
            }
        });
		group.add(rbBabyMenuItem);
		drawMenu.add(rbBabyMenuItem);
		// baby button
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/baby.GIF"));
		else
			imgIcon = new ImageIcon("images/baby.GIF");
		button= new JButton(imgIcon);
        button.setToolTipText("Baby");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Baby");
				rbBabyMenuItem.setSelected(true);
            }
        });
        drawToolBar.add(button);


		// marriage menu item
		final JRadioButtonMenuItem rbMarriageMenuItem = new JRadioButtonMenuItem("Marriage");
		rbMarriageMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Marriage");
            }
        });
		group.add(rbMarriageMenuItem);
		drawMenu.add(rbMarriageMenuItem);
		// marriage button
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/marriage.GIF"));
		else
			imgIcon = new ImageIcon("images/marriage.GIF");
		button= new JButton(imgIcon);
        button.setToolTipText("Marriage");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Marriage");
				rbMarriageMenuItem.setSelected(true);
            }
        });
        drawToolBar.add(button);


		// child menu item
		final JRadioButtonMenuItem rbChildMenuItem = new JRadioButtonMenuItem("Child");
		rbChildMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Child");
            }
        });
		group.add(rbChildMenuItem);
		drawMenu.add(rbChildMenuItem);
		// child button
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/child.GIF"));
		else
			imgIcon = new ImageIcon("images/child.GIF");
		button= new JButton(imgIcon);
        button.setToolTipText("Child");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Child");
				rbChildMenuItem.setSelected(true);
            }
        });
        drawToolBar.add(button);


		// Delete menu item
		final JRadioButtonMenuItem rbDeleteMenuItem = new JRadioButtonMenuItem("Delete");
		rbDeleteMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Delete");
            }
        });
		group.add(rbDeleteMenuItem);
		drawMenu.add(rbDeleteMenuItem);
		// delete button
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/delete.GIF"));
		else
			imgIcon = new ImageIcon("images/delete.GIF");
		button= new JButton(imgIcon);
        button.setToolTipText("Delete");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Delete");
				rbDeleteMenuItem.setSelected(true);
            }
        });
        drawToolBar.add(button);

		// Tool menu item
		final JRadioButtonMenuItem rbTestMenuItem = new JRadioButtonMenuItem("Test");
		rbTestMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Test");
            }
        });
		group.add(rbTestMenuItem);
		drawMenu.add(rbTestMenuItem);
		// gs button
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/inference.GIF"));
		else
			imgIcon = new ImageIcon("images/inference.GIF");
		button= new JButton(imgIcon);
        button.setToolTipText("Test");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.setMode("Test");
				rbTestMenuItem.setSelected(true);
            }
        });
        drawToolBar.add(button);


		// inferences menu item
		JMenuItem menuItem = new JMenuItem("Update inferences");
		menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.updatePersonInferences();
            }
        });
		drawMenu.add(menuItem);
		// inferences button
		if (bIsApplet) {
			//button= new JButton(new ImageIcon(getURL("images/inference.GIF")));
			imgIcon= new ImageIcon(getURL("images/inference.GIF"));
			Image img = imgIcon.getImage();
			debug.setText(img.toString());
			button= new JButton(imgIcon);
			//imgIcon.setImageObserver(button);
		}
		else
			button= new JButton(new ImageIcon("images/inference.GIF"));
        button.setToolTipText("Update inferences");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pedCanvas.updatePersonInferences();
            }
        });
        drawToolBar.add(button);
	}

	// this function loads customized cursors for using a pedigreeCanvas.
	// because JDK 1.2 is required, this is commented out to allow it to complile
	// to enable custom cursors, add a line in each button action listener to set pedigreecanvas cursor
	// and call loadCursors when loading this applet
	private void loadCursors() {
		Toolkit tk = Toolkit.getDefaultToolkit();
		ImageIcon imgIcon;  // I will use imageIcon since it handles image loading for me

		// pointer
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/pointer.GIF"));
		else
			imgIcon = new ImageIcon("images/pointer.GIF");
		//pointerCursor = tk.createCustomCursor(imgIcon.getImage(), new Point(0,0), "Pointer");

		// Male
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/male.GIF"));
		else
			imgIcon = new ImageIcon("images/male.GIF");
		//maleCursor =	tk.createCustomCursor(imgIcon.getImage(), new Point(10,11), "Male");

		// female
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/female.GIF"));
		else
			imgIcon = new ImageIcon("images/female.GIF");
		//femaleCursor =	tk.createCustomCursor(imgIcon.getImage(), new Point(10,11), "Female");

		// baby
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/baby.GIF"));
		else
			imgIcon = new ImageIcon("images/baby.GIF");
		//babyCursor = tk.createCustomCursor(imgIcon.getImage(), new Point(10,11), "Baby");


		// marriage
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/marriage.GIF"));
		else
			imgIcon = new ImageIcon("images/marriage.GIF");
	//	marriageCursor =tk.createCustomCursoe(imgIcon.getImage(), new Point(10,11), "Marriage");


		// child
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/child.GIF"));
		else
			imgIcon = new ImageIcon("images/child.GIF");
	//	childCursor =	tk.createCustomCursor(imgIcon.getImage(), new Point(10,11), "Child");


		// Delete
		if (bIsApplet)
			imgIcon = new ImageIcon(getURL("images/delete.GIF"));
		else
			imgIcon = new ImageIcon("images/delete.GIF");
	//	deleteCursor= tk.createCustomCursor(imgIcon.getImage(), new Point(10,11), "Delete");
	}


	// Called in response to a user request for a new pedigree, this method
	// will close the open pedigree is there is one (just as if the user had requested
	// to close the open pedigree) and start a fresh pedigree
	private void newPedigree() {
		if (pedCanvas != null) {
			if (!closePedigree())
				return;
		}

		Disease currDisease = getDiseaseFromFile();
		if (currDisease != null) {
			pedCanvas = new PedigreeCanvas(frame, currDisease);
			setupPedCanvas();
		}
	}

	// open pedigree, closing the current one (as if the user hit close first)
	private void openPedigree() {
		if (pedCanvas != null) {
			if (!closePedigree())
				return;
		}
		
		if (!bHasPrivilege) {
			JOptionPane.showMessageDialog(frame, "Permission was denied by user for file access.  Cannot open pedigree",
					"File Open Error", JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		if (bIsApplet)
			PrivilegeManager.enablePrivilege("UniversalFileAccess");

		fd.setTitle("Open Pedigree");
		fd.setMode(FileDialog.LOAD);
		fd.setVisible(true);

		if (fd.getFile() != null) {
			pedigreeFile = new File(fd.getDirectory(), fd.getFile());

			try {
				FileReader fr = new FileReader(pedigreeFile);
				pedCanvas = new PedigreeCanvas(fr, frame);
				setupPedCanvas();
			}
			catch(FileNotFoundException e) {
				// display warning that we can't access file
				JOptionPane.showMessageDialog(frame, "File " + pedigreeFile + " not found." +
					"\nPlease verify that the file exists.",
					"File Open Error", JOptionPane.INFORMATION_MESSAGE);
				pedigreeFile = null;
			}
			catch(IOException e) {
				// display warning that we can't access file
				JOptionPane.showMessageDialog(frame, "File " + pedigreeFile + " could not be read." +
					"\nIt is in use by another application or does not contain the correct format for a pedigree file.",
					"File Open Error", JOptionPane.INFORMATION_MESSAGE);
				pedigreeFile = null;
			}

		}  // user selected a file to open
		frame.repaint();
	}		

	// save the current Pedigree, request a filename if open does not already exist
	// assumes the save is only requested when there is an open pedigree
	// return true if the file is actually saved
	private boolean savePedigree() {
		int returnVal;

		if (!bHasPrivilege) {
			JOptionPane.showMessageDialog(frame, "Permission was denied by user for file access.  Can not save pedigree",
					"File Save Error", JOptionPane.INFORMATION_MESSAGE);
			return false;
		}
		if (bIsApplet)
			PrivilegeManager.enablePrivilege("UniversalFileAccess");

		if (pedigreeFile == null) {
			fd.setTitle("Save Disease");
			fd.setMode(FileDialog.SAVE);
			fd.setVisible(true);

			if (fd.getFile() != null)
				pedigreeFile =  new File(fd.getDirectory(), fd.getFile());
			else {
				frame.repaint();
				return false;
			}

			// don't need to check since FileDialog will check for us
		}
		// use already existing File or the file just got from user
		try {
			FileWriter fr = new FileWriter(pedigreeFile);
			pedCanvas.save(fr);
			frame.repaint();
			return true;
		}
		catch(IOException e) {
				// display warning that we can't access file
				JOptionPane.showMessageDialog(frame, "Error writing to " + pedigreeFile +
					"\nFile save was aborted!",
					"File Save Error", JOptionPane.INFORMATION_MESSAGE);
				pedigreeFile = null;
				frame.repaint();
				return false;
		}
	}


	//Called to save as, assumes that a pedigree is open
	// return true if the pedigree is actually saved
	private boolean saveAsPedigree() {
		int returnVal;

		if (!bHasPrivilege) {
			JOptionPane.showMessageDialog(frame, "Permission was denied by user for file access.  Can not save pedigree",
					"File Open Error", JOptionPane.INFORMATION_MESSAGE);
			return false;
		}

		if (bIsApplet)
			PrivilegeManager.enablePrivilege("UniversalFileAccess");

		fd.setTitle("Save Pedigree As");
		fd.setMode(FileDialog.SAVE);
		fd.setVisible(true);
		
		if (fd.getFile() != null) {
			File newPedigreeFile = new File(fd.getDirectory(), fd.getFile());

			// don't need to check if file exists since fd does it for us
			try {
				FileWriter fr = new FileWriter(newPedigreeFile);
				pedCanvas.save(fr);
				pedigreeFile = newPedigreeFile;
				frame.repaint();
				return true;
			}
			catch(IOException e) {
				// display warning that we can't access file
				JOptionPane.showMessageDialog(frame, "Error writing to " + pedigreeFile +
					"\nFile save was aborted!",
					"File Save Error", JOptionPane.INFORMATION_MESSAGE);
				frame.repaint();
				return false;
			}
		} /// do save as
			
		frame.repaint();
		return false;
	}
	

	// Called to close an open pedigree (we assume one is open) and disable menu items
	// returns true if the pedigree was closed
	private boolean closePedigree() {
		final String saveAndExit = "Save work and exit";
		final String justExit = "Don't save work and exit";
		final String dontExit = "Don't exit";

		try {
			pedCanvas.close();
		}
		catch(NotSavedException e) {
			Object[] options = {saveAndExit, justExit, dontExit};
			final JOptionPane optionPane = new JOptionPane("This pedigree contains unsaved work.",
                                    JOptionPane.QUESTION_MESSAGE,
									JOptionPane.YES_NO_CANCEL_OPTION,
                                    null,
                                    options,
                                    options[0]);

			final JDialog dialog = new JDialog(frame, "Close Pedigree File", true);
			dialog.setContentPane(optionPane);
			dialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
			optionPane.addPropertyChangeListener(
			new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent e) {
					String prop = e.getPropertyName();

					if (dialog.isVisible() 
						&& (e.getSource() == optionPane)
						&& (prop.equals(JOptionPane.VALUE_PROPERTY) ||
						prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
						Object value = optionPane.getValue();
						dialog.setVisible(false);
					}
				}
			});

			dialog.pack();
			dialog.setLocationRelativeTo(frame);
			dialog.setVisible(true);

			String input = (String)optionPane.getValue();

            if (input.equals(saveAndExit)) {
				boolean bSaved = savePedigree();
				if (!bSaved) {
					frame.repaint();
					return false;
				}
            } else if (input.equals(justExit)) {
                       // don't do anything, will be close below
            } else  {
                // don't quit
				frame.repaint();
				return false;
            } 


		}

		// now close it
		frame.getContentPane().removeAll();
		pedCanvas = null;
		pedigreeFile = null;

		// disable all relavent Menu Items from PedigreeMenu and Disease Menu
		enablePedigreeMenus(false);
		frame.repaint();
		return true;
	}

	// called when user pressed exit
	private void pedigreeExit() {
		if (pedCanvas != null) {
			if (!closePedigree())
				return;
		}

		frame.setVisible(false);
		frame.dispose();

		if (!bIsApplet)
			System.exit(0);
	}

	// setup the pedigree canvas
	// requires that pedCanvas is not null
	private void setupPedCanvas() {
		pedCanvas.setMode("Pointer");

		JScrollPane scroller = new JScrollPane(pedCanvas);
        scroller.setPreferredSize(new Dimension(200,200));

		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add(drawToolBar, BorderLayout.NORTH);
		frame.getContentPane().add(scroller, BorderLayout.CENTER);

		// enable menu items for pedigree menu and disease menu
		enablePedigreeMenus(true);
		
		frame.validate();
		frame.pack();			
		frame.repaint();
	}


	//  enable or disable the proper menus when a pedigree is opened or closed
	private void enablePedigreeMenus(boolean openPedigree) {
		if (openPedigree) {
			miSavePedigree.setEnabled(true);
			miSaveAsPedigree.setEnabled(true);
			miClosePedigree.setEnabled(true);
			diseaseMenu.setEnabled(false);
			drawMenu.setEnabled(true);
			miAddGSD.setEnabled(true);
			miRemoveGSD.setEnabled(true);
		}
		else {
			drawMenu.setEnabled(false);
			miSavePedigree.setEnabled(false);
			miSaveAsPedigree.setEnabled(false);
			miClosePedigree.setEnabled(false);
			diseaseMenu.setEnabled(true);
			miAddGSD.setEnabled(false);
			miRemoveGSD.setEnabled(false);
		}
	}

	// called when user asks to create a disease.  If the user enters a  new diease, the
	// user is asked where to save it.
	private void createDisease() {
		//DiseaseDialog dd = new DiseaseDialog(frame, null);
		//dd.setVisible(true);
		NewDiseaseDialog ndd = new NewDiseaseDialog(frame, null);
		ndd.setVisible(true);

		Disease newDisease = ndd.getNewDisease();
		if (newDisease != null) {
			System.out.println(newDisease.toString());
			saveDiseaseToFile(newDisease);
		}
	}

	// called when user asks to create a gsd.  If the user enters a  new gsd, the
	// user is asked where to save it.
	private void createGSD() {
		GeneticScreenDescriptorDialog gsdd = new GeneticScreenDescriptorDialog(frame, null, fd);
		gsdd.setLocationRelativeTo(frame);
		gsdd.setVisible(true);

		if (gsdd.didGeneticScreenDescriptorChange()) {
			GeneticScreenDescriptor newGSD= gsdd.getGeneticScreenDescriptor();
			saveGSDToFile(newGSD);
		}
	}

	// save the current disease to a file. The user is asked for the file
	private void saveDiseaseToFile(Disease newDisease) {
	// requires:  newDisease is not null
	// effects: asks user where to save disease to file

		if (!bHasPrivilege) {
			JOptionPane.showMessageDialog(frame, "Permission was denied by user for file access.  Can not save disease",
					"File Save Error", JOptionPane.INFORMATION_MESSAGE);
			return;
		}

		if (bIsApplet)
			PrivilegeManager.enablePrivilege("UniversalFileAccess");
	
					
		fd.setTitle("Save Disease");
		fd.setMode(FileDialog.SAVE);
		fd.setVisible(true);

		if (fd.getFile() == null)
			return;

		File diseaseFile = new File(fd.getDirectory(), fd.getFile());

		// don't need to check if file exists, this is done by fd

		// write disease to file
		try {
			FileWriter fw = new FileWriter(diseaseFile);
			PrintWriter pr = new PrintWriter(fw);
			pr.print(newDisease.toString());

			// close file
			pr.close();
		}
		catch(IOException e) {
			// display warning that we can't access file
			JOptionPane.showMessageDialog(frame, "Error writing to " + diseaseFile +
				"\nFile save was aborted!",
				"File Save Error", JOptionPane.INFORMATION_MESSAGE);
		}			
		
	}

	// get a Disease from a file.  User is asked for a file name.
	// null is returned if the user does not enter a filename or if an error occurs
	private Disease getDiseaseFromFile() {
		if (!bHasPrivilege) {
			JOptionPane.showMessageDialog(frame, "Permission was denied by user for file access.  Cannot open disease",
					"File Open Error", JOptionPane.INFORMATION_MESSAGE);
			return null;
		};

		if (bIsApplet)
			PrivilegeManager.enablePrivilege("UniversalFileAccess");

		fd.setTitle("Select Disease");
		fd.setMode(FileDialog.LOAD);
		fd.setVisible(true);

		if (fd.getFile() != null) {
			File diseaseFile = new File(fd.getDirectory(), fd.getFile());
			try {
				FileReader fr = new FileReader(diseaseFile);
				BufferedReader in = new BufferedReader(fr);
				String sDisease = new String();
				while (in.ready())
					sDisease += in.readLine() + "\n";
				in.close();

				Disease disease = Disease.createDisease(sDisease);
				return disease;
			}
			catch(IOException e) {
				// display warning that we can't access file
				JOptionPane.showMessageDialog(frame, "File " + diseaseFile + " could not be read." +
					"\nPlease verify that the file exists and is not in use by another application",
					"File Open Error", JOptionPane.INFORMATION_MESSAGE);
				return null;
			}
			catch(InvalidDiseaseException e) {
				// display warning that we can't access file
				JOptionPane.showMessageDialog(frame, "File " + diseaseFile + " does not contain a valid disease.  In particular: " +e.toString(),
					"Disease File Open Error", JOptionPane.INFORMATION_MESSAGE);
				return null;
			}
		} // user selects a file

		return null;
	}

	// get a GSD from a file.  User is asked for a file name.
	// null is returned if the user does not enter a filename or if an error occurs
	private GeneticScreenDescriptor getGSDFromFile() {
		if (!bHasPrivilege) {
			JOptionPane.showMessageDialog(frame, "Permission was denied by user for file access.  Can not open genetic screen",
					"File Open Error", JOptionPane.INFORMATION_MESSAGE);
			return null;
		};

		if (bIsApplet)
			PrivilegeManager.enablePrivilege("UniversalFileAccess");

		fd.setTitle("Select Genetic Screen");
		fd.setMode(FileDialog.LOAD);
		fd.setVisible(true);

		if (fd.getFile() != null) {
			File gsdFile = new File(fd.getDirectory(), fd.getFile());
			try {
				FileReader fr = new FileReader(gsdFile);
				BufferedReader in = new BufferedReader(fr);
				String sGSD = new String();
				while (in.ready())
					sGSD += in.readLine() + "\n";
				in.close();

				GeneticScreenDescriptor gsd = new GeneticScreenDescriptor(sGSD);
				return gsd;
			}
			catch(IOException e) {
				// display warning that we can't access file
				JOptionPane.showMessageDialog(frame, "File " + gsdFile + " could not be read." +
					"\nPlease verify that the file exists and is not in use by another application",
					"File Open Error", JOptionPane.INFORMATION_MESSAGE);
				return null;
			}
			catch(InvalidGeneticScreenDescriptorException e) {
				// display warning that we can't access file
				JOptionPane.showMessageDialog(frame, "File " + gsdFile + " does not contain a valid genetic screen.",
					"Disease File Open Error", JOptionPane.INFORMATION_MESSAGE);
				return null;
			}
		} // user selects a file

		return null;
	}

	private void saveGSDToFile(GeneticScreenDescriptor newGSD) {
	// requires:  newGSD is not null
	// effects: asks user where to save GSD to file

		if (!bHasPrivilege) {
			JOptionPane.showMessageDialog(frame, "Permission was denied by user for file access.  Can not save genetic screen",
					"File Save Error", JOptionPane.INFORMATION_MESSAGE);
			return;
		}

		if (bIsApplet)
			PrivilegeManager.enablePrivilege("UniversalFileAccess");
	
					
		fd.setTitle("Save Genetic Screen");
		fd.setMode(FileDialog.SAVE);
		fd.setVisible(true);

		if (fd.getFile() == null)
			return;

		File gsdFile = new File(fd.getDirectory(), fd.getFile());

		// don't need to check if file exists, this is done by fd

		// write disease to file
		try {
			FileWriter fw = new FileWriter(gsdFile);
			PrintWriter pr = new PrintWriter(fw);
			pr.print(newGSD.toString());

			// close file
			pr.close();
		}
		catch(IOException e) {
			// display warning that we can't access file
			JOptionPane.showMessageDialog(frame, "Error writing to " + gsdFile +
				"\nFile save was aborted!",
				"File Save Error", JOptionPane.INFORMATION_MESSAGE);
		}			
		
	}

	// called in response to user asking to edit a disease.
	private void editDisease() {
		Disease disease = getDiseaseFromFile();
		if (disease != null) {
			// now that we have the disease, let's edit it
			NewDiseaseDialog dd = new NewDiseaseDialog(frame, disease);
			dd.setVisible(true);
			Disease newDisease = dd.getNewDisease();
			if (newDisease != null) {
				// save the new disease to a file
				saveDiseaseToFile(newDisease);
			}
		}		
	}

	// called in response to user asking to edit a GSD.
	private void editGSD() {
		GeneticScreenDescriptor gsd = getGSDFromFile();
		if (gsd != null) {
			// now that we have the gsd, let's edit it
			GeneticScreenDescriptorDialog gsdd = new GeneticScreenDescriptorDialog(frame, gsd, fd);
			gsdd.setVisible(true);
			if (gsdd.didGeneticScreenDescriptorChange()) {
				GeneticScreenDescriptor newGSD= gsdd.getGeneticScreenDescriptor();
				saveGSDToFile(newGSD);
			}

		}		
	}

	// add to gsd to the pedigreeCanvas
	private void addGSD() {
		GeneticScreenDescriptor gsd = getGSDFromFile();
		if (gsd != null) {
			try {
				pedCanvas.addGeneticScreenDescriptor(gsd);
			}
			catch(InvalidGeneticScreenDescriptorException e) {
				JOptionPane.showMessageDialog(frame, gsd.getName() + " could not be added to the pedigree.\n" +
					"The genetic screen is either incompatible with the disease or a genetic screen with it's name is in the pedigree",
				"Genetic Screen Error", JOptionPane.INFORMATION_MESSAGE);
			}
		}		
	}


	// create the URL for a relitive filename on the server
	// return null if not successful
    private URL getURL(String filename) {
        URL codeBase = getCodeBase();
        URL url = null;

        try {
            url = new URL(codeBase, filename);
        } catch (java.net.MalformedURLException e) {
            System.err.println("Couldn't create image: badly specified URL");
            return null;
        }

        return url;
    }

}














