//package GeneticScreen;

//import Person.*;
//import BayesNetwork.*;
//import Pedigree.*;
//import GenExceptions.*;

import java.util.*;

public class GeneticScreenDescriptor {

	//local state
	String		testName;
	String		diseaseName;
	int			supportedSexType;
	String[]	possResults;
	String[]	genotypes;
	double[][]	probs;

	public GeneticScreenDescriptor(String testName, String diseaseName, int suppportSexType, String[] genotypes, String[] possResults, double[][] probs) 
		throws InvalidGeneticScreenDescriptorException {
	// requires:  none of the inputs can be null, genotypes is a valid list from disease.getGenotypes
	// effects:  Creates a geneticScreenDescriptor with name testName, having possResults for a test
	//			 .  Genotypes is the genotypes this test is for which is from a disease.  The order
	//				should match the disease.getGenotype order.  probs is the probabilty of each result
	//				for each genotype, using indexes from possResults and genotypes.
	//				A InvalidGeneticScreenDescriptorException is thrown if possReslts or genotypes
	//				has less than two, or if the size of probs is incorrect, or if a row of probs does
	//				not sum close to 1.0
		System.out.println("!");
		if ((possResults.length < 2) || (genotypes.length < 2))
			throw new InvalidGeneticScreenDescriptorException();

		System.out.println("!");
		if (probs.length != genotypes.length)
			throw new InvalidGeneticScreenDescriptorException();

		System.out.println("!");
		for(int x = 0; x < probs.length; x++) {
			if (probs[x].length != possResults.length)
				throw new InvalidGeneticScreenDescriptorException();
			double dSum = 0;
			for(int y=0; y < possResults.length; y++)
				dSum += probs[x][y];
			System.out.println("!" + dSum);
			// the sum has to be within 0.1 of 1
			if (java.lang.Math.abs(dSum - 1.0) > .1)
				throw new InvalidGeneticScreenDescriptorException();
		}


		this.testName = testName;
		this.diseaseName = diseaseName;
		this.supportedSexType = supportedSexType;
		this.possResults = (String[]) possResults.clone();
		this.genotypes = (String[])genotypes.clone();
		this.probs = (double[][])probs.clone();
	}

	public GeneticScreenDescriptor(String s) throws InvalidGeneticScreenDescriptorException {
		StringTokenizer st = new StringTokenizer(s);
		try {
			if (!st.nextToken().equals("*GeneticScreenDescriptor*"))
				throw new InvalidGeneticScreenDescriptorException();

			testName = st.nextToken();
			diseaseName = st.nextToken();
			supportedSexType = Integer.valueOf(st.nextToken()).intValue();
			if ((supportedSexType < Person.MALE) || (supportedSexType > Person.UNKNOWN))
				throw new InvalidGeneticScreenDescriptorException();

			if (!st.nextToken().equals("Results"))
				throw new InvalidGeneticScreenDescriptorException();
			int possResultsLength = Integer.valueOf(st.nextToken()).intValue();;
			if (possResultsLength < 2)
				throw new InvalidGeneticScreenDescriptorException();

			possResults = new String[possResultsLength];			
			for(int x=0; x < possResultsLength; x++)
				possResults[x] = st.nextToken();

			if (!st.nextToken().equals("Genotypes"))
				throw new InvalidGeneticScreenDescriptorException();
			int genotypesLength = Integer.valueOf(st.nextToken()).intValue();;
			if (genotypesLength < 2)
				throw new InvalidGeneticScreenDescriptorException();

			genotypes = new String[genotypesLength];
			for(int x=0; x < genotypesLength; x++)
				genotypes[x] = st.nextToken();

			if (!st.nextToken().equals("Probs"))
				throw new InvalidGeneticScreenDescriptorException();
			
			probs = new double[genotypesLength][possResultsLength];
			for(int x=0; x < genotypes.length; x++) {
				for(int y=0; y < possResults.length; y++)
					probs[x][y] = Double.valueOf(st.nextToken()).doubleValue();
			}
		}
		catch(NoSuchElementException e) {
			throw new InvalidGeneticScreenDescriptorException();
		}
		catch(NumberFormatException e) {
			throw new InvalidGeneticScreenDescriptorException();
		}

		for(int x = 0; x < probs.length; x++) {
			if (probs[x].length != possResults.length)
				throw new InvalidGeneticScreenDescriptorException();
			double dSum = 0;
			for(int y=0; y < possResults.length; y++)
				dSum += probs[x][y];

			// the sum has to be within 0.1 of 1
			if (java.lang.Math.abs(dSum - 1.0) > .1)
				throw new InvalidGeneticScreenDescriptorException();
		}
	}

	public String getName() {
		return testName;
	}

	public String getDiseaseName() {
		return diseaseName;
	}

	public int getSupportedSexType() {
		return supportedSexType;
	}

	public String[] getPossibleResults() {
		return (String[])possResults.clone();
	}

	public String[] getGenotypes() {
		return (String[])genotypes.clone();
	}

	public double[][] getProbs() {
		return (double[][])probs.clone();
	}

	public boolean canTest(String[] pGenotypes) {
	// requires: pGenotype is not null
	// effects:  Returns true if this test can be performed on a person with pGenotype (ie do the genotypes match)
		if (pGenotypes.length != genotypes.length)
			return false;

		for(int x=0; x < genotypes.length; x++)
			if (!genotypes[x].equals(pGenotypes[x]))
				return false;

		return true;
	}

	public String toString() {
		String s = new String("*GeneticScreenDescriptor*\n");
		s += testName + "\n";
		s += diseaseName + "\n";
		s += supportedSexType + "\n";
		s += "Results " + possResults.length + "\n";
		for(int x=0; x < possResults.length; x++)
			s += possResults[x] + "\n";
		s += "Genotypes " + genotypes.length + "\n";
		for(int x=0; x < genotypes.length; x++)
			s += genotypes[x] + "\n";
		s += "Probs\n";
		for(int x=0; x < genotypes.length; x++) {
			for(int y=0; y < possResults.length; y++)
				s += probs[x][y] + "\n";
		}
		return s;
	}

	public GeneticScreen createGeneticScreen(Pedigree ped, Person p, BayesNetwork bn) throws InvalidGeneticScreenDescriptorException {
		try {
			BayesNetworkNode testNode = bn.createNode(possResults, probs[0]);  // prob is temp assignment
			BayesNetworkNode genoNode = p.getGenotypeNode();
			bn.createArc(genoNode, testNode);
			BayesNetworkNode[] parentNodes = {genoNode};

			for (int x = 0; x < probs.length; x++) {
				String[] parentValues = {genotypes[x]};
				testNode.changeProbFunction(parentNodes, parentValues, probs[x]);
			}

			return new GenericGeneticScreen(this, ped, p, testNode);
		}
		catch(InvalidArcException e) {
			throw new InvalidGeneticScreenDescriptorException();
		}
		catch(ProbabilityException e) {
			throw new InvalidGeneticScreenDescriptorException();
		}
		catch(InvalidValueException e) {
			throw new InvalidGeneticScreenDescriptorException();
		}
	}

	public void removeGeneticScreen(GeneticScreen gs, BayesNetwork bn) {
		bn.removeNode(((GenericGeneticScreen)gs).getBayesNetworkNode());
	}

	public boolean equals(Object o) {
		// effects:  returns true if o is a gsd with the same name as this
		try {
			GeneticScreenDescriptor gsd = (GeneticScreenDescriptor)o;
			if (gsd.getName().equals(this.getName()))
				return true;
			else
				return false;
		}
		catch(ClassCastException e) {
			return false;
		}
	}

}

class GenericGeneticScreen extends GeneticScreen {

	// local state
	GeneticScreenDescriptor gsd;
	Pedigree ped;
	Person p;
	BayesNetworkNode bnn;

	public GenericGeneticScreen(GeneticScreenDescriptor gsd, Pedigree ped, Person p, BayesNetworkNode inferenceNode) {
		this.gsd = gsd;
		this.ped = ped;
		this.p = p;
		this.bnn = inferenceNode;
		String[] values = bnn.getValues();
		try {
			bnn.makeObservation(values[0]);
		}
		catch(InvalidValueException e) {
			System.err.println("Unexpected InvaludValueException in constructor of GenericGeneticScreen");
		}
		catch(ProbabilityException e){
		  System.err.println("ProbabilityException occured");
		}
	}

	public Pedigree getPedigree() {
		return ped;
	}

	public Person getPerson() {
		return p;
	}

	public String getResult() {
		return bnn.getObservation();
	}

	public void setResult(String result) throws InvalidValueException, ProbabilityException {
		bnn.makeObservation(result);
		ped.clearAllInferences();
	}

	public GeneticScreenDescriptor getGeneticScreenDescriptor() {
		return gsd;
	}

	public BayesNetworkNode getBayesNetworkNode() {
		return bnn;
	}
}
