//package GeneticScreen;

//import Person.*;
//import BayesNetwork.*;
//import Disease.*;
//import GenExceptions.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*; //Property change stuff
import javax.swing.border.LineBorder;
import java.io.*;
import java.util.Vector;

public class GeneticScreenDescriptorDialog extends JDialog {

	boolean bGeneticScreenChanged;
	String	    gsdName;
	String		diseaseName;
	String[]	genotypes;
	double[][]	probs;
	int		    iSexSupport;

	JFrame fOwner;
	GeneticScreenDescriptor gsd;
	FileDialog fd;
	JComboBox resultsComboBox;
	JLabel sexLabel;

	// constructor
	public GeneticScreenDescriptorDialog(JFrame f, GeneticScreenDescriptor gsd, FileDialog fd) {
		super(f, "Edit genetic screen", true);

		this.fOwner = f;
		this.fd = fd;
		this.gsd = gsd;
		final Vector vResults = new Vector();

		if (gsd != null) {
			gsdName = gsd.getName();
			diseaseName = gsd.getDiseaseName();
			genotypes = gsd.getGenotypes();
			probs = gsd.getProbs();
			iSexSupport = gsd.getSupportedSexType();
			bGeneticScreenChanged = false;
		}
		else {
			gsdName = null;
			diseaseName = null;
			genotypes = null;
			probs =null;
			bGeneticScreenChanged = true;
		}

		// create name panel
		JPanel namePanel = new JPanel();
		namePanel.add(new JLabel("Name: "));
		final JTextField nameTextField = new JTextField(10);
		if (gsdName != null)
			nameTextField.setText(gsdName);
		namePanel.add(nameTextField);
		namePanel.setBorder(LineBorder.createBlackLineBorder());

		// create disease panel
		JPanel diseasePanel = new JPanel();
		diseasePanel.setLayout(new BorderLayout());
		JPanel diseaseNamePanel = new JPanel();
		diseaseNamePanel.add(new JLabel("Disease: "));
		final JLabel diseaseNameLabel = new JLabel();
		if (diseaseName == null)
			diseaseNameLabel.setText("None");
		else
			diseaseNameLabel.setText(diseaseName);
		diseaseNamePanel.add(diseaseNameLabel);
		diseasePanel.add(diseaseNamePanel, BorderLayout.NORTH);
		JButton selectDiseaseButton = new JButton("Select Disease");
		selectDiseaseButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (selectNewDisease())
						bGeneticScreenChanged = true;
					// if (diseaseName != null)
						diseaseNameLabel.setText(diseaseName);
				}
		});
		diseasePanel.add(selectDiseaseButton, BorderLayout.SOUTH);
		sexLabel = new JLabel("");
		if (gsd != null)
			setSexLabel();				
		diseasePanel.add(sexLabel, BorderLayout.CENTER);
		diseasePanel.setBorder(LineBorder.createBlackLineBorder());

		// create results panel
		JPanel resultsPanel = new JPanel();
		resultsPanel.setLayout(new BorderLayout());
		JPanel topResultsPanel = new JPanel();
		final JButton removeResultButton = new JButton("Remove Result");
		resultsComboBox = new JComboBox();
		removeResultButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String sResult = (String)resultsComboBox.getSelectedItem();
					resultsComboBox.removeItem(sResult);
					vResults.removeElement(sResult);
					if (resultsComboBox.getItemCount() == 0)
						removeResultButton.setEnabled(false);
					bGeneticScreenChanged=true;
					probs = null;
				}
		});
		if (gsd != null) {
			String [] results = gsd.getPossibleResults();
			for(int x = 0; x < results.length; x++)
				resultsComboBox.addItem(results[x]);
		}
		else
			removeResultButton.setEnabled(false);
		topResultsPanel.add(removeResultButton);
		topResultsPanel.add(resultsComboBox);
		JPanel bottomResultsPanel = new JPanel();
		final JTextField resultTextField = new JTextField(10);
		JButton addResultButton = new JButton("Add Result");
		addResultButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String sResult = resultTextField.getText();
					if (sResult.equals(""))
						return;
					if (vResults.contains(sResult)) {
						errorMessage("This result has already been added");
						return;
					}
					resultsComboBox.addItem(sResult);
					vResults.addElement(sResult);
					if (resultsComboBox.getItemCount() == 1)
						removeResultButton.setEnabled(true);
					resultTextField.setText("");
					bGeneticScreenChanged=true;
					probs = null;
				}
		});
		bottomResultsPanel.add(addResultButton);
		bottomResultsPanel.add(resultTextField);
		resultsPanel.add(topResultsPanel, BorderLayout.NORTH);
		resultsPanel.add(bottomResultsPanel, BorderLayout.SOUTH);
		resultsPanel.setBorder(LineBorder.createBlackLineBorder());


		// Probability Table
		JPanel probPanel = new JPanel();
		probPanel.setLayout(new BorderLayout());
		probPanel.add(new JLabel("Probability Table"), BorderLayout.NORTH);
		JButton probTableButton = new JButton("Edit Probability Table");
		probTableButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					editProbTable();
				}
		});
		probPanel.add(probTableButton, BorderLayout.SOUTH);
		probPanel.setBorder(LineBorder.createBlackLineBorder());


		Object[] array = {namePanel, diseasePanel, resultsPanel, probPanel};

        final String btnString1 = "OK";
        final String btnString2 = "Cancel";
        Object[] options = {btnString1, btnString2};

        final JOptionPane optionPane = new JOptionPane(array, 
                                    JOptionPane.PLAIN_MESSAGE,
                                    JOptionPane.YES_NO_OPTION,
                                    null,
                                    options, 
									options[0]);
        setContentPane(optionPane);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                optionPane.setValue(new Integer(
                                        JOptionPane.CLOSED_OPTION));
            }
        });

        optionPane.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
                String prop = e.getPropertyName();
				if (isVisible() 
                 && (e.getSource() == optionPane)
                 && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
                     prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
                    Object value = optionPane.getValue();

                    if (value == JOptionPane.UNINITIALIZED_VALUE) {
                        //ignore reset
                        return;
                    }

                    // Reset the JOptionPane's value.
                    // If you don't do this, then if the user
                    // presses the same button next time, no
                    // property change event will be fired.
                    optionPane.setValue(
                            JOptionPane.UNINITIALIZED_VALUE);

                    if (value.equals(btnString1)) {  // the user pushed ok
						String testName = nameTextField.getText().trim();
						if (testName.equals("")) {
							errorMessage("Genetic Screen must be named");
						}
						if (diseaseName == null) {
							errorMessage("A disease must be selected.");
							return;
						}
						if (resultsComboBox.getItemCount() < 2) {
							errorMessage("A genetic screen must have at least two possible results.");
							return;
						}
						if (probs == null) {
							errorMessage("The probability table must be updated first.");
							return;
						}

						try {
							String[] sPossibleResults = new String[resultsComboBox.getItemCount()];
							for (int x =0; x < resultsComboBox.getItemCount(); x++)
								sPossibleResults[x] = (String)resultsComboBox.getItemAt(x);
							GeneticScreenDescriptor newGsd = new GeneticScreenDescriptor(testName, diseaseName, iSexSupport, genotypes, sPossibleResults, probs);
							setNewGeneticScreenDescriptor(newGsd);
						}
						catch(InvalidGeneticScreenDescriptorException igsde) {
							// this should never happen
							errorMessage("A genetic screen can not be created with this description.\n" +
										 "The probability table must be completed with rows summing to 1");
							return;
						}
						bGeneticScreenChanged = true;
						setVisible(false);
					}
					else { // user clicked cancel
                        bGeneticScreenChanged = false;
                        setVisible(false);
                    }
                }
            }
        });

		this.pack();
	}

	private void errorMessage(String eMessage) {
		JOptionPane.showMessageDialog(this, eMessage,
					"Genetic Screen Edit Error", JOptionPane.INFORMATION_MESSAGE);
	}

	public boolean didGeneticScreenDescriptorChange() {
		return bGeneticScreenChanged;
	}

	public GeneticScreenDescriptor getGeneticScreenDescriptor() {
		return gsd;
	}

	private void setNewGeneticScreenDescriptor(GeneticScreenDescriptor gsd) {
		this.gsd = gsd;
	}

	private boolean selectNewDisease() {
		fd.setTitle("Select Disease");
		fd.setMode(FileDialog.LOAD);
	//	fd.setLocationRelativeTo(fOwner);
		fd.setVisible(true);

		if (fd.getFile() != null) {
			File diseaseFile = new File(fd.getDirectory(), fd.getFile());
			try {
				FileReader fr = new FileReader(diseaseFile);
				BufferedReader in = new BufferedReader(fr);
				String sDisease = new String();
				while (in.ready())
					sDisease += in.readLine() + "\n";
				in.close();

				Disease d = Disease.createDisease(sDisease);
				diseaseName = d.getName();
				
				// see if sex is an issue by comparing male and child genotypes
			//	String[] mGenotypes = d.getGenotypes(Person.MALE);
			//	String[] cGenotypes = d.getGenotypes(Person.UNKNOWN);
			//	if (mGenotypes.length == cGenotypes.length) {

				// instead use sexlinked function
				if (!d.isSexLinked()) {
					iSexSupport = Person.UNKNOWN;
					System.out.println("sexsupport " + Person.UNKNOWN);
					genotypes = d.getGenotypes(Person.MALE);
					return true;
				}

				ChooseSexSupportDialog sexDialog = new ChooseSexSupportDialog(fOwner);
				iSexSupport = sexDialog.getSexChoice();
				genotypes = d.getGenotypes(iSexSupport);
				return true;

			}
			catch(IOException e) {
				errorMessage("File " + diseaseFile + " could not be read." +
					"\nPlease verify that the file exists and is not in use by another application");
				return false;
			}
			catch(InvalidDiseaseException e) {
				errorMessage("File " + diseaseFile + " does not contain a valid disease.");
				return false;
			}
		} // user selects a file
		return false;
	}

	private void editProbTable() {
		if ((genotypes==null) || (resultsComboBox.getItemCount() < 2)) {
			errorMessage("A disease must be selected and there must be at least two results before the probability table can be created.");
			return;
		}

		if (probs == null) 
			probs = new double[genotypes.length][resultsComboBox.getItemCount()];

		ProbTableDialog dialog = new ProbTableDialog(fOwner, genotypes, resultsComboBox, probs);
		dialog.pack();
		dialog.setLocationRelativeTo(this);
		dialog.setVisible(true);
	}

	private void setSexLabel() {
		if (iSexSupport == Person.UNKNOWN)
			sexLabel.setText("Support for both sex genotypes.");
		else if (iSexSupport == Person.MALE)
			sexLabel.setText("Support for male genotype.");
		else
			sexLabel.setText("Support for female genotype.");
		pack();
	}

}


class ProbTableDialog extends JDialog {

	int indexOfOldSelection;
	int numResults;

	public ProbTableDialog(JFrame fOwner, String[] sGenotypes, JComboBox resultsComboBox, double[][] iprobs) {
		super(fOwner, "Edit genetic screen probability table", true);

		final double[][] probs = iprobs;
		final JTextField[] probFields= new JTextField[resultsComboBox.getItemCount()];

		JPanel genotypePanel = new JPanel();
		genotypePanel.add(new JLabel("Genotype", JLabel.CENTER));
		final JComboBox genoComboBox = new JComboBox(sGenotypes);
		genotypePanel.add(genoComboBox);
		genoComboBox.addActionListener(new ActionListener() {
			 public void actionPerformed(ActionEvent e) {
				 try {
				 // save to old values
				 double dSum = 0;
				 for (int x = 0; x < numResults; x++)
					 dSum += probs[indexOfOldSelection][x] = Double.valueOf(probFields[x].getText()).doubleValue();
				 // make sure the row sums to 1.0
				 if (java.lang.Math.abs(dSum - 1.0) > .1) {
					errorMessage("The probabilities for this row must sum to 1.0");
					genoComboBox.setSelectedIndex(indexOfOldSelection);
					return;
				 }

				 // set up the new row
				 indexOfOldSelection = genoComboBox.getSelectedIndex();
				 for (int x=0; x < numResults; x++)
					 probFields[x].setText(Double.toString(probs[indexOfOldSelection][x]));
				 }
				 catch(NumberFormatException nfe) {
					 errorMessage("Please enter valid probabilities");
				 }
			 }
        });
		indexOfOldSelection = 0;
	
		JPanel probPanel = new JPanel();
		probPanel.setLayout(new BorderLayout());
		probPanel.add(new JLabel("Probabilities", JLabel.CENTER), BorderLayout.NORTH);
		numResults = resultsComboBox.getItemCount();
		String[] results = new String[numResults];
		for(int x=0; x < numResults; x++)
			results[x] = (String)resultsComboBox.getItemAt(x);
	
		JPanel probEntryPanel = new JPanel();
		
		for (int x=0; x < results.length; x++) {
			JPanel panel = new JPanel();
			panel.setLayout(new BorderLayout());
			panel.add(new JLabel(results[x], JLabel.CENTER), BorderLayout.NORTH);
			probFields[x] = new JTextField(Double.toString(probs[0][x]), 5);
			panel.add(probFields[x], BorderLayout.SOUTH);
			probEntryPanel.add(panel);
		}
		probPanel.add(probEntryPanel, BorderLayout.CENTER);

		Object[] array = {genotypePanel, probPanel};

        final String btnString1 = "OK";
        final String btnString2 = "Cancel";
        Object[] options = {btnString1, btnString2};

        final JOptionPane optionPane = new JOptionPane(array, 
                                    JOptionPane.PLAIN_MESSAGE,
                                    JOptionPane.YES_NO_OPTION,
                                    null,
                                    options, 
									options[0]);
        setContentPane(optionPane);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                optionPane.setValue(new Integer(
                                        JOptionPane.CLOSED_OPTION));
            }
        });

        optionPane.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
                String prop = e.getPropertyName();
				if (isVisible() 
                 && (e.getSource() == optionPane)
                 && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
                     prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
                    Object value = optionPane.getValue();

                    if (value == JOptionPane.UNINITIALIZED_VALUE) {
                        //ignore reset
                        return;
                    }

                    // Reset the JOptionPane's value.
                    // If you don't do this, then if the user
                    // presses the same button next time, no
                    // property change event will be fired.
                    optionPane.setValue(
                            JOptionPane.UNINITIALIZED_VALUE);

                    if (value.equals(btnString1)) {  // the user pushed ok
					// save to old values
						double dSum = 0;
						for (int x = 0; x < numResults; x++)
							dSum += probs[indexOfOldSelection][x] = Double.valueOf(probFields[x].getText()).doubleValue();
						// make sure the row sums to 1.0
						if (java.lang.Math.abs(dSum - 1.0) > .1) {
							errorMessage("The probabilities for this row must sum to 1.0");
							return;
						}
						setVisible(false);
					}
					else { // user clicked cancel
                        setVisible(false);
                    }
                }
            }
        });
	}

	private void errorMessage(String eMessage) {
		JOptionPane.showMessageDialog(this, eMessage,
					"Genetic Screen Edit Error", JOptionPane.INFORMATION_MESSAGE);
	}
}



		


