//package Disease;

//import Person.*;
//import BayesNetwork.*;
//import GenExceptions.*;

import java.util.*;

public class NewXlinkedDisease extends Disease {
	final int NUM_AGE_GROUPS= 9;

	String name;
	String[] badAlleles;
	String normalAllele;
	double[] mutationRates;
	double[][] expressionTable;
	Vector vRaces;
	double[][] alleleFreqRates;

	// these are locals created by init
  String[] mGenotypes;
  String[] fGenotypes;
  String[] unknownGenotypes;
	double[][] mGenotypeFreqs;  // this is the race/ genotype frequency calculated from alleleFreq
  double[][] fGenotypeFreqs;
  double[][] unknownGenotypeFreqs;
  double[][] unknownInheritenceTable;
	double noMutationRate;  // this is the rate by which the normal allele does not mutatate
	boolean bHasMutationRate;
	boolean bAgeDependentExpression;
	boolean bRaceDependent;
	Hashtable mutationAlleleIndexTable;

	public NewXlinkedDisease(String name, String normalAllele,  String[] badAlleles,
		double[] mutationRates, double[][] expressionRates, Vector vRaces, double[][] freqRates) {

		this.name = name;
		this.normalAllele = normalAllele;
		this.badAlleles = badAlleles;

		this.mutationRates = mutationRates;
		this.expressionTable = expressionRates;
		this.vRaces = vRaces;
		this.alleleFreqRates = freqRates;

		init();
	}

	public NewXlinkedDisease(String s) throws InvalidDiseaseException {
		StringTokenizer st = new StringTokenizer(s, "\n");

		try {
			String sNext = st.nextToken();
			if (!sNext.equals("NewXlinkedDisease"))
				throw new InvalidDiseaseException("Invalid Disease Title");

			this.name = st.nextToken();

			this.normalAllele = st.nextToken();

			int iNumBadAlleles = Integer.valueOf(st.nextToken()).intValue();
			if (iNumBadAlleles < 1)
				throw new InvalidDiseaseException("Number of Alleles incorrect");

			this.badAlleles = new String[iNumBadAlleles];
			for (int x= 0; x < iNumBadAlleles; x++)
				this.badAlleles[x] = st.nextToken();

			sNext = st.nextToken();
			if (sNext.equals("Mutation Rates")) {
				// the number of mutations is equals to number of bad alleles
				this.mutationRates = new double[iNumBadAlleles];
				for (int x = 0; x < iNumBadAlleles; x++)
					this.mutationRates[x] = Double.valueOf(st.nextToken()).doubleValue();

				// get the next string for age dependency
				sNext = st.nextToken();
			}

			int iNumOfAlleles = iNumBadAlleles + 1;
			int iNumGenotypes = iNumOfAlleles * (iNumOfAlleles + 1)/2 + iNumOfAlleles;  // Female and male genotypes
			if (sNext.equals("Age dependent")) {
				this.expressionTable = new double[NUM_AGE_GROUPS][iNumGenotypes];
				for (int x= 0; x < this.expressionTable.length; x++) {
					for (int y = 0; y < this.expressionTable[x].length; y++)
						this.expressionTable[x][y] = Double.valueOf(st.nextToken()).doubleValue();
				}
			}
			else if (sNext.equals("Not age dependent")) {
				this.expressionTable = new double[1][iNumGenotypes];
				for (int x=0; x < this.expressionTable[0].length; x++)
					this.expressionTable[0][x] = Double.valueOf(st.nextToken()).doubleValue();
			}
			else
				throw new InvalidDiseaseException("Age dependency not indicated");

			sNext = st.nextToken();
			if (sNext.equals("Race dependent")) {
				int iNumRaces = Integer.valueOf(st.nextToken()).intValue();
				this.vRaces = new Vector();
				for (int x=0; x < iNumRaces; x++)
					this.vRaces.addElement(st.nextToken());

				// next string for freq
				sNext = st.nextToken();
			}
		
			if (!sNext.equals("Freq rates"))
				throw new InvalidDiseaseException("Missing Freq Rates");

			// the format for freqRates is one table entry per race, with the first entry
			// in the table is normal alleles, and then the bad alleles in the same order as in the badAllele array
			if (vRaces != null)
				this.alleleFreqRates = new double[vRaces.size()][iNumOfAlleles];
			else
				this.alleleFreqRates = new double[1][iNumOfAlleles];

			for (int x = 0; x < alleleFreqRates.length; x++) {
				for (int y = 0; y < alleleFreqRates[0].length; y++)
					alleleFreqRates[x][y] = Double.valueOf(st.nextToken()).doubleValue();
			}

			// now that all the input variables are set, we can init all other locals
			init();

			System.out.println(this.toString());
		}
		catch(NoSuchElementException e) {
			throw new InvalidDiseaseException("No Such Element");
		}
		catch(NumberFormatException e) {
			throw new InvalidDiseaseException("Number Formatted Incorrectly");
		}
	}


	private void init() {

		if (mutationRates == null)
			bHasMutationRate = false;
		else
			bHasMutationRate = true;

		if (expressionTable.length > 1)
			bAgeDependentExpression = true;
		else
			bAgeDependentExpression = false;

		if (vRaces == null)
			bRaceDependent = false;
		else
			bRaceDependent = true;

		// create the hashtable for the mutationIndex
		mutationAlleleIndexTable = new Hashtable(badAlleles.length);
		for(int x = 0; x < badAlleles.length; x++)
			mutationAlleleIndexTable.put(badAlleles[x], new Integer(x));

		// noMutationRate calculation
		noMutationRate = 1.0;
		if (bHasMutationRate) {
			for(int x=0; x < mutationRates.length; x++)
				noMutationRate = noMutationRate - mutationRates[x];
		}

		// create the genotypes and freqs
		int iNumOfAlleles = badAlleles.length + 1;
		fGenotypes = new String[iNumOfAlleles * (iNumOfAlleles + 1)/2];
		mGenotypes = new String[iNumOfAlleles];
		unknownGenotypes = new String[fGenotypes.length+mGenotypes.length];
		if (bRaceDependent){
			mGenotypeFreqs = new double[vRaces.size()][mGenotypes.length];
			fGenotypeFreqs = new double[vRaces.size()][fGenotypes.length];
			unknownGenotypeFreqs = new double[vRaces.size()][unknownGenotypes.length];
		}
		else{
		  mGenotypeFreqs = new double[1][mGenotypes.length];
		  fGenotypeFreqs = new double[1][fGenotypes.length];
		  unknownGenotypeFreqs = new double[1][unknownGenotypes.length];
		}
		
		int index=0;
		fGenotypes[index] = normalAllele + "-" + normalAllele;
		mGenotypes[index] = normalAllele;
		unknownGenotypes[index] = normalAllele + "-" + normalAllele;
		unknownGenotypes[unknownGenotypes.length-iNumOfAlleles+index] = normalAllele;
		for(int x = 0; x < unknownGenotypeFreqs.length; x++){
		  unknownGenotypeFreqs[x][index] = 0.5*alleleFreqRates[x][index]*alleleFreqRates[x][index];  // Fills freqs of normal-normal for each race.  *0.5 because there is a 0.5 prob of being guy or girl
		  fGenotypeFreqs[x][index] = alleleFreqRates[x][index]*alleleFreqRates[x][index];
		  unknownGenotypeFreqs[x][unknownGenotypes.length-iNumOfAlleles+index] = 0.5*alleleFreqRates[x][index];  // Fills freqs of normal for each race.
		  mGenotypeFreqs[x][index] = alleleFreqRates[x][index];
		}
		index++; // Next fill normal-bad freqs.  Note: There is only 1 normal allele.
		for (int x =0; x < badAlleles.length; x++) {
			fGenotypes[index] = normalAllele + "-" + badAlleles[x];
			mGenotypes[index] = badAlleles[x];  // males only have one allele
			unknownGenotypes[index] = normalAllele + "-" + badAlleles[x];
			unknownGenotypes[unknownGenotypes.length-iNumOfAlleles+index] = badAlleles[x];
			for(int y = 0; y < unknownGenotypeFreqs.length; y++){
			  unknownGenotypeFreqs[y][index] = 2 * 0.5 * alleleFreqRates[y][0]*alleleFreqRates[y][x+1];  // First allele is normal allele
			  fGenotypeFreqs[y][index] = 2*alleleFreqRates[y][0]*alleleFreqRates[y][x+1];
				unknownGenotypeFreqs[y][unknownGenotypes.length-iNumOfAlleles+index] = 0.5 * alleleFreqRates[y][x+1];
				mGenotypeFreqs[y][index] = alleleFreqRates[y][x+1];
			}
			index++;
		}
		// Next fill bad-bad freqs
		for(int x=0; x < badAlleles.length; x++) {
			for(int y=x; y < badAlleles.length; y++) {
				fGenotypes[index] = badAlleles[x] + "-" + badAlleles[y];
				unknownGenotypes[index] = badAlleles[x] + "-" +badAlleles[y];
				if (x == y) { // same allele (bad)
				  for(int z = 0; z < unknownGenotypeFreqs.length; z++){
						unknownGenotypeFreqs[z][index] = 0.5*alleleFreqRates[z][x+1]*alleleFreqRates[z][y+1];
						fGenotypeFreqs[z][index] = alleleFreqRates[z][x+1]*alleleFreqRates[z][y+1];
				  }
				}
				else{ // different alleles (bad)
				  for(int z = 0; z < unknownGenotypeFreqs.length; z++){
						unknownGenotypeFreqs[z][index] = 2 * 0.5 * alleleFreqRates[z][x+1]*alleleFreqRates[z][y+1];
						fGenotypeFreqs[z][index] = 2*alleleFreqRates[z][x+1]*alleleFreqRates[z][y+1];
				  }
				}

				index++;
			}
		}

		// create the inheritence table which has an entry for each combination of parent genotypes
		// size of inheritenceTable is taking two elements from genotypes with repetition
		// the array is set to all zeros by java
		double[][] mInheritenceTable = new double[(fGenotypes.length)*(mGenotypes.length)][unknownGenotypes.length];  // Will only use the single allele section but the entire array gets filled anyway
		double[][] fInheritenceTable = new double[(fGenotypes.length)*(mGenotypes.length)][unknownGenotypes.length];  // Will only use the double allele section but the entire array gets filled anyway
		unknownInheritenceTable = new double[(fGenotypes.length)*(mGenotypes.length)][unknownGenotypes.length];
		int theIndexInheritenceTable = 0;
		for(int x = 0; x < fGenotypes.length; x++) {
			int iIndex = fGenotypes[x].indexOf('-');
			String alleleOne = fGenotypes[x].substring(0,iIndex);
			String alleleTwo = fGenotypes[x].substring(iIndex + 1);
			for (int y = 0; y < mGenotypes.length; y++) {
				String alleleThree = mGenotypes[y];
				// WHAT DID I DO HERE?
				setInheritenceTableEntry(mInheritenceTable[theIndexInheritenceTable], alleleOne, alleleTwo);
				setInheritenceTableEntry(fInheritenceTable[theIndexInheritenceTable], alleleOne, alleleThree);
				setInheritenceTableEntry(fInheritenceTable[theIndexInheritenceTable], alleleTwo, alleleThree);
				unknownInheritenceTable[theIndexInheritenceTable] = fInheritenceTable[theIndexInheritenceTable];
				for(int i=(unknownGenotypes.length-(badAlleles.length+1)); i < unknownGenotypes.length; i++){
				  unknownInheritenceTable[theIndexInheritenceTable][i] = mInheritenceTable[theIndexInheritenceTable][i]*2;
				}
				theIndexInheritenceTable++;
			}
		}
		
	}

  public String getName(){
    return name;
  }

	private int getMutationIndex(String badAllele) {
	// requires:  mutationAlleleIndexTable has been initialized with all the badAlleles and their indexes by init()
		Integer iResult = (Integer)mutationAlleleIndexTable.get(badAllele);
		return iResult.intValue();
	}

	private boolean isNormalAllele(String allele) {
		if (allele.equals(normalAllele))
			return true;
		else return false;
	}

	// given the table entry and the two alleles, the genotypes that can result (taking mutation into account)
	// is added to the entry for the resulting genotypes
	private void setInheritenceTableEntry(double[] probEntry , String alleleOne, String alleleTwo) {
		boolean normalOne = isNormalAllele(alleleOne);
		boolean normalTwo = isNormalAllele(alleleTwo);
		int iSingleAlleleIndex = unknownGenotypes.length-(badAlleles.length+1);
		if (normalOne || normalTwo) {

			if (normalOne && normalTwo) {
				int index = 0;

				if (bHasMutationRate) {
				  probEntry[index] += .25 * noMutationRate * noMutationRate;  // Ex. AA
				  probEntry[iSingleAlleleIndex+index] += 2*.125*noMutationRate;  // Ex. A and A
					for (int x = 0; x < badAlleles.length; x++){
					  index++;
					  probEntry[index] += 2*.25 * noMutationRate * mutationRates[x];  // Ex. Aa and aA
					  probEntry[iSingleAlleleIndex+index]+=2*.125*mutationRates[x]; // Ex. a and a
					}
					for(int x = 0; x < badAlleles.length; x++) {
						for(int y = x; y < badAlleles.length; y++) {
						  if (x == y){
						    index++;
						    probEntry[index] += .25 * mutationRates[x] * mutationRates[x];  // Ex. aa
						  }
						  else{
						    index++;
						    probEntry[index] += 2*.25 * mutationRates[x] * mutationRates[y];  // Ex. ab
						  }
						}
					}
				}
				else{  // no mutation rate
				  probEntry[index] += .25;
				  probEntry[iSingleAlleleIndex+index] += .25;
				}
			}
			else {
				// only one is normal
				String theNormalAllele;
				String theMutationAllele;
				if (normalOne) {
					theNormalAllele = alleleOne;
					theMutationAllele = alleleTwo;
				}
				else {
					theNormalAllele = alleleTwo;
					theMutationAllele = alleleOne;
				}
				int indexOfMutationAllele = getMutationIndex(theMutationAllele);

				if (bHasMutationRate) {
				  probEntry[1+indexOfMutationAllele] += .25 * noMutationRate;  // Ex. Aa
				  probEntry[iSingleAlleleIndex] += .125 * noMutationRate;  // Ex. A
				  probEntry[iSingleAlleleIndex+1] += .125;  // Ex. a
					int indexIntoGenotypeArray = 0;
					for (int x = 0;	x <= indexOfMutationAllele; x ++) {
						indexIntoGenotypeArray += badAlleles.length - x + 1;
						probEntry[indexIntoGenotypeArray + indexOfMutationAllele - x] += .25 * mutationRates[x];  // Ex. aa
						probEntry[iSingleAlleleIndex+x+1] += .125 * mutationRates[x];  // Ex. a
					}
					for(int x = 1; x< badAlleles.length-indexOfMutationAllele-1; x++){
					  probEntry[indexIntoGenotypeArray+x] += 0.25*mutationRates[x+indexOfMutationAllele];
					  probEntry[iSingleAlleleIndex+indexOfMutationAllele+x-1] += .125*mutationRates[x];
					}
				}
				else{  // no mutation rate
				  probEntry[1+indexOfMutationAllele] += .25;  // Ex. Aa
				  probEntry[iSingleAlleleIndex] += .125;  // Ex. A
				  probEntry[iSingleAlleleIndex+1+indexOfMutationAllele] += .125;  // Ex. a
				}
			}
		}
		else {  // no normal alleles, no mutation's can occur
			int indexAlleleOne = getMutationIndex(alleleOne);
			int indexAlleleTwo = getMutationIndex(alleleTwo);

			// make the first one the higher allele by swapping
			if (indexAlleleOne < indexAlleleTwo) {
				int temp = indexAlleleOne;
				indexAlleleOne = indexAlleleTwo;
				indexAlleleTwo = temp;
			}

			// only deal with indexes from here on
			int indexIntoGenotypeArray = 0;
			for(int i = 0; i <= indexAlleleTwo; i++){
			  indexIntoGenotypeArray += badAlleles.length-i+1;
			}
			probEntry[indexIntoGenotypeArray + indexAlleleOne - indexAlleleTwo] += 0.25;  // Ex. ab
			probEntry[iSingleAlleleIndex+indexAlleleOne+1] += 0.125;  // Ex. a
			probEntry[iSingleAlleleIndex+indexAlleleTwo+1] += 0.125;  // Ex. b
		}
	}

	public String[] getGenotypes(int pPersonSexType) {
	  if(pPersonSexType == 0){
		return mGenotypes;
	  }
	  if(pPersonSexType == 1){
	    return fGenotypes;
	  }
	  else{
	    return unknownGenotypes;
	  }
	}

	public String[] getPhenotypes(int pPersonSexType) {
		String[] phenotypes = {"Unaffected", "Affected"};
		return phenotypes;
	}

	public void addPerson(BayesNetwork bn, Person indiv) {
		try {
		BayesNetworkNode genotypeNode;
		if (bRaceDependent){
		  switch(indiv.getSex()){
		  case 0: genotypeNode = bn.createNode(mGenotypes, mGenotypeFreqs[indexForRace(indiv)]);
		    break;
		  case 1: genotypeNode = bn.createNode(fGenotypes, fGenotypeFreqs[indexForRace(indiv)]);
		    break;
		  default: genotypeNode = bn.createNode(unknownGenotypes, unknownGenotypeFreqs[indexForRace(indiv)]);
		    break;
		  }
		}
		else{
		  switch(indiv.getSex()){
		  case 0: genotypeNode = bn.createNode(mGenotypes, mGenotypeFreqs[0]);
		    System.out.println("Male a freq: " + mGenotypeFreqs[0][1]);
		    break;
		  case 1: genotypeNode = bn.createNode(fGenotypes, fGenotypeFreqs[0]);
		    break;
		  default: genotypeNode = bn.createNode(unknownGenotypes, unknownGenotypeFreqs[0]);
		    break;
		  }
		}
		System.out.println("created genotypeNode");
		double[] tempPhenoFreq = {1.0, 0};
		String[] phenotypes = {"Unaffected", "Affected"};
		BayesNetworkNode phenotypeNode = bn.createNode(phenotypes, tempPhenoFreq);
		bn.createArc(genotypeNode, phenotypeNode);

		// need to set up the expression of phenotype based on genotype
		double[] expressionTableEntry;
		if (bAgeDependentExpression)
			expressionTableEntry = expressionTable[getAgeIndex(indiv)];
		else
			expressionTableEntry = expressionTable[0];
		BayesNetworkNode[] parentNodes = {genotypeNode};
		double[] expressionProbs = new double[2];
		String[] parentValues = new String[1];
		int start;
		int end;
		switch(indiv.getSex()){
		case 0: start = unknownGenotypes.length-(badAlleles.length+1); end = unknownGenotypes.length-1;
		    break;
		case 1: start = 0; end = unknownGenotypes.length-(badAlleles.length+1)-1;
		    break;
		default: start = 0; end = unknownGenotypes.length-1;
		    break;
		  }
		for(int x = start; x <= end; x++) {
			expressionProbs[1] = expressionTableEntry[x];
			expressionProbs[0] = 1-expressionProbs[1];
			parentValues[0] = unknownGenotypes[x];
			phenotypeNode.changeProbFunction(parentNodes, parentValues, expressionProbs);
		}

		indiv.setGenotypeNode(genotypeNode);
		indiv.setPhenotypeNode(phenotypeNode);
		}
		catch(ProbabilityException e) {
			System.err.println("Disease add person had a prob exception");
		}
		catch(InvalidValueException e) {
			System.err.println("Disease add person had a value exception");
		}
		catch(InvalidArcException e) {
			System.err.println("Disease add person had a arc exception");
		}
	}

	private int getAgeIndex(Person p) {
		return p.getAge() / 10;
	}

	public void removePerson(BayesNetwork bn, Person indiv) {
		// make sure no children or parents of genotype
		BayesNetworkNode genotypeNode = indiv.getGenotypeNode();
		Enumeration enumParents = genotypeNode.getParents();
		if (enumParents.hasMoreElements())
			return;  // don't do anything if has indiv has parents
		BayesNetworkNode phenotypeNode = indiv.getPhenotypeNode();
		Enumeration enumChildren = genotypeNode.getChildren();
		// must have at least one child, phenotype node
		enumChildren.nextElement();
		if (enumChildren.hasMoreElements())
			return;  // indiv has children

		bn.removeNode(phenotypeNode);
		bn.removeNode(genotypeNode);

		indiv.setPhenotypeNode(null);
		indiv.setGenotypeNode(null);
	}
	

	public void addChild(BayesNetwork bn, Person child, Person parent1, Person parent2) {
		// try making links
		BayesNetworkNode childNode = child.getGenotypeNode();
		Person pParent1 = parent1;
		Person pParent2 = parent2;
		if(parent1.getSex() == 0){  // Put female first
		  pParent1 = parent2;
		  pParent2 = parent1;
		}
		BayesNetworkNode parentNode1 = pParent1.getGenotypeNode();
		BayesNetworkNode parentNode2 = pParent2.getGenotypeNode();
		try {
			bn.createArc(parentNode1, childNode);
			bn.createArc(parentNode2, childNode);
		}
		catch (InvalidArcException e) {
			System.err.println("Disease.addChild had an arc exception");
			return;  // created a loop - time to bail
		}

		// need to reinit child genotypenode
		try {
			BayesNetworkNode[] bnns = {parentNode1, parentNode2};
			int theIndexInheritenceTable = 0;
			String[] parentValues = new String[2];
			int start;
			int end;
			int multiplier;
			double[][] sexInheritenceTable;
			switch(child.getSex()){
			case 0: start = unknownGenotypes.length-(badAlleles.length+1);
			  end = unknownGenotypes.length-1;
			  multiplier = 2;
			  break;
			case 1: start = 0;
			  end = unknownGenotypes.length-(badAlleles.length+1)-1;
			  multiplier = 2;
			  break;
			default: start = 0;
			  end = unknownGenotypes.length-1;
			  multiplier = 1;
			  break;
			}
			for(int x = 0; x < fGenotypes.length; x++) {
				for (int y = 0; y < mGenotypes.length; y++) {
					parentValues[0] = fGenotypes[x];
					parentValues[1] = mGenotypes[y];
					double[] cInheritenceTable = new double[end-start+1];
					for(int z=start; z <= end; z++){
					  cInheritenceTable[z-start] = unknownInheritenceTable[theIndexInheritenceTable][z]*multiplier;
					}
					childNode.changeProbFunction(bnns, parentValues, cInheritenceTable);
					theIndexInheritenceTable++;
				}
			}
		}
		catch(ProbabilityException e) {
			System.err.println("Disease add child had an exception");
		}
		catch(InvalidValueException e) {
			System.err.println("Disease add child had a value exception.");
		}
	}

	public void removeChild(BayesNetwork bn, Person child, Person parent1, Person parent2) {
		// remove the links if they exist
		BayesNetworkNode childNode = child.getGenotypeNode();
		BayesNetworkNode parentNode1 = parent1.getGenotypeNode();
		BayesNetworkNode parentNode2 = parent2.getGenotypeNode();

		bn.removeArc(parentNode1, childNode);
		bn.removeArc(parentNode2, childNode);

		// init the genotype node again
		try {
		  if(bRaceDependent){
		    switch(child.getSex()){
		    case 0: 
		      childNode.changeProbFunction(null, null, mGenotypeFreqs[indexForRace(child)]);
		      break;
		    case 1: childNode.changeProbFunction(null, null, fGenotypeFreqs[indexForRace(child)]);
		      break;
		    default: childNode.changeProbFunction(null, null, unknownGenotypeFreqs[indexForRace(child)]);
		      break;
		    }
		  }
		  else{
		    switch(child.getSex()){
		    case 0: 
			childNode.changeProbFunction(null, null, mGenotypeFreqs[0]);
			break;
		    case 1: childNode.changeProbFunction(null, null, fGenotypeFreqs[0]);
		      break;
		    default: childNode.changeProbFunction(null, null, unknownGenotypeFreqs[0]);
		      break;
		    }
		  }
		}
		catch(ProbabilityException e) {
			System.err.println("Disease remove child had an exception");
		}
		catch(InvalidValueException e) {
			System.err.println("Disease remove child had a value exception");
		}
	}

	public String toString() {
		String s = "NewXlinkedDisease\n";
		s += name + "\n";
		s += normalAllele + "\n";
		s += badAlleles.length + "\n";
		for (int x= 0; x < badAlleles.length; x++)
			s += badAlleles[x] + "\n";

		if (bHasMutationRate) {
			s += "Mutation Rates\n";
			for (int x = 0; x < mutationRates.length; x++)
				s += mutationRates[x] + "\n";
		}

		if (bAgeDependentExpression)
			s += "Age dependent\n";
		else
			s += "Not age dependent\n";
		for (int x= 0; x < expressionTable.length; x++) {
			for (int y = 0; y < expressionTable[0].length; y ++)
				s += expressionTable[x][y] + "\n";
		}
		
		
		if (bRaceDependent) {
			s += "Race dependent\n";
			s += vRaces.size() + "\n";
			for (int x=0; x < vRaces.size(); x++)
				s += vRaces.elementAt(x) + "\n";
		}

		s += "Freq rates\n";
		for (int x = 0; x < alleleFreqRates.length; x++) {
			for (int y = 0; y < alleleFreqRates[0].length; y++)
				s += alleleFreqRates[x][y] + "\n";
		}

		return s;
	}

	private int indexForRace(Person p) {
	// requires that person has a valid race 
		String sRace = p.getRace();
		for(int x = 0; x < vRaces.size(); x++)
			if (sRace.equals(vRaces.elementAt(x)))
			return x;

		return 0;  // this should never happen
	}

	public boolean hasMutationRate() {
		return bHasMutationRate;
	}

	public boolean isRaceDependent() {
		return bRaceDependent;
	}

	public String[] getRaces() {
		if (vRaces == null)
			return null;
		else {
			String[] races = new String[vRaces.size()];
			vRaces.copyInto(races);
			return races;
		}
	}

	public boolean isAgeDependent() {
		return bAgeDependentExpression;
	}

	public String getNormalAllele() {
		return normalAllele;
	}

	public String[] getDefectiveAlleles() {
		return badAlleles;
	}

	public boolean isSexLinked() {
		return false;
	}

	public double[] getMutationRates() {
		return mutationRates;
	}

	public double[][] getFrequencyRates() {
		return alleleFreqRates;
	}

	public double[][] getExpressionRates(int iSexType) {	  
	  return expressionTable;
	}


}
