//package PedigreeCanvas;

//import Pedigree.*;

// this is an abstract class.
// It is subclassed by PedCanMarriageLine and PedCanChildLine
// It allows for drawing a continuous line consisting of connected line segments

import java.awt.*;
import java.util.*;

public abstract class PedCanLine {



	Vector points;   // a collection of the connecting points in order of the line
	int iType;       // either CHILD or MARRIAGE or TEST
	int lineWidth;


	static final int MARRIAGE = 0;
	static final int CHILD = 1;
	static final int TEST = 2;

	// CONTRUCTOR

	public PedCanLine(int type, int lineWidth) {
	// requires: type is either PedCanLine.MARRIAGE or PedCanLine.CHILD or PedCanLine.TEST
	// effects:  creates a line of width lineWidth of the type.  The lineWidth is not
	//			 visually apparent, and only factors into the isOneLine method.
		if (type == PedCanLine.MARRIAGE)
			iType = PedCanLine.MARRIAGE;
		else if (type == PedCanLine.CHILD)
			iType = PedCanLine.CHILD;
		else 
			iType = PedCanLine.TEST;

		this.lineWidth = lineWidth;
		points = new Vector();
	}


	// METHODS

	public boolean isChildLine() {
	// effects: Returns true if this line is a child line
		return (iType == PedCanLine.CHILD);
	}

	public boolean isMarriageLine() {
	// effects: Returns true if this is a marriage line
		return (iType == PedCanLine.MARRIAGE);
	}

	public boolean isTestLine() {
	// effects: Returns true if this is a test line
		return (iType == PedCanLine.TEST);
	}
	
	public void clear() {
	// effects: empty out the collection of points that this object represents.
		points.removeAllElements();
	}

	public void addPoint(Point pt) {
	// effects: Add this point the collection of points, extending the line that this represents
		points.addElement(pt);
	}

	public Enumeration getPoints() {
	// effects: Returns all the points in the order they were added.
		return points.elements();
	}


	public boolean isOnLine(Point pt) {
	// effects: determine if a point if on the line using the line width
		Enumeration enum = points.elements();
		Point startOfSeg, endOfSeg;

		if (enum.hasMoreElements())
			startOfSeg = (Point) enum.nextElement();
		else
			return false;

		while (enum.hasMoreElements()) {
			endOfSeg = (Point) enum.nextElement();
			if (ptSegDist((double)startOfSeg.x, (double)startOfSeg.y, 
				(double)endOfSeg.x, (double)endOfSeg.y, 
				(double)pt.x, (double)pt.y) < lineWidth / 2)
				return true;
			startOfSeg = endOfSeg;
		}

		return false;
	}

	// this is from the java.swt.geom.Line2D source code
	private double ptSegDist(double X1, double Y1,
				     double X2, double Y2,
				     double PX, double PY) {
	// Adjust vectors relative to X1,Y1
	// X2,Y2 becomes relative vector from X1,Y1 to end of segment
	X2 -= X1;
	Y2 -= Y1;
	// PX,PY becomes relative vector from X1,Y1 to test point
	PX -= X1;
	PY -= Y1;
	double dotprod = PX * X2 + PY * Y2;
	double projlenSq;
	if (dotprod <= 0.0) {
	    // PX,PY is on the side of X1,Y1 away from X2,Y2
	    // distance to segment is length of PX,PY vector
	    // "length of its (clipped) projection" is now 0.0
	    projlenSq = 0.0;
	} else {
	    // switch to backwards vectors relative to X2,Y2
	    // X2,Y2 are already the negative of X1,Y1=>X2,Y2
	    // to get PX,PY to be the negative of PX,PY=>X2,Y2
	    // the dot product of two negated vectors is the same
	    // as the dot product of the two normal vectors
	    PX = X2 - PX;
	    PY = Y2 - PY;
	    dotprod = PX * X2 + PY * Y2;
	    if (dotprod <= 0.0) {
		// PX,PY is on the side of X2,Y2 away from X1,Y1
		// distance to segment is length of (backwards) PX,PY vector
		// "length of its (clipped) projection" is now 0.0
		projlenSq = 0.0;
	    } else {
		// PX,PY is between X1,Y1 and X2,Y2
		// dotprod is the length of the PX,PY vector
		// projected on the X2,Y2=>X1,Y1 vector times the
		// length of the X2,Y2=>X1,Y1 vector
		projlenSq = dotprod * dotprod / (X2 * X2 + Y2 * Y2);
	    }
	}
	// Distance to line is now the length of the relative point
	// vector minus the length of its projection onto the line
	// (which is zero if the projection falls outside the range
	//  of the line segment).
	return Math.sqrt(PX * PX + PY * PY - projlenSq);
    }

	
	public void paint(Graphics g) {
	// requires:  g is not null
	// effects: Draw this line on G using the current color, does not take width into account
	//          by drawing lines to connect the points in the order they were added.
		Enumeration enum = points.elements();
		Point startOfSeg, endOfSeg;

		if (enum.hasMoreElements())
			startOfSeg = (Point) enum.nextElement();
		else
			return;

		while (enum.hasMoreElements()) {
			endOfSeg = (Point) enum.nextElement();
			g.drawLine(startOfSeg.x, startOfSeg.y, endOfSeg.x, endOfSeg.y);
			startOfSeg = endOfSeg;
		}
	}

	public String toString() {
		Enumeration enum = points.elements();
		String s = new String();
		while(enum.hasMoreElements())
			s = s + (Point)enum.nextElement();
		return s;
	}


}

