//package PedigreeCanvas;

//import Pedigree.*;
//import Person.*;

// A PedCanPerson is a object representing a Person on a PedigreeCanvas.
// It includes the person's location and all lines he is connected to.
// The location is the upper left point of its rectangluar area.

import java.awt.*;
import java.util.*;


public class PedCanPerson {

	// local state
	Person person;
	Point location;
	Vector pcLines;
	int personWidth;

	// constructor

	public PedCanPerson(Person person, Point location, int width) {
	// requires: person and location are not null
	// effects:  Create a PedCanPerson for person at location "location", with his pixel width = width
		this.person = person;
		this.location = location;
		personWidth = width;
		pcLines = new Vector();
	}

	// methods
	public Person getPerson() {
		return person;
	}

	public Point getLocation() {
		return location;
	}

	public void setLocation(Point newLocation) {
		location = newLocation;
	}

	public void addLine(PedCanLine pcLine) {
		pcLines.addElement(pcLine);
	}

	public void removeLine(PedCanLine pcLine) {
		pcLines.removeElement(pcLine);
	}

	public Enumeration getLines() {
		return pcLines.elements();
	}


	public void paint(Graphics g) {
		if (person.getSex() == Person.MALE)
			g.fillRect(location.x, location.y, personWidth - 1, personWidth - 1);
		else if (person.getSex() == Person.FEMALE)
			g.fillOval(location.x, location.y, personWidth - 1, personWidth - 1);
		else {
			// draw a triangle for baby sex
			int[] x = new int[3];
			int[] y = new int[3];
			x[0] = location.x + personWidth/2;
			y[0] = location.y;
			x[1] = location.x + personWidth;
			y[1] = location.y + personWidth;
			x[2] = location.x;
			y[2] = location.y + personWidth;
			g.fillPolygon(x, y, 3);
		}

		// draw the name too
		FontMetrics fm = g.getFontMetrics();
		int nameWidth = fm.stringWidth(person.getName());
		g.drawString(person.getName(), location.x + ((personWidth - nameWidth)/2),
									   location.y + personWidth + fm.getHeight());
	}


	// see if a hit occurred in a figure
	public boolean isOnPerson(int x, int y) {
	// effects:  Determines whether or not this point(x,y) on other the painted portion of this object.
	//			 Being a the person's name does not constitute a hit.
		if (person.getSex() == Person.MALE) {
			if ((x > location.x) && (x < location.x + personWidth) &&
				(y > location.y) && (y < location.y + personWidth))
				return true;
			else
				return false;
		}
		else if (person.getSex() == Person.FEMALE) {
			Point hitPoint = new Point(x,y);
			Point center = new Point(location.x + personWidth /2, location.y + personWidth / 2);
			// get the distance
			double PX = hitPoint.x - center.x;
			double PY = hitPoint.y - center.y;
			
			if (Math.sqrt(PX * PX + PY * PY) > personWidth / 2)
				return false;
			else return true;
		}
		else {  // this is for the unknown baby sex
			// get the three points of triangle
			int[] aX = new int[3];
			int[] aY = new int[3];
			aX[0] = location.x + personWidth/2;
			aY[0] = location.y;
			aX[1] = location.x + personWidth;
			aY[1] = location.y + personWidth;
			aX[2] = location.x;
			aY[2] = location.y + personWidth;
			Polygon p = new Polygon(aX, aY, 3);
			return p.contains(x,y);
		}
	}

	public String toString() {
		return "PedigreeCanvasPerson " + getLocation();
	}
}
