//package PedigreeCanvas;

//import Pedigree.*;
//import Disease.*;
//import GeneticScreen.*;
//import Person.*;
//import GenExceptions.*;

import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.net.URL;

public class PedigreeCanvas extends JPanel 
							implements MouseListener, MouseMotionListener { // implements Scrollable {

// constants
	final int borderSize = 18;
	final int personWidth = 40;
	final int gridSize = 76;
	final int lineWidth = 10;
	final int geneticScreenWidth = 30;
	// NOTE: some of this information is just for convience, since gridSize = personWidth + 2 * borderSize

	final Color backgroundColor = Color.white;
	final Color unaffectedColor = Color.blue;
	final Color affectedColor = Color.red;
	final Color testColor = Color.green;
	final Color lineColor = Color.black;
	final Color selectedColor = Color.gray;


// local state
	Pedigree pedigree;
	Disease disease;
	String drawMode;  // DrawMode can be "Pointer", "Male", "Female", "Baby", "Marriage", "Child", "Delete", "Test" 
	boolean bModified;

	Vector pcPersons;
	Vector pcLines;
	Vector pcGeneticScreens;
	Vector geneticDescriptors;


	Object selectedFigure;  // currently selected person or geneticScreen
	Point selectedSource;		  // where the selected person was when he was selected (used for undoing moves)
	PedCanLine drawLine;

	int IDofNextPerson;

	JFrame f;  // this is the parent frame which is used to create dialog boxes
	Dimension currentSize;  // this is used for scroll bar enabling

//constructor
	public PedigreeCanvas(JFrame f, Disease d) {
		super();

		this.f = f;

		disease = d;

		pedigree = new Pedigree(d);

		drawMode = "Male";
		pcPersons = new Vector();
		pcLines = new Vector();
		pcGeneticScreens = new Vector();
		geneticDescriptors = new Vector();
		selectedFigure = null;
		drawLine = null;

		addMouseListener(this);
		addMouseMotionListener(this);

		IDofNextPerson = Integer.MIN_VALUE;
		bModified = true;

		currentSize = new Dimension(0,0);
		setBackground(backgroundColor);
		//tForeground(Color.red);

		// do class loading to speed things up for user
		performanceEnhancement();
	}
	
	public PedigreeCanvas(FileReader filereader, JFrame f) throws IOException {
		// format of a pedigree file is :
		// <disease>
		// "*End of Disease*"
		// <IDofNextPerson>
		// <person location> followed by the person.toString()
		// more people
		// "Lines"
		//  marriage pcLines
		//  child pcLines

		super();

		this.f = f;

		BufferedReader in = new BufferedReader(filereader);

		// get disease from file using disease file io
		try {
			String sLine;
			String sDisease = new String();
			while (in.ready()) {
				sLine = in.readLine();
				if (sLine.equals("*End Of Disease*"))
					break;
				sDisease += sLine + "\n";				
			}
			disease = Disease.createDisease(sDisease);
		}
		catch(NoSuchElementException e) {
			System.err.println("unexpected end of file");
			throw new IOException();
		}
		catch(InvalidDiseaseException e) {
			System.err.println("invalid disease format");
			throw new IOException();
		}

		// getting id
		try {
			String sLine = in.readLine();
			if (sLine != null)
				IDofNextPerson = Integer.valueOf(sLine).intValue();
			else
				throw new IOException();
		}
		catch(NumberFormatException e) {
			throw new IOException();
		}

		
		drawMode = "Male";
		bModified = false;
		
		selectedFigure = null;
		drawLine = null;

		addMouseListener(this);
		addMouseMotionListener(this);

		currentSize = new Dimension(0,0);
		setBackground(backgroundColor);
	//etForeground(Color.red);

		pcPersons = new Vector();
		pcLines = new Vector();
		pcGeneticScreens = new Vector();
		geneticDescriptors = new Vector();
		pedigree = new Pedigree(disease);
		
		// construct pedigree and pcLines and pcPersons from file using add person ...
		// get all the pcPerson from file
		try {
			String sLocation;
			while(in.ready() && !(sLocation = in.readLine()).equals("Lines")) {
				StringTokenizer st = new StringTokenizer(sLocation);
				int newX = Integer.valueOf(st.nextToken()).intValue();
				int newY = Integer.valueOf(st.nextToken()).intValue();
				String sPerson = new String();
				// last two lines are genotype and phentyope which are read later
				for(int x = 0; x < Person.numLines - 2; x++) {
					if (in.ready())
						sPerson += in.readLine() + "\n";
					else
						throw new IOException();
				}
				Person nP = new Person(sPerson, disease);
				// now add new person
				pedigree.addPerson(nP);
				Point gridPoint = getGridPoint(newX, newY);
				PedCanPerson newpcPerson = new PedCanPerson(nP, new Point(gridPoint.x + borderSize,
																		  gridPoint.y + borderSize), personWidth);
				pcPersons.addElement(newpcPerson);
				updateSize(newpcPerson.getLocation(), true);

				// now set person's genotype and phenotype
				String sObs;
				if (in.ready())
					sObs = in.readLine() + "\n";
				else
					throw new IOException();
				if (in.ready())
					sObs += in.readLine();
				else 
					throw new IOException();
				st = new StringTokenizer(sObs);
				String sTemp = st.nextToken();
				if (!sTemp.equals("GenotypeObservation"))
					throw new MalformedPersonException();
				sTemp = st.nextToken();
				if (!sTemp.equals("unknown"))
					nP.setGenotype(sTemp);
				sTemp = st.nextToken();
				if (!sTemp.equals("PhenotypeObservation"))
					throw new MalformedPersonException();
				sTemp = st.nextToken();
				nP.setPhenotype(sTemp);
			}
		}
		catch(MalformedPersonException e) {
			System.err.println("Malformed person exception");
			throw new IOException();
		}
		catch(NumberFormatException e) {
			System.err.println("number format exception");
			throw new IOException();
		}
		catch(NoSuchElementException e) {
			System.err.println("unexpected end of file");
			throw new IOException();
		}
		catch(ProbabilityException e) {
			System.err.println("probability exception");
			throw new IOException();
		}
		catch(InvalidValueException e) {
			System.err.println("invalid value exception");
			throw new IOException();
		}

		// all the persons have been loaded ... now time to get all the lines
		if (!in.ready())  // there are no lines
			return;

		try {
			String sLine = null;
			while (in.ready()) {
				sLine = in.readLine();
				StringTokenizer st = new StringTokenizer(sLine);
				String sTemp = st.nextToken();
				if (!sTemp.equals("MarriageLine"))
					break;  // either beginning of child lines or an error
				int IDperson1 = Integer.valueOf(st.nextToken()).intValue();
				int IDperson2 = Integer.valueOf(st.nextToken()).intValue();
				PedCanPerson p1 = getPCPersonFromID(IDperson1);
				PedCanPerson p2 = getPCPersonFromID(IDperson2);
				if ((p1 == null) || (p2 == null))
					throw new IOException();
				addMarriage(p1, p2);
			}
			while(true) {
				// first child line is already in sLine
				StringTokenizer st = new StringTokenizer(sLine);
				String sTemp = st.nextToken();
				if (!sTemp.equals("ChildLine"))
					throw new IOException();
				int IDchild = Integer.valueOf(st.nextToken()).intValue();
				int IDparent1 = Integer.valueOf(st.nextToken()).intValue();
				int IDparent2 = Integer.valueOf(st.nextToken()).intValue();
				PedCanPerson child = getPCPersonFromID(IDchild);
				PedCanPerson parent1 = getPCPersonFromID(IDparent1);
				PedCanPerson parent2 = getPCPersonFromID(IDparent2);
				if ((child == null) || (parent1 == null) || (parent2 == null))
					throw new IOException();
				addChild(parent1, parent2, child);
				if (in.ready())
					sLine = in.readLine();
				else
					break;
			}
		}
		catch(NumberFormatException e) {
			throw new IOException();
		}
		catch(NoSuchElementException e) {
			throw new IOException();
		}

		// do class loading to speed things up for user
		performanceEnhancement();
	}

	private void performanceEnhancement() {
		// effects: forces the loading of some classes so that the user doesn't have long pauses
		//			when using the pedigree

		// need to save load state so that I can preserve it
		int oldID = IDofNextPerson;
		boolean oldbModified = bModified;

		boolean bAddedPerson = false;
		if (pcPersons.isEmpty()) {
			addPerson(0, 0, "Male");  // this will force loading of bayesnet stuff
			bAddedPerson = true;
		}

		updatePersonInferences();	// this will finish loading all Bayes net stuff

		PedCanPerson pcPerson = (PedCanPerson)pcPersons.elementAt(0);
		PersonDialog pd = new PersonDialog(f, pcPerson.getPerson());  // speed loading of this dialog

		if (bAddedPerson)
			deletePerson(pcPerson);  // this will be the person we added above

		IDofNextPerson = oldID;
		bModified = oldbModified;
	}

	public void save(FileWriter fr) {
		// effects: Saves this pedcan in the file fr using the format:
		// <disease>
		// "*End of Disease*"
		// <IDofNextPerson>
		// <person location> followed by the person.toString()
		// more people
		// "Lines"
		//  marriage pcLines
		//  child pcLines

		PrintWriter pr = new PrintWriter(fr);

		// add disease to file
		pr.print(disease.toString());  // disease toString always end in a "\n"
		pr.println("*End Of Disease*");

		pr.println(IDofNextPerson);
		for (int x=0; x < pcPersons.size(); x++) {
			PedCanPerson pcPerson = (PedCanPerson)pcPersons.elementAt(x);
			pr.println(Integer.toString(pcPerson.getLocation().x) + " " + Integer.toString(pcPerson.getLocation().y));
			pr.println(pcPerson.getPerson().toString());
		}
		pr.println("Lines");
		// add marriage lines first
		for(int x=0; x < pcLines.size(); x++) {
			PedCanLine pcLine = (PedCanLine)pcLines.elementAt(x);
			if (pcLine.isMarriageLine()) 
				pr.println(pcLine.toString());
		}
		// now add child lines
		for(int x=0; x < pcLines.size(); x++) {
			PedCanLine pcLine = (PedCanLine)pcLines.elementAt(x);
			if (pcLine.isChildLine()) 
				pr.println(pcLine.toString());
		}
		pr.close();
		bModified = false;
	}


// methods
	public void setMode(String editMode) {
		// requires: editMode is either "Pointer", "Male", "Female", "Baby", "Marriage", "Child", "Delete", "Test"
		// effects: set the new edit mode
		drawMode = editMode;
	}

	public void updatePersonInferences() {
		try {
			pedigree.updatePersonInferences();
		}
		catch(ProbabilityException e) {
			JOptionPane.showMessageDialog(f, "This pedigree contains inconsistant observations which make "+
											 "inferences impossible to determine.",
					"Inferences Error", JOptionPane.INFORMATION_MESSAGE);
		}
	}

	// get the person who has this ID, returns null if no such person exists
	private PedCanPerson getPCPersonFromID(int ID) {
		PedCanPerson pcPerson;
		for(int x= 0; x < pcPersons.size(); x++) {
			pcPerson = (PedCanPerson)pcPersons.elementAt(x);
			if (pcPerson.getPerson().getID() == ID)
				return pcPerson;
		}
		return null;
	}
			

	public void close() throws NotSavedException {
		// effects: Closes an resources this pedcan has.
		//   Throws a NotSavedException if this pedcan has changes not yet saved to a file
		if (bModified)
			throw new NotSavedException();
	}

	// painting
	public void paintComponent(Graphics g) {

		super.paintComponent(g); 

		// iterate through the list of individuals and draw them
		g.setColor(unaffectedColor);
		Enumeration enumpcPersons = pcPersons.elements();
		while (enumpcPersons.hasMoreElements()) {
			PedCanPerson pcPerson = (PedCanPerson) enumpcPersons.nextElement();
			if (pcPerson.getPerson().isAffected()) {
				g.setColor(affectedColor);
				pcPerson.paint(g);
				g.setColor(unaffectedColor);
			}
			else
				pcPerson.paint(g);
		}


		// paint all the lines
		g.setColor(lineColor);
		Enumeration enumpcLines = pcLines.elements();
		while (enumpcLines.hasMoreElements()) {
			PedCanLine pcLine = (PedCanLine)enumpcLines.nextElement();
			pcLine.paint(g);
		}
	 
		// paint the drawline
		if (drawLine != null)
			drawLine.paint(g);

		// paint all the test objects last so they are on top
		g.setColor(testColor);
		Enumeration enumpcGeneticScreens = pcGeneticScreens.elements();
		while (enumpcGeneticScreens.hasMoreElements())
			((PedCanGeneticScreen)enumpcGeneticScreens.nextElement()).paint(g);

		// repaint the selected object
		if (selectedFigure != null) {
			g.setColor(selectedColor);
			try {
				((PedCanPerson)selectedFigure).paint(g);
			}
			catch(ClassCastException e) {
				((PedCanGeneticScreen)selectedFigure).paint(g);
			}
		}

	
    }

	
	void updateSize(Point p, boolean bpersonIsAdded) {
		// requires:  if pcPerson is to be deleted, then he has already been deleted
		// effects:  if someone is being added at point p if bpersonisAdded is true
		//           if someone is being moved from or deleted from p, bpersonisadded is false
		//           This method sets the preferred size based on this event

		boolean newSize = false;

		if (bpersonIsAdded) {  // we are adding a person
			if ((p.x + gridSize) > currentSize.width) {
				currentSize.width = p.x + gridSize;
				newSize = true;
			}
			if ((p.y + gridSize) > currentSize.height) {
				currentSize.height = p.y + gridSize;
				newSize = true;
			}
		}
		else if (((p.x + gridSize) >= currentSize.width) || 
			     ((p.y + gridSize) >= currentSize.height)) {
			newSize = true;
			currentSize.width = currentSize.height = 0;
			for(int x=0; x < pcPersons.size(); x++) {
				PedCanPerson nextPerson = (PedCanPerson)pcPersons.elementAt(x);
				if ((nextPerson.getLocation().x + gridSize) > currentSize.width)
					currentSize.width = nextPerson.getLocation().x + gridSize;
				if ((nextPerson.getLocation().y + gridSize) > currentSize.height)
					currentSize.height= nextPerson.getLocation().y + gridSize;
			}
		}

		if (newSize) {
			this.setPreferredSize(currentSize);
			this.revalidate();
		}
	}


	// mouse events
	public void mousePressed(MouseEvent e) {
		if (selectedFigure != null) {
			deselectFigure();   // unselect old things if any
			repaint();
		}

		PedCanGeneticScreen pcGS = getPedCanGeneticScreenHit(e.getX(), e.getY());
		if (pcGS != null) {
			if (SwingUtilities.isRightMouseButton(e)) {
					handlePopupMenu(pcGS);
					selectFigure(pcGS);
					repaint();
			}
			else if (drawMode == "Delete") {
				deleteGeneticScreen(pcGS);
				repaint();
			}
			else if ((drawMode != "Marriage") && (drawMode != "Child")) {
				selectFigure(pcGS);
				repaint();
			}
			return;
		}

		PedCanPerson pcPersonInGrid =getPersonInGridHit(e.getX(), e.getY());

		if (pcPersonInGrid == null) { // this is an empty grid
			if ((drawMode == "Male") || (drawMode == "Female") || (drawMode == "Baby")) {
				addPerson(e.getX(), e.getY(), drawMode);
				repaint();
			}
			else if (drawMode == "Delete") {
				PedCanLine pcLineHit = getLineHit(e.getPoint());
				if (pcLineHit != null) {
					deleteLine(pcLineHit);
					repaint();
				}
			}
		}
		else {  // someone in the grid hit
			if (pcPersonInGrid.isOnPerson(e.getX(), e.getY())) {  // hit someone
				if (SwingUtilities.isRightMouseButton(e)) {
					handlePopupMenu(pcPersonInGrid);
					selectFigure(pcPersonInGrid);
				}
				else if (drawMode == "Delete") {
					deletePerson(pcPersonInGrid);
					repaint();
				}
				else if (drawMode == "Test") {
					addGeneticScreen(pcPersonInGrid);
					repaint();
				}				
				else {
					selectFigure(pcPersonInGrid);
					// need to create a line ... since it is only used for drawing, it doesn't 
					// matter what type it is and what it contains
					if ((drawMode == "Marriage") || (drawMode == "Child"))
						drawLine = new PedCanMarriageLine((PedCanPerson)selectedFigure, (PedCanPerson)selectedFigure, lineWidth);
					repaint();
				}
			}
			else if (drawMode == "Delete") {
				PedCanLine pcLineHit = getLineHit(e.getPoint());
				if (pcLineHit != null) {
					deleteLine(pcLineHit);
					repaint();
				}
			}
		}
	}


    public void mouseReleased(MouseEvent e) {
		if (selectedFigure != null) {
			try {
				PedCanGeneticScreen pcGS = (PedCanGeneticScreen)selectedFigure;
			//	pcGS.setLocation(e.getPoint());
			//	constructLine(pcGS.getPedCanTestLine());
				repaint();
				return;
			}
			catch(ClassCastException cce) {
			}
			// the selected Figure must be a PedCanPerson
			
			if ((drawMode == "Male") || (drawMode == "Female") || (drawMode == "Baby") || (drawMode == "Pointer")) {
				PedCanPerson pcPersonInGrid =getPersonInGridHit(e.getX(), e.getY());
				if (pcPersonInGrid == null) { // this is an empty grid
					// move the person
					Point gridPoint = getGridPoint(e.getX(), e.getY());
					// move the selected figure to this new location
					movePerson((PedCanPerson)selectedFigure, new Point(gridPoint.x + borderSize, gridPoint.y + borderSize));
					// now need to updatesize
					if ((gridPoint.x > selectedSource.x) || (gridPoint.y > selectedSource.y))
						updateSize(((PedCanPerson)selectedFigure).getLocation(), true);
					else
						updateSize(selectedSource, false);
					bModified = true;
					repaint();
				}
				else {
					// released in a bad place, need to move it back
					movePerson((PedCanPerson)selectedFigure, selectedSource);
					repaint();
				}
			}
			else if (drawMode == "Marriage") {
				// need to make sure we are on somebody
				PedCanPerson pcPersonInGrid =getPersonInGridHit(e.getX(), e.getY());
				if ((pcPersonInGrid != null) && (pcPersonInGrid.isOnPerson(e.getX(), e.getY())))
					addMarriage((PedCanPerson)selectedFigure, pcPersonInGrid);

				drawLine = null;
				repaint();
			}
			else if (drawMode == "Child") {
				PedCanPerson pcPersonInGrid =getPersonInGridHit(e.getX(), e.getY());
				if ((pcPersonInGrid != null) && (pcPersonInGrid.isOnPerson(e.getX(), e.getY()))) {
					// if this person is married to only one other person, then we have a child
					Enumeration enumLines = pcPersonInGrid.getLines();
					boolean onlyOneSpouse = true;
					PedCanMarriageLine pcMLine = null;
					while((onlyOneSpouse) && (enumLines.hasMoreElements())) {
						PedCanLine pcLine = (PedCanLine)enumLines.nextElement();
						if (pcLine.isMarriageLine()) {
							if (pcMLine == null)
								pcMLine = (PedCanMarriageLine)pcLine;
							else
								onlyOneSpouse = false;
						}
					}
					if ((onlyOneSpouse) && (pcMLine != null)) {
						Enumeration enumCouple = pcMLine.getCouple();
						addChild((PedCanPerson)enumCouple.nextElement(), (PedCanPerson)enumCouple.nextElement(),
							(PedCanPerson)selectedFigure);
					}
						
				}
				else {  // may be adding child using a marriage line
					PedCanLine line = getLineHit(e.getPoint());
					if ((line != null) && (line.isMarriageLine())) {
						PedCanMarriageLine pcMLine = (PedCanMarriageLine)line;
						Enumeration enumCouple = pcMLine.getCouple();
						addChild((PedCanPerson)enumCouple.nextElement(), 
							     (PedCanPerson)enumCouple.nextElement(), (PedCanPerson)selectedFigure);
					}
				}

				drawLine = null;
				repaint();
			}
		}
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

	// mouse motion events
	public void mouseDragged(MouseEvent e) {
		if (selectedFigure != null) {
			try {
				PedCanGeneticScreen pcGS = (PedCanGeneticScreen)selectedFigure;
				pcGS.setLocation(e.getPoint());
				constructLine(pcGS.getPedCanTestLine());
				repaint();
				return;
			}
			catch(ClassCastException cce) {
			}

			// the selected Figure must be a PedCanPerson

			if ((drawMode == "Male") || (drawMode == "Female") || (drawMode == "Baby") || (drawMode == "Pointer")) {
				Point mousePoint = e.getPoint();
				movePerson((PedCanPerson)selectedFigure, new Point(mousePoint.x - personWidth/2, mousePoint.y - personWidth/2));
				repaint();
			}
			else if ((drawMode == "Child") || (drawMode == "Marriage")) {
				drawLine.clear();
				drawLine.addPoint(new Point(selectedSource.x + personWidth/2, selectedSource.y + personWidth/2));
				drawLine.addPoint(e.getPoint());
				repaint();
			}
		}				
	}

	public void mouseMoved(MouseEvent e)  {
	}
		

	private void updatePersonLines(PedCanPerson pcPerson) {
		// effects: Constuct this person's marriage and child lines
		Enumeration enum = pcPerson.getLines();
		// this only works becuase a child line's marriage line always comes first
		while (enum.hasMoreElements())
			constructLine((PedCanLine)enum.nextElement());
	}

	// set up the points in a line
	private void constructLine(PedCanLine pcLine) {
		if (pcLine.isMarriageLine()) {
			PedCanMarriageLine pcMLine = (PedCanMarriageLine)pcLine;
			Enumeration enum = pcMLine.getCouple();
			PedCanPerson pcpPartner1 = (PedCanPerson)enum.nextElement();
			PedCanPerson pcpPartner2 = (PedCanPerson)enum.nextElement();

			
			Point sourceLoc = pcpPartner1.getLocation();
			Point destLoc = pcpPartner2.getLocation();
			
			int dx =  destLoc.x - sourceLoc.x;
			


			if (dx < 0) {
				// to simplify things, only deal with the case when dx > 0
				// since this is not the case now, make it the case by switching source and dest
				Point tempPoint = sourceLoc;
				sourceLoc = destLoc;
				destLoc = tempPoint;
				dx = -dx;
			}

			int dy =  destLoc.y - sourceLoc.y;

			pcLine.clear();
			Point segPoint = new Point(sourceLoc);
			
			segPoint.translate(personWidth, personWidth/2);
			pcLine.addPoint((Point)segPoint);
			segPoint = new Point(segPoint);

			// lets take care of the vertical dimension first
			if (dy != 0) {
				segPoint.translate(borderSize/2, 0);
				pcLine.addPoint(segPoint);
				segPoint = new Point(segPoint);
				segPoint.translate(0, dy);
				pcLine.addPoint(segPoint);
				segPoint = new Point(segPoint);
			}

			// now lets do the horizontal

			//dx = 0 is a special case which I handle seperate - this can only happen during moves
			if (dx == 0) {
				pcLine.addPoint(new Point(destLoc.x + personWidth, destLoc.y + personWidth/2));
				return;
			}

			// I will go straight until I hit someone and will stay up until I can go straight
			int startUp=0, endUp=0;
			boolean isSomeoneInWay = false;

			Enumeration enumPeople = pcPersons.elements();
			PedCanPerson pcPersonInWay;

			while (enumPeople.hasMoreElements()) {
				pcPersonInWay = (PedCanPerson)enumPeople.nextElement();
				Point personInWayLoc = pcPersonInWay.getLocation();
				// if point is a same y height and in the way
				if ((personInWayLoc.y == (segPoint.y - personWidth /2)) && (personInWayLoc.x > segPoint.x)
					&& (personInWayLoc.x < destLoc.x)) {
					if (!isSomeoneInWay) {
						isSomeoneInWay = true;
						startUp = personInWayLoc.x - 3/2 * borderSize;
						endUp = personInWayLoc.x + gridSize - borderSize / 2;
					}
					else {
						if (personInWayLoc.x < startUp)
							startUp = personInWayLoc.x - 3/2 * borderSize;
						if (personInWayLoc.x > endUp)
							endUp = personInWayLoc.x + gridSize - borderSize / 2;
					}
				}
			}

			if (isSomeoneInWay) {
			// create line to startup point
				segPoint.translate(startUp - segPoint.x, 0);
				pcLine.addPoint(segPoint);
				segPoint = new Point(segPoint);
				segPoint.translate(0,-personWidth / 2 - borderSize);
				pcLine.addPoint(segPoint);
				segPoint = new Point(segPoint);
				segPoint.translate(endUp - segPoint.x, 0);
				pcLine.addPoint(segPoint);
				segPoint = new Point(segPoint);
				segPoint.translate(0, personWidth /2 + borderSize);
				pcLine.addPoint(segPoint);
				segPoint = new Point(segPoint);
			}
			

			// now just complete the line to the partner
			pcLine.addPoint(new Point(destLoc.x, destLoc.y + personWidth/2));

		}
		else if (pcLine.isChildLine()) {
			PedCanChildLine pcCLine = (PedCanChildLine)pcLine;

			// figure out where in marriage line to connect
			// this is done by finding the midpoint of marriage line, rounding
			// this up to a grid boder, and then finding the y-coor of marriage line at this point
			PedCanMarriageLine pcMLine = pcCLine.getMarriageLine();
			Enumeration enumMCouple = pcMLine.getCouple();
			PedCanPerson parent1 = (PedCanPerson)enumMCouple.nextElement();
			PedCanPerson parent2 = (PedCanPerson)enumMCouple.nextElement();

			// the y-value here does not matter, just need to find x midpoint
			Enumeration enumMPoints = pcMLine.getPoints();
			Point startOfMLine = (Point)enumMPoints.nextElement();
			Point endOfMLine = (Point)enumMPoints.nextElement();
			while (enumMPoints.hasMoreElements())
				endOfMLine = (Point)enumMPoints.nextElement();
			Point midPoint = getGridPoint((startOfMLine.x + endOfMLine.x)/2, 0);

			// make sure midPoint.x is acutally between the endpoints of the line, this may not
			// be the case if getGridPoint rounded the miodPoint down past the left endPoint of line
			//  to fix this, add gridSize to the midPoint
			if (midPoint.x < startOfMLine.x)
				midPoint.translate(gridSize, 0);

			// now find y-value of marriage line at x from midPoint
			enumMPoints = pcMLine.getPoints();
			
			Point prevPoint = (Point)enumMPoints.nextElement();
			int y = prevPoint.y;
			while (enumMPoints.hasMoreElements()) {
				Point pt = (Point)enumMPoints.nextElement();
				if (((pt.x < midPoint.x) && (midPoint.x < prevPoint.x)) ||
					((pt.x > midPoint.x) && (midPoint.x > prevPoint.x))) {
					y = pt.y;
					break;
				}
				else
					prevPoint = pt;
			}

			Point destPoint = new Point(midPoint.x, y);

			PedCanPerson pcChild = pcCLine.getChild();
			Point sourcePoint = pcChild.getLocation();

			pcCLine.clear();
			Point segPoint = new Point(sourcePoint.x + personWidth/2, sourcePoint.y);
			pcCLine.addPoint(segPoint);
			segPoint = new Point(segPoint);
			segPoint.translate(0, -borderSize);
			pcCLine.addPoint(segPoint);
			segPoint = new Point(segPoint);

			// take care of dx
			segPoint.translate(destPoint.x - segPoint.x, 0);
			pcCLine.addPoint(segPoint);
			segPoint = new Point(segPoint);

			// take care of dy
			pcCLine.addPoint(destPoint);
		}
		else {
			// this is a GeneticScreen test line
			pcLine.clear();

			PedCanTestLine pcTLine = (PedCanTestLine)pcLine;

			PedCanGeneticScreen pcGS = pcTLine.getPedCanGeneticScreen();
			Point sourcePt = pcGS.getLocation();
			pcTLine.addPoint(new Point(sourcePt.x + geneticScreenWidth/2, sourcePt.y + geneticScreenWidth/2));

			PedCanPerson pcPerson = pcTLine.getPedCanPerson();
			Point destPt = pcPerson.getLocation();
			pcTLine.addPoint(new Point(destPt.x + personWidth/2, destPt.y + personWidth/2));
		}


	}



	// Selection stuff
	private void selectFigure(Object o) {
		selectedFigure = o;
		try {
			selectedSource = ((PedCanPerson)o).getLocation();
			return;
		}
		catch(ClassCastException e) {
		}
			
		selectedSource = ((PedCanGeneticScreen)o).getLocation();
	}

	private void deselectFigure() {
		selectedFigure = null;
	}

	public void handlePopupMenu(PedCanGeneticScreen pcGS) {
		GeneticScreenDialog gsDialog = new GeneticScreenDialog(f, pcGS.getGeneticScreen());
		gsDialog.pack();
		gsDialog.setLocationRelativeTo(f);
		gsDialog.setVisible(true);
		if (gsDialog.didGeneticScreenChange())
			bModified = true;
		repaint();
	}

	public void handlePopupMenu(PedCanPerson pcPerson) {
		Person p = pcPerson.getPerson();
		PersonDialog pDialog = new PersonDialog(f, p);
		pDialog.pack();
		pDialog.setLocationRelativeTo(f);
		pDialog.setVisible(true);
		if (pDialog.didPersonChange()) {
			// this is a hack
			pedigree.clearAllInferences();
			bModified = true;
		}
		repaint();
	}


	// get the person in the grid hit, return null if no one is in this grid
	private PedCanPerson getPersonInGridHit(int x, int y) {
		Point gridPoint = getGridPoint(x,y);

		Enumeration enumpcPersons = pcPersons.elements();
		while (enumpcPersons.hasMoreElements()) {
			PedCanPerson pcPerson = (PedCanPerson) enumpcPersons.nextElement();
			Point personPoint = pcPerson.getLocation();
			if (gridPoint.equals(new Point(personPoint.x - borderSize, personPoint.y - borderSize)))
				return pcPerson;
		}
		return null;
	}

	//get the line hit, if any
	private PedCanLine getLineHit(Point pt) {
		Enumeration enumpcLines = pcLines.elements();
		while (enumpcLines.hasMoreElements()) {
			PedCanLine pcLine = (PedCanLine)enumpcLines.nextElement();
			if (pcLine.isOnLine(pt))
				return pcLine;
		}
		return null;
	}

	// get the grid corresponding to this point
	private Point getGridPoint(int x, int y) {
		return new Point(x - (x % gridSize), y - (y % gridSize));
	}


	// does not repaint or resize the preferred size or update bModified flag
	private void movePerson(PedCanPerson pcPerson, Point newLocation) {
		pcPerson.setLocation(newLocation);
		updatePersonLines(pcPerson);
	}



	// this is called with a location in an empty grid
	private void addPerson(int x, int y, String sPersonType) {
		int sexType;
		if (sPersonType == "Male")
			sexType = Person.MALE;
		else if (sPersonType == "Female")
			sexType = Person.FEMALE;
		else
			sexType = Person.UNKNOWN;
		Point gridPoint = getGridPoint(x,y);
		Person p = new Person(IDofNextPerson++, "Unknown", sexType, disease);
		pedigree.addPerson(p);
		PedCanPerson newpcPerson = new PedCanPerson(p, new Point(gridPoint.x + borderSize, gridPoint.y + borderSize), personWidth);
		pcPersons.addElement(newpcPerson);
		bModified = true;

		updateSize(newpcPerson.getLocation(), true);
	}


	// this will check if a valid marriageConnection can be made, and if so makes it.
	private boolean addMarriage(PedCanPerson person1, PedCanPerson person2) {
		try {
			pedigree.makeMarriage(person1.getPerson(), person2.getPerson());
		}
		catch (InvalidRelationshipException e) {
			return false;
		}
		// this will check for me
		PedCanMarriageLine newMarriageLine = new PedCanMarriageLine(person1, person2, lineWidth);
		person1.addLine(newMarriageLine);
		person2.addLine(newMarriageLine);
		pcLines.addElement(newMarriageLine);
		constructLine(newMarriageLine);
		bModified = true;
		return true;
	}

	// this will check for valid child and make the relationship
	private boolean addChild(PedCanPerson parent1, PedCanPerson parent2, PedCanPerson child) {
		try {
			pedigree.makeChild(child.getPerson(), parent1.getPerson(), parent2.getPerson());
		}
		catch (InvalidRelationshipException e) {
			return false;
		}

		// need to find the marriage line for this child
		PedCanMarriageLine pcMLine = null;
		Enumeration enumLines = parent1.getLines();
		while (enumLines.hasMoreElements()) {
			PedCanLine pcLine = (PedCanLine)enumLines.nextElement();
			if (pcLine.isMarriageLine()) {
				pcMLine = (PedCanMarriageLine)pcLine;
				Enumeration enumCouple = pcMLine.getCouple();
				if (((PedCanPerson)enumCouple.nextElement() == parent2) ||
					((PedCanPerson)enumCouple.nextElement() == parent2))
					break;
			}
		}

		// add this child using the marriage line
		PedCanChildLine newChildLine = new PedCanChildLine(child, pcMLine, lineWidth);
		parent1.addLine(newChildLine);
		parent2.addLine(newChildLine);
		child.addLine(newChildLine);
		pcLines.addElement(newChildLine);
		constructLine(newChildLine);
		bModified = true;
		return true;
	}

	private void deleteLine(PedCanLine pcLine) {
		// if spouse line and we have children, do nothing
		if (pcLine.isMarriageLine()) {
			PedCanMarriageLine pcmLine = (PedCanMarriageLine)pcLine;
			Enumeration enumCouple = pcmLine.getCouple();
			PedCanPerson parent1 = (PedCanPerson)enumCouple.nextElement();
			PedCanPerson parent2 = (PedCanPerson)enumCouple.nextElement();
			Enumeration enumChildren = pedigree.getChildren(parent1.getPerson(), parent2.getPerson());
			if (enumChildren.hasMoreElements())
				return;  // can't remove a marriage line if there are children
			try {
				pedigree.removeMarriage(parent1.getPerson(), parent2.getPerson());
			}
			catch(InvalidRelationshipException e) {
				System.err.println("Unexpected exception in PedigreeCanvas.deleteLine()");
			}
			parent1.removeLine(pcLine);
			parent2.removeLine(pcLine);
			pcLines.removeElement(pcLine);
			bModified = true;
		}
		else if (pcLine.isChildLine()) {  // a child line
			PedCanChildLine pccLine = (PedCanChildLine)pcLine;
			PedCanPerson child = pccLine.getChild();
			PedCanMarriageLine pcmLine = pccLine.getMarriageLine();
			Enumeration enumCouple = pcmLine.getCouple();
			PedCanPerson parent1 = (PedCanPerson)enumCouple.nextElement();
			PedCanPerson parent2 = (PedCanPerson)enumCouple.nextElement();
			pedigree.removeChild(child.getPerson());
			child.removeLine(pcLine);
			parent1.removeLine(pcLine);
			parent2.removeLine(pcLine);
			pcLines.removeElement(pcLine);
			bModified = true;
		}
		else {  // this is a test line
			// because a test can't exist without its test line, we will remove the test too.
			deleteGeneticScreen(((PedCanTestLine)pcLine).getPedCanGeneticScreen());
		}
	}

	private void deletePerson(PedCanPerson pcPerson) {
		Vector personLines = new Vector();
		Enumeration enumLines = pcPerson.getLines();
		while (enumLines.hasMoreElements()) {
			PedCanLine pcLine = (PedCanLine)enumLines.nextElement();
			if (pcLine.isChildLine())
				personLines.addElement(pcLine);
		}
		// remove all child lines
		for(int x=0; x < personLines.size(); x++) 
			deleteLine((PedCanLine)personLines.elementAt(x));
		// remove spouse and lines
		enumLines = pcPerson.getLines();
		personLines.removeAllElements();
		while (enumLines.hasMoreElements())
			personLines.addElement(enumLines.nextElement());
		for(int x=0; x < personLines.size(); x++) 
			deleteLine((PedCanLine)personLines.elementAt(x));
		// now remove the person
		try {
			pedigree.removePerson(pcPerson.getPerson());
		}
		catch(InvalidRelationshipException e) {
			System.err.println("*** Unexpected exception in PedigreeCanvas.deletePerson()");
		}
		pcPersons.removeElement(pcPerson);
		bModified = true;

		updateSize(pcPerson.getLocation(), false);
	}

	public void addGeneticScreenDescriptor(GeneticScreenDescriptor gsd) throws InvalidGeneticScreenDescriptorException {
	// requires: gsd is not null
	// effects:  adds this gsd to this pedcan's list of possible tests.
	//			 Throws an InvalidGeneticScreenDescriptorException is this gsd can't be conducted on people
	//			 or it's name has already been taken

		// make sure this gsd is valid for people and that it's name isn't already used
		int sexSupport = gsd.getSupportedSexType();
		if (!gsd.canTest(disease.getGenotypes(sexSupport)))
			throw new InvalidGeneticScreenDescriptorException();

		for(int x=0; x < geneticDescriptors.size(); x++) {
			GeneticScreenDescriptor oldGSD = (GeneticScreenDescriptor)geneticDescriptors.elementAt(x);
			if (oldGSD.getName().equals(gsd.getName()))
				throw new InvalidGeneticScreenDescriptorException();
		}

		// add it to our list
		geneticDescriptors.addElement(gsd);
		bModified = true;
	}

	public void removeGeneticScreenDescriptor() {
	// effects: ask the user to remove a gsd if there are any

		// create a dialog with list
		if (geneticDescriptors.size() == 0)
			return;

		GeneticScreenPickDialog dialog = new GeneticScreenPickDialog(f, geneticDescriptors);
		dialog.pack();
		dialog.setVisible(true);
		GeneticScreenDescriptor gsd = dialog.getGeneticScreenDescriptor();
		if (gsd == null)
			return;

		// see if it's in use
		for(int x=0; x < pcGeneticScreens.size(); x++) {
			PedCanGeneticScreen pcGS = (PedCanGeneticScreen)pcGeneticScreens.elementAt(x);
			if (pcGS.getGeneticScreen().getGeneticScreenDescriptor().getName().equals(gsd.getName())) {
				JOptionPane.showMessageDialog(f, "This genetic screen is currently in use.\n"+
					"Remove all instances of this test before removing the screen from the pedigree.",
					"Remove Genetic Screening Test", JOptionPane.INFORMATION_MESSAGE);
				return;
			}
		}

		// remove it from list
		geneticDescriptors.removeElement(gsd);
		bModified = true;
	}

	private PedCanGeneticScreen getPedCanGeneticScreenHit(int x, int y) {
	// effects:  Returns the PedCanGeneticScreen hit by this point, else return null

		Enumeration enumPCGeneticScreens = pcGeneticScreens.elements();
		while (enumPCGeneticScreens.hasMoreElements()) {
			PedCanGeneticScreen pcGS = (PedCanGeneticScreen)enumPCGeneticScreens.nextElement();
			if (pcGS.isOn(x,y))
				return pcGS;
		}
		return null;
	}

	private void addGeneticScreen(PedCanPerson pcPerson) {
	// requires:  pcPerson is not null
	// effects:  Creates a GeneticScreen test for this person.  If there are more than
	//			 one possible test that can be added, the user is asked to pick the test

		// are there any tests?
		if (geneticDescriptors.size() == 0) {
			JOptionPane.showMessageDialog(f, "There are no genetic screening tests in this pedigree.",
					"No Genetic Screening Tests", JOptionPane.INFORMATION_MESSAGE);
			return;
		}

		// create a vector of all possible genetic tests
		Vector vGeneticScreenDescriptors = new Vector();
		Enumeration enumPossibleGeneticDescriptors = geneticDescriptors.elements();
		String[] pGenotypes = pcPerson.getPerson().getDisease().getGenotypes(pcPerson.getPerson().getSex());
		while (enumPossibleGeneticDescriptors.hasMoreElements()) {
			GeneticScreenDescriptor possGsd = (GeneticScreenDescriptor)enumPossibleGeneticDescriptors.nextElement();
			if (possGsd.canTest(pGenotypes))
				vGeneticScreenDescriptors.addElement(possGsd);
		}

		// are there any tests?
		if (vGeneticScreenDescriptors.size() == 0) {
			JOptionPane.showMessageDialog(f, "There are no genetic screening tests in this pedigree for this individual.",
					"No Genetic Screening Tests", JOptionPane.INFORMATION_MESSAGE);
			return;
		}

		// remove all tests that are currently used by this person
		Enumeration enumGeneticScreens = pcPerson.getPerson().getGeneticScreens();
		while (enumGeneticScreens.hasMoreElements()) {
			GeneticScreenDescriptor nextGSD = ((GeneticScreen)enumGeneticScreens.nextElement()).getGeneticScreenDescriptor();
			vGeneticScreenDescriptors.removeElement(nextGSD);
		}

		if (vGeneticScreenDescriptors.size() == 0) {
			JOptionPane.showMessageDialog(f, "There are no remaining genetic screening tests for this individual",
					"No Genetic Screening Tests", JOptionPane.INFORMATION_MESSAGE);
			return;
		}

		GeneticScreenDescriptor gsd;
		if (vGeneticScreenDescriptors.size() != 1) {
			GeneticScreenPickDialog dialog = new GeneticScreenPickDialog(f, vGeneticScreenDescriptors);
			dialog.setVisible(true);
			gsd = dialog.getGeneticScreenDescriptor();
			if (gsd == null)
				return;
		}
		else
			gsd = (GeneticScreenDescriptor)vGeneticScreenDescriptors.elementAt(0);

		try {
			GeneticScreen gs = pedigree.createGeneticScreen(pcPerson.getPerson(), gsd);
			Point location = pcPerson.getLocation();
			PedCanGeneticScreen pcGS = new PedCanGeneticScreen(gs, pcPerson, 
				new Point(location.x + gridSize, location.y + gridSize), geneticScreenWidth, lineWidth);
			pcGeneticScreens.addElement(pcGS);
			pcPerson.addLine(pcGS.getPedCanTestLine());
			pcLines.addElement(pcGS.getPedCanTestLine());
			constructLine(pcGS.getPedCanTestLine());
			bModified = true;
		}
		catch(InvalidGeneticScreenDescriptorException e) {
			// should never happen since I made sure this test is ok with person using gsd.canTest(Person)
			System.err.println("Unexpected InvalidGeneticScreenDescriptorException in PedigreeCanvas in addGS");
		}
	}

	private void deleteGeneticScreen(PedCanGeneticScreen pcGS) {
	// requires:  pcGS is not null
	// effects:  Removes the pcGS
		pcGeneticScreens.removeElement(pcGS);
		PedCanPerson pcPerson = pcGS.getPedCanTestLine().getPedCanPerson();
		pcPerson.removeLine(pcGS.getPedCanTestLine());
		pcLines.removeElement(pcGS.getPedCanTestLine());
		pedigree.removeGeneticScreen(pcGS.getGeneticScreen());
		bModified = true;
	}

	public String toString() {
		return "PedigreeCanvas";
	}


}
