//package Person;

//import BayesNetwork.*;
//import Disease.*;
//import GenExceptions.*;
//import GeneticScreen.*;

import java.util.*;

public class Person {

	private int IDnumber;
	private String name;
	private int sexType;
	private String race;
	private int age;

	private Disease disease;


	private double[] genotypeInferences;
	private double[] phenotypeInferences;

	private BayesNetworkNode genotypeNode, phenotypeNode;

	private Vector vGeneticScreens;
	private boolean bIsAffected;

	public static final int MALE = 0;
	public static final int FEMALE = 1;
	public static final int UNKNOWN = 2;

	public static final int numLines = 5; // the number of lines that are in a Person.toString()

// constructor

	public Person(int ID, String name, int sexType, Disease d) {
		// requires:  ID is unqiue for this person, name is not null, sexType is MALE, FEMALE, or UNKNOWN, d is not null
		// effects:  Create a person with these input paramters.
		if ((sexType >= Person.MALE) && (sexType <= Person.UNKNOWN))
			this.sexType = sexType;
		else
			sexType = Person.MALE;

		IDnumber = ID;
		this.name = name;
		disease = d;

		genotypeInferences = null;
		phenotypeInferences = null;
		genotypeNode = null;
		phenotypeNode = null;
		vGeneticScreens = new Vector();
		bIsAffected = false;
	}

	public Person(String s, Disease d) throws MalformedPersonException {
		// requires: s and d are not null
		// effects: Creates a new person using the information in string s
		// the string should be in the format:
		// "Idnumber <idNumber>\n"
		// "Name <name>\n"
		// "Sex <sexType>\n"
		//   - No information about the BayesNodes are included here becuase a pedigree must create these
		//		nodes.
		// A MalformedPersonExcpetion occurs if the string does not contain a valid person


		disease = d;
		genotypeInferences = null;
		phenotypeInferences = null;
		genotypeNode = null;
		phenotypeNode = null;
		vGeneticScreens = new Vector();
		bIsAffected = false;

		StringTokenizer st = new StringTokenizer(s);
		try {
		// read ID and set
			String sTemp = st.nextToken();
			if (!sTemp.equals("IDnumber"))
				throw new MalformedPersonException();
			sTemp = st.nextToken();
			IDnumber = Integer.valueOf(sTemp).intValue();

		// read name and set
			sTemp = st.nextToken();
			if (!sTemp.equals("Name"))
				throw new MalformedPersonException();
			name = st.nextToken();

		// read sex and set
			sTemp = st.nextToken();
			if (!sTemp.equals("Sex"))
				throw new MalformedPersonException();
			sTemp = st.nextToken();
			sexType = Integer.valueOf(sTemp).intValue();
			if ((sexType < Person.MALE) || (sexType > Person.UNKNOWN))
				throw new MalformedPersonException();		
		}
		catch(NumberFormatException e) {
			throw new MalformedPersonException();
		}
		catch(NoSuchElementException e) {
			throw new MalformedPersonException();
		}
		

			
	}

// methods
	public String toString() {
		// effects: returns a string of the following format:
		// "Idnumber <idNumber>\n"
		// "Name <name>\n"
		// "Sex <sexType>\n"
		// "GenotypeObservation <genotypeObs>\n" where genotypeObs is "unknown" if it is null
		// "PhenotypeObservation <phentypeObs>\n"
		String s = new String();
		s += "IDnumber " + IDnumber + "\n";
		s += "Name " + name + "\n";
		s += "Sex " + sexType + "\n";
		s += "GenotypeObservation ";
		String sTemp = getGenotype();
		if (sTemp == null)
			s += "unknown\n";
		else
			s += sTemp + "\n";
		s += "PhenotypeObservation " + getPhenotype();

		return s;
	}

	/// *** methods to access private data

	public String getName() {
		return name;
	}

	public void setName(String sName) {
		name = sName;
	}

	public int getSex() {
		return sexType;
	}

	public Disease getDisease() {
		return disease;
	}

	public int getID() {
		return IDnumber;
	}

	public void setRace(String sRace) {
		race = sRace;
	}

	public String getRace() {
	// effects: the result may be null, especially if the person's disease does not support races
		return race;
	}

	public void setAge(int iAge) {
		age = iAge;
	}

	public int getAge() {
		return age;
	}

	public boolean equals(Object o) {
		try {
			Person p = (Person)o;
			return (p.IDnumber == IDnumber);
		}
		catch(ClassCastException e) {
			return false;
		}
	}

	/// ***** Inference methods
	public double[] getGenotypeInferences() {
		// effects: Returns an array containing the current inference data.  Returns null if no data is available.
		//		To get current data, ask pedigree to update all data.
		return genotypeInferences;
	}
	 
	public void setGenotypeInferences(double[] gInf) {
		genotypeInferences = gInf;
		System.out.println("Genotype Inference array:" + gInf);
		//DEBUG-ALREADY WRONG HERE
		if(gInf != null)
		System.out.println("First element:" +gInf[0]+ "Second element: " +gInf[1]);
	}

	public double[] getPhenotypeInferences() {
		// effects: Returns an array containing the current inference data.  Returns null if no data is available.
		//		To get current data, ask pedigree to update all data.
		return phenotypeInferences;
	}

	public void setPhenotypeInferences(double[] pInf) {
		phenotypeInferences = pInf;
		System.out.println("Pheotype Inference array:" + pInf);
	}


	/// **** Observation methods

  /** @requires: genotypeNode is not null.
   **/
	public String getGenotype() {
	    return genotypeNode.getObservation();
	}

	// sGenotype can't be null, invalid genotype resets
	public void setGenotype(String sGenotype) throws ProbabilityException, InvalidValueException {
	// effects: Set this persons observed genotype value to sGenotype
	//			A probabilityException is thrown if the person cannot be set to this value based on
	//			the probabilistic state of the net (it is impossible, i.e. prob = 0).
	//			An invalidValueException is thrown if the value is not valid.
	//			An exception leaves this node unchanged.
		genotypeNode.makeObservation(sGenotype);
		System.out.println("Set Genotype to: " + sGenotype);
	}

	public void clearGenotypeObservation() {
		genotypeNode.clearObservation();
	}

  /** @require: phenotypeNode is not null.
   **/
	public String getPhenotype() {
	    return phenotypeNode.getObservation();
	}

	public void setPhenotype(String sPhenotype) throws ProbabilityException, InvalidValueException {
	// effects: Set this persons observed phenotype value to sGenotype
	//			A probabilityException is thrown if the person can not be set this this value based on
	//			the probabilistic state of the net (it is impossible, i.e. prob = 0).
	//			An invalidValueException is thrown if the value is not valid.
	//			An exception leaves this node unchanged.
		phenotypeNode.makeObservation(sPhenotype);
		if (sPhenotype.equals("Affected"))
			bIsAffected = true;
		else
			bIsAffected = false;			
	}

	public void clearPhenotypeObservation() {
		phenotypeNode.clearObservation();
		bIsAffected = false;
	}

	public boolean isAffected() {
		return bIsAffected;
	}

	/// BayesNode methods
	public void setPhenotypeNode(BayesNetworkNode bnn) {
		phenotypeNode = bnn;
	}

	public BayesNetworkNode getPhenotypeNode() {
		return phenotypeNode;
	}

	public void setGenotypeNode(BayesNetworkNode bnn) {
		genotypeNode = bnn;
	}

	public BayesNetworkNode getGenotypeNode() {
		return genotypeNode;
	}

	//Genetic Screen stuff
	public void addGeneticScreen(GeneticScreen gs) {
		vGeneticScreens.addElement(gs);
	}

	public void removeGeneticScreen(GeneticScreen gs) {
		vGeneticScreens.removeElement(gs);
	}

	public Enumeration getGeneticScreens() {
		return vGeneticScreens.elements();
	}

}

