//package Person;

//import BayesNetwork.*;
//import Disease.*;
//import GenExceptions.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*; //Property change stuff

public class PersonDialog extends JDialog {
	boolean bPersonChanged;
	JTextField ageTextField; // this may be null if not age dependent
	JComboBox raceComboBox;  // this may be null if not race dependent


	// constructor
	public PersonDialog(JFrame f, Person pPer) {
		super(f, "Edit person attributes", true);

		final Person p = pPer;
		// create name panel
		JPanel namePanel = new JPanel();
		namePanel.setLayout(new BorderLayout());
		JLabel nameLabel = new JLabel("Name:");
		final JTextField nameTextField = new JTextField(20);
		nameTextField.setText(p.getName());
		namePanel.add(nameLabel, BorderLayout.WEST);
		namePanel.add(nameTextField, BorderLayout.EAST);

		// create sex label - this is uneditable
		final JLabel sexLabel;
		if (p.getSex() == Person.MALE)
			sexLabel = new JLabel("Sex:  Male");
		else if (p.getSex() == Person.FEMALE)
			sexLabel = new JLabel("Sex:  Female");
		else
			sexLabel = new JLabel("Sex:  Unknown");

		// create the age panel
		JPanel agePanel = new JPanel();
		if (p.getDisease().isAgeDependent()) {
			ageTextField = new JTextField((new Integer(p.getAge())).toString());
			agePanel.add(new JLabel("Age"));
			agePanel.add(ageTextField);
		}

		// create the race panel
		JPanel racePanel = new JPanel();
		if (p.getDisease().isRaceDependent()) {
			raceComboBox = new JComboBox(p.getDisease().getRaces());
			raceComboBox.setSelectedItem(p.getRace());
			racePanel.add(new JLabel("Race"));
			racePanel.add(raceComboBox);
		}

		// create genotype combobox
		JPanel geneObsPanel = new JPanel();
		geneObsPanel.setLayout(new BorderLayout());
		geneObsPanel.add(new JLabel("Genotype:    "), BorderLayout.WEST);
		Disease d = p.getDisease();
		String[] sGenotypes = d.getGenotypes(p.getSex());
		String[] sPosGenotypes = new String[sGenotypes.length + 1];
		sPosGenotypes[0] = "Unknown";
		for (int x = 1; x <= sGenotypes.length; x++)
			sPosGenotypes[x] = sGenotypes[x - 1];
		final JComboBox genotypesComboBox = new JComboBox(sPosGenotypes);
		String sTemp = p.getGenotype();
		if (sTemp == null)
			sTemp = "Unknown";
		final String sCurGenotype = sTemp;
		genotypesComboBox.setSelectedItem(sCurGenotype);
		geneObsPanel.add(genotypesComboBox);

		// create phenotype combo box
		JPanel phenObsPanel = new JPanel();
		phenObsPanel.setLayout(new BorderLayout());
		phenObsPanel.add(new JLabel("Phenotype:  "), BorderLayout.WEST);
		String[] sPhenotypes = d.getPhenotypes(p.getSex());
		String[] sPosPhenotypes = new String[sPhenotypes.length + 1];
		sPosPhenotypes[0] = "Unknown";
		for (int x = 1; x <= sPhenotypes.length; x++)
			sPosPhenotypes[x] = sPhenotypes[x - 1];
		final JComboBox phenotypesComboBox = new JComboBox(sPosPhenotypes);
		sTemp = p.getPhenotype();
		if (sTemp == null)
			sTemp = "Unknown";
		final String sCurPhenotype = sTemp;
		phenotypesComboBox.setSelectedItem(sTemp);
		phenObsPanel.add(phenotypesComboBox);

		// if inferences are available, add display for them
		Object[] array;

		if ((p.getPhenotypeInferences() != null) && (p.getGenotypeInferences() != null)) {

			// create the inferences label
			JLabel inferenceLabel = new JLabel("Inferences:");

			// create the genotypeInf panel
			JPanel genInfPanel = new JPanel();
			genInfPanel.setLayout(new BorderLayout());
			JComboBox genotypesInfComboBox = new JComboBox(d.getGenotypes(p.getSex()));
			genotypesInfComboBox.setSelectedItem(sGenotypes[0]);
			final JLabel gInfLabel = new JLabel(Double.toString(p.getGenotypeInferences()[0]));
			genotypesInfComboBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JComboBox cb = (JComboBox)e.getSource();
					String sGenotype = (String)cb.getSelectedItem();
					// need to figure out index of this genotype
					String[] aGenotypes = p.getDisease().getGenotypes(p.getSex());
					int x;
					for(x = 0; x < aGenotypes.length; x++)
						if (sGenotype.equals(aGenotypes[x]))
							break;
					gInfLabel.setText(new Float(p.getGenotypeInferences()[x]).toString()); // Display the probs. shortened
				}
			});
			genInfPanel.add(new JLabel("Genotype Inference:    "), BorderLayout.WEST);
			genInfPanel.add(genotypesInfComboBox, BorderLayout.CENTER);
			genInfPanel.add(gInfLabel, BorderLayout.EAST);

			// create the phenotypeInf Panel
			JPanel phenInfPanel = new JPanel();
			phenInfPanel.setLayout(new BorderLayout());
			JComboBox phenotypesInfComboBox = new JComboBox(d.getPhenotypes(p.getSex()));
			phenotypesInfComboBox.setSelectedItem(d.getPhenotypes(p.getSex())[0]);
			final JLabel pInfLabel = new JLabel(Double.toString(p.getPhenotypeInferences()[0]));
			phenotypesInfComboBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JComboBox cb = (JComboBox)e.getSource();
					String sPhenotype = (String)cb.getSelectedItem();
					// need to figure out index of this Phenotype
					String[] aPhenotypes = p.getDisease().getPhenotypes(p.getSex());
					int x;
					for(x = 0; x < aPhenotypes.length; x++)
						if (sPhenotype.equals(aPhenotypes[x]))
							break;
					pInfLabel.setText(new Float(p.getPhenotypeInferences()[x]).toString());
				}
			});
			phenInfPanel.add(new JLabel("Phenotype Inference:  "), BorderLayout.WEST);
			phenInfPanel.add(phenotypesInfComboBox, BorderLayout.CENTER);
			phenInfPanel.add(pInfLabel, BorderLayout.EAST);

			Object[] array2 = {namePanel, sexLabel, geneObsPanel, phenObsPanel, 
				     inferenceLabel, genInfPanel, phenInfPanel};
			array = array2;
		}
		else {
			JLabel inferenceLabel = new JLabel("Inferences are not currently available.");
			Object[] array2 = {namePanel, sexLabel, geneObsPanel, phenObsPanel, inferenceLabel};
			array = array2;
		}

        

        

        final String btnString1 = "OK";
        final String btnString2 = "Cancel";
        Object[] options = {btnString1, btnString2};

        final JOptionPane optionPane = new JOptionPane(array, 
                                    JOptionPane.PLAIN_MESSAGE,
                                    JOptionPane.YES_NO_OPTION,
                                    null,
                                    options, 
									options[0]);
        setContentPane(optionPane);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                optionPane.setValue(new Integer(
                                        JOptionPane.CLOSED_OPTION));
            }
        });

        optionPane.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
                String prop = e.getPropertyName();
				if (isVisible() 
                 && (e.getSource() == optionPane)
                 && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
                     prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
                    Object value = optionPane.getValue();

                    if (value == JOptionPane.UNINITIALIZED_VALUE) {
                        //ignore reset
                        return;
                    }

                    // Reset the JOptionPane's value.
                    // If you don't do this, then if the user
                    // presses the same button next time, no
                    // property change event will be fired.
                    optionPane.setValue(
                            JOptionPane.UNINITIALIZED_VALUE);

                    if (value.equals(btnString1)) {  // the user pushed ok
						// see if anything changed
						bPersonChanged = false;
						String newName = nameTextField.getText();
						String newGenotype = (String)genotypesComboBox.getSelectedItem();
						String newPhenotype = (String)phenotypesComboBox.getSelectedItem();

						if ((!newGenotype.equals(sCurGenotype)) || (!newPhenotype.equals(sCurPhenotype))) {
							try {
								if (newGenotype.equals("Unknown"))
									p.clearGenotypeObservation();
								else
									p.setGenotype(newGenotype);
								if (newPhenotype.equals("Unknown"))
									p.clearPhenotypeObservation();
								else
									p.setPhenotype(newPhenotype);
								bPersonChanged = true;
							}
							catch(InvalidValueException ive) {
								// this should never happen
								System.err.println("Invalid value exception in person dialog.");
							}
							catch(ProbabilityException pe) {
								// this will happen if the user tries to set an invalid
								// phenotype/genotype combination
								errorMessage("The phenotype-genotype combination is probabilistically impossible\n" +
									"based on the disease model and information in the pedigree");
								return;
							}
						}
						if (!p.getName().equals(newName)) {
							p.setName(newName);
							bPersonChanged = true;
						}
						setVisible(false);
					}
					else { // user clicked cancel
                        bPersonChanged = false;
                        setVisible(false);
                    }
                }
            }
        });
	}

	private void errorMessage(String eMessage) {
		JOptionPane.showMessageDialog(this, eMessage,
					"Person Edit Error", JOptionPane.INFORMATION_MESSAGE);
	}

	public boolean didPersonChange() {
		return bPersonChanged;
	}

} // PersonDialog class
