// Import classes used in AutosomalDisease
//import Disease.*;
//import Person.*;
//import BayesNetwork.*;
//import GenExceptions.*;

import java.util.*;

// Import classes used for the testing framework (used in 6.170)
import junit.framework.*;

/** Defines a fixture to run multiple tests on the class Autosomal Disease
 **/
public class testAutosomalDisease extends TestCase{
  private String sDiseaseOne = "DiseaseOne";
  private String sDiseaseTwo = "DiseaseTwo";
  private String sDiseaseThree = "DiseaseThree";
  
  private String sAlleleOne = "AlleleOne";
  private String sAlleleTwo = "AlleleTwo";
  private String[] saTwoAllele = {sAlleleOne, sAlleleTwo};
  
  private String sPhenotypeOne = "PhenotypeOne";
  private String sPhenotypeTwo = "PhenotypeTwo";
  private String sPhenotypeThree = "PhenotypeThree";
  private String[] saThreePhenotype = {sPhenotypeOne, sPhenotypeTwo, sPhenotypeThree};
  
  private AutosomalDisease adDiseaseOne;
  private String sDiseaseTwoString;
  private AutosomalDisease adDiseaseTwo;
  private AutosomalDisease adDiseaseThree;
  
  private double[] daThreeProb = {0.2, 0.3, 0.5};
  
  /** @return Creates a test case with the given name.
   **/
  public testAutosomalDisease(String name){
    super(name);
  }
  
  /** @return Initializes the fixture.
   * Tests the creation of autosomal diseases in the process.
   **/
  protected void setUp(){
    // Setting up cases that should throw exceptions
    /**
    String sAlleleThree = "AlleleThree";
    String[] saOneAllele = {sAlleleOne};
    String[] saThreeAllele = {sAlleleOne, sAlleleTwo, sAlleleThree};
    String[] saDuplicateAllele = {sAlleleOne, sAlleleTwo, sAlleleTwo};
    
    String sPhenotypeFour = "PhenotypeFour";
    String[] saOnePhenotype = {sPhenotypeOne};
    String[] saFourPhenotype = {sPhenotypeOne, sPhenotypeTwo, sPhenotypeThree, sPhenotypeFour};
    String[] saDuplicatePhenotype = {sPhenotypeOne, sPhenotypeOne, sPhenotypeOne};
    
    double[] daOneProb = {1};
    double[] daFourProb = {0.1, 0.2, 0.3, 0.4};
    double[] daSumUnderOneProb = {0.1, 0.2, 0.3};
    double[] daSumOverOneProb = {0.3, 0.4, 0.5};
    **/
    /** Comment each line out as it throws the proper exception.
     * Might want to use ExceptionTestCase here if I figure out how to use it.
     **/
    /**
    try{
    AutosomalDisease adOneAllele = new AutosomalDisease(sDiseaseOne, saOneAllele, saThreePhenotype, daThreeProb);
    AutosomalDisease adThreeAllele = new AutosomalDisease(sDiseaseOne, saThreeAllele, saThreePhenotype, daThreeProb);
    AutosomalDisease adDuplicateAllele = new AutosomalDisease(sDiseaseOne, saDuplicateAllele, saThreePhenotype, daThreeProb);
    AutosomalDisease adOnePhenotype = new AutosomalDisease(sDiseaseOne, saTwoAllele, saOnePhenotype, daThreeProb);
    AutosomalDisease adFourPhenotype = new AutosomalDisease(sDiseaseOne, saTwoAllele, saFourPhenotype, daThreeProb);
    AutosomalDisease adDuplicatePhenotype = new AutosomalDisease(sDiseaseOne, saTwoAllele, saDuplicatePhenotype, daThreeProb);
    AutosomalDisease adOneProb = new AutosomalDisease(sDiseaseOne, saTwoAllele, saThreePhenotype, daOneProb);
    AutosomalDisease adFourProb = new AutosomalDisease(sDiseaseOne, saTwoAllele, saThreePhenotype, daFourProb);
    AutosomalDisease adSumUnderOne = new AutosomalDisease(sDiseaseOne, saTwoAllele, saThreePhenotype, daSumUnderOneProb);
    AutosomalDisease adSumOverOne = new AutosomalDisease(sDiseaseOne, saTwoAllele, saThreePhenotype, daSumOverOneProb);
    **/
    // Now try to create invalid diseases from a string.
    /**
    String sNoTitle = sDiseaseOne +" "+ sAlleleOne +" "+ sAlleleTwo +" "+ sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree +" "+ 0.2 +" "+ 0.3 +" "+ 0.5;
    String sWrongTitle = "X-linkedDisease" +" "+ sDiseaseOne +" "+ sAlleleOne +" "+ sAlleleTwo +" "+ sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree +" "+ 0.2 +" "+ 0.3 +" "+ 0.5;
    String sNoName = "AutosomalDisease" +" "+ sAlleleOne +" "+ sAlleleTwo +" "+ sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree + 0.2 + 0.3 + 0.5;
    String sTwoName = "AutosomalDisease" +" "+ sDiseaseOne +" "+ sDiseaseTwo +" "+ sAlleleOne +" "+ sAlleleTwo +" "+ sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree +" "+ 0.2 +" "+ 0.3 +" "+ 0.5;
    String sOneProb = "AutosomalDisease" +" "+ sDiseaseOne +" "+ sAlleleOne +" "+ sAlleleTwo +" "+ sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree + 0.2;
    String sFourProb = "AutosomalDisease" +" "+ sDiseaseOne +" "+ sAlleleOne +" "+ sAlleleTwo +" "+ sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree +" "+ 0.2 +" "+ 0.3 +" "+ 0.4 +" "+ 0.1;
    String sMissingSpace = "AutosomalDisease" +" "+ sDiseaseOne +" "+ sAlleleOne +" "+ sAlleleTwo +" "+ sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree +" "+ 0.2 +" "+ 0.3 + 0.5;
    AutosomalDisease adNoTitle = new AutosomalDisease(sNoTitle);
    AutosomalDisease adWrongTitle = new AutosomalDisease(sWrongTitle);
    AutosomalDisease adNoName = new AutosomalDisease(sNoName);
    AutosomalDisease adTwoName = new AutosomalDisease(sTwoName);
    AutosomalDisease adOneProbFromString = new AutosomalDisease(sOneProb);
    AutosomalDisease adFourProbFromString = new AutosomalDisease(sFourProb);
    AutosomalDisease adMissingSpace = new AutosomalDisease(sMissingSpace);
    **/
    // None of the above should actually be created.
      try{
    // Create regular disease.
    adDiseaseOne = new AutosomalDisease(sDiseaseOne, saTwoAllele, saThreePhenotype, daThreeProb);
    sDiseaseTwoString = "AutosomalDisease" +" "+ sDiseaseTwo +" "+ sAlleleOne +" "+ sAlleleTwo +" "+ sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree +" "+ 0.2 +" "+ 0.3 +" "+ 0.5;
    adDiseaseTwo = new AutosomalDisease(sDiseaseTwoString);
    adDiseaseThree = new AutosomalDisease();  // The name of this disease will be "Default Autosomal Disease".
    }
    catch (InvalidDiseaseException e){
      InvalidDiseaseException ide = new InvalidDiseaseException("Attempted to create invalid disease");
    }
  } // setUp
  
  /** @return Tests getName(), getGenotypes(int sexType), getPhenotypes(int sexType), getDefaultPhenotype(int sexType), getAlleles(), getPhenotypes(), getGenoProbs().
   **/
  public void testObservers(){
    // Test getName()
    assertEquals("The name should be: " + sDiseaseOne, sDiseaseOne, adDiseaseOne.getName());
    assertEquals("The name should be: " + sDiseaseTwo, sDiseaseTwo, adDiseaseTwo.getName());
    assertEquals("The name should be Default Autosomal Disease.", "Default Autosomal Disease", adDiseaseThree.getName());
    assertEquals("The name should be equal to itself.", adDiseaseOne.getName(), adDiseaseOne.getName());
    String sTestImmutability = adDiseaseOne.getName();
    sTestImmutability = sDiseaseTwo;
    assertEquals("The name of the disease should be immutable", sDiseaseOne, adDiseaseOne.getName());
    
    // Test getGenotypes(int sexType)
    String sA1A1 = sAlleleOne +"-"+ sAlleleOne;
    String sA1A2 = sAlleleOne +"-"+ sAlleleTwo;
    String sA2A2 = sAlleleTwo +"-"+ sAlleleTwo;
    String[] saThreeGenotype = {sA1A1, sA1A2, sA2A2};
    assertEquals("The genotypes should be: " + sA1A1 +" "+ sA1A2 +" "+ sA2A2, saThreeGenotype, adDiseaseOne.getGenotypes(0));
    assertEquals("The genotypes should be: " + sA1A1 +" "+ sA1A2 +" "+ sA2A2, saThreeGenotype, adDiseaseTwo.getGenotypes(1));
    assertEquals("The genotypes should be: " + sA1A1 +" "+ sA1A2 +" "+ sA2A2, saThreeGenotype, adDiseaseThree.getGenotypes(2));
    assertEquals("The genotypes should be equal to itself.", adDiseaseOne.getGenotypes(0), adDiseaseOne.getGenotypes(0));
    assertEquals("The sex should not matter in an autosomal disease", adDiseaseOne.getGenotypes(1), adDiseaseOne.getGenotypes(2));
    String[] saTestImmutability = adDiseaseOne.getGenotypes(0);
    saTestImmutability = saThreePhenotype;
    assertEquals("The genotypes should be immutable.", saThreeGenotype, adDiseaseOne.getGenotypes(0));
    
    // Test getPhenotypes(int sexType)
    assertEquals("The phenotypes should be: " + sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree, saThreePhenotype, adDiseaseOne.getPhenotypes(0));
    assertEquals("The phenotypes should be: " + sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree, saThreePhenotype, adDiseaseTwo.getPhenotypes(1));
    assertEquals("The phenotypes should be: " + sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree, saThreePhenotype, adDiseaseThree.getPhenotypes(2));
    assertEquals("The phenotypes should be equal to itself.", adDiseaseOne.getPhenotypes(0), adDiseaseOne.getPhenotypes(0));
    assertEquals("The sex should not matter in an autosomal disease", adDiseaseOne.getPhenotypes(1), adDiseaseOne.getPhenotypes(2));
    saTestImmutability = adDiseaseOne.getPhenotypes(0);
    saTestImmutability = saThreeGenotype;
    assertEquals("The phenotypes should be immutable.", saThreePhenotype, adDiseaseOne.getPhenotypes(0));
    
    // Test getDefaultPhenotype(int sexType)
    assertEquals("The phenotypes should be: " + sPhenotypeOne, sPhenotypeOne, adDiseaseOne.getDefaultPhenotype(0));
    assertEquals("The phenotypes should be: " + sPhenotypeOne, sPhenotypeOne, adDiseaseTwo.getDefaultPhenotype(1));
    assertEquals("The phenotypes should be: " + sPhenotypeOne, sPhenotypeOne, adDiseaseThree.getDefaultPhenotype(2));
    assertEquals("The phenotypes should be equal to itself.", adDiseaseOne.getDefaultPhenotype(0), adDiseaseOne.getDefaultPhenotype(0));
    assertEquals("The sex should not matter in an autosomal disease", adDiseaseOne.getDefaultPhenotype(1), adDiseaseOne.getDefaultPhenotype(2));
    sTestImmutability = adDiseaseOne.getDefaultPhenotype(0);
    sTestImmutability = sPhenotypeTwo;
    assertEquals("The phenotypes should be immutable.", sPhenotypeOne, adDiseaseOne.getDefaultPhenotype(0));
    
    // Test getAlleles()
    assertEquals("The alleles should be: " + sAlleleOne +" "+ sAlleleTwo, saTwoAllele, adDiseaseOne.getAlleles());
    assertEquals("The alleles should be: " + sAlleleOne +" "+ sAlleleTwo, saTwoAllele, adDiseaseTwo.getAlleles());
    assertEquals("The alleles should be: " + sAlleleOne +" "+ sAlleleTwo, saTwoAllele, adDiseaseThree.getAlleles());
    assertEquals("The alleles should be equal to itself.", adDiseaseOne.getAlleles(), adDiseaseOne.getAlleles());
    saTestImmutability = adDiseaseOne.getAlleles();
    String[] saTwoStrings = {"This", "wrong"};
    saTestImmutability = saTwoStrings;
    assertEquals("The alleles should be immutable.", saTwoAllele, adDiseaseOne.getAlleles());
    
    // Test getPhenotypes()
    assertEquals("The phenotypes should be: " + sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree, saThreePhenotype, adDiseaseOne.getPhenotypes());
    assertEquals("The phenotypes should be: " + sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree, saThreePhenotype, adDiseaseTwo.getPhenotypes());
    assertEquals("The phenotypes should be: " + sPhenotypeOne +" "+ sPhenotypeTwo +" "+ sPhenotypeThree, saThreePhenotype, adDiseaseThree.getPhenotypes());
    assertEquals("The phenotypes should be equal to itself.", adDiseaseOne.getPhenotypes(0), adDiseaseOne.getPhenotypes(0));
    assertEquals("The sex should not matter in an autosomal disease", adDiseaseOne.getPhenotypes(), adDiseaseOne.getPhenotypes(2));
    saTestImmutability = adDiseaseOne.getPhenotypes();
    saTestImmutability = saThreeGenotype;
    assertEquals("The phenotypes should be immutable.", saThreePhenotype, adDiseaseOne.getPhenotypes());
    
    // Test getGenoProbs()
    assertEquals("The probabilities should be: " + 0.2 +" "+ 0.3 +" "+ 0.5, daThreeProb, adDiseaseOne.getGenoProbs());
    assertEquals("The probabilities should be: " + 0.2 +" "+ 0.3 +" "+ 0.5, daThreeProb, adDiseaseTwo.getGenoProbs());
    assertEquals("The probabilities should be: " + 0.2 +" "+ 0.3 +" "+ 0.5, daThreeProb, adDiseaseThree.getGenoProbs());
    assertEquals("The probabilities should be equal to itself.", adDiseaseOne.getGenoProbs(), adDiseaseOne.getGenoProbs());
    double[] daTestImmutability = adDiseaseOne.getGenoProbs();
    daTestImmutability = daThreeProb;
    assertEquals("The probabilities should be immutable.", daThreeProb, adDiseaseOne.getGenoProbs());
    
  } // testObservers
  
  /** @return Tests addPerson, removePerson, addChild, removeChild.
   **/
  public void testBayesNetworkInterface(){
    GNUJavaBayesNet gjbnBayesNetOne = new GNUJavaBayesNet(); // BayesNetwork
    Person pParentOne = new Person(1, "ParentOne", 0, adDiseaseOne); // Male parent
    Person pParentTwo = new Person(2, "ParentTwo", 1, adDiseaseOne); // Female parent
    Person pParentThree = new Person(3, "ParentThree", 2, adDiseaseOne); // Unknown parent
    Person pParentFour = new Person(4, "ParentFour", 0, adDiseaseOne);
    Person pParentFive = new Person(5, "ParentFive", 1, adDiseaseOne);
    Person pParentSix = new Person(6, "ParentSix", 2, adDiseaseOne);
    Person pParentSeven = new Person(7, "ParentSeven", 2, adDiseaseOne);
    Person pChildOne = new Person(8, "ChildOne", 2, adDiseaseOne); // Unknown child
    Person pChildTwo = new Person(9, "ChildTwo", 2, adDiseaseOne);
    Person pChildThree = new Person(10, "ChildThre", 2, adDiseaseOne);
    
    boolean bParentOneGenoNodeFound;
    boolean bParentOnePhenoNodeFound;
    boolean bParentTwoGenoNodeFound;
    boolean bParentTwoPhenoNodeFound;
    boolean bParentThreeGenoNodeFound;
    boolean bParentThreePhenoNodeFound;
    boolean bParentFourGenoNodeFound;
    boolean bParentFourPhenoNodeFound;
    boolean bParentFiveGenoNodeFound;
    boolean bParentFivePhenoNodeFound;
    boolean bParentSixGenoNodeFound;
    boolean bParentSixPhenoNodeFound;
    boolean bParentSevenGenoNodeFound;
    boolean bParentSevenPhenoNodeFound;
    boolean bChildOneGenoNodeFound;
    boolean bChildTwoGenoNodeFound;
    boolean bChildThreeGenoNodeFound;
    boolean bExtraNodeFound;
    
    // Test addPerson
    adDiseaseOne.addPerson(gjbnBayesNetOne, pParentOne); // Creates genotype and phenotype node
    Enumeration eBayesNetNodes = gjbnBayesNetOne.getNodes();
    bParentOneGenoNodeFound = false;
    bParentOnePhenoNodeFound = false;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode ==  pParentOne.getGenotypeNode()) bParentOneGenoNodeFound = true;
      else if (gjbnnNextNode == pParentOne.getGenotypeNode()) bParentOnePhenoNodeFound = true;
      else bExtraNodeFound = true;
    }
    assert("Person has not been added.", (bParentOneGenoNodeFound && bParentOnePhenoNodeFound));
    assert("Extra elements have been added.", !bExtraNodeFound);
    
    adDiseaseOne.addPerson(gjbnBayesNetOne, pParentThree);
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    bParentOneGenoNodeFound = false;
    bParentOnePhenoNodeFound = false;
    bParentThreeGenoNodeFound = false;
    bParentThreePhenoNodeFound = false;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if(gjbnnNextNode == pParentOne.getGenotypeNode()) bParentOneGenoNodeFound = true;
     else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == pParentThree.getGenotypeNode()) bParentThreeGenoNodeFound = true;
      else if (gjbnnNextNode == pParentThree.getPhenotypeNode()) bParentThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
    } // Must take this approach because the elements could be added in any order according to the specs
    assert("Person has not been added.", (bParentOneGenoNodeFound && bParentOnePhenoNodeFound && bParentThreeGenoNodeFound && bParentThreePhenoNodeFound));
    assert("Extra elements have been added.", !bExtraNodeFound);
    
    adDiseaseOne.addPerson(gjbnBayesNetOne, pParentOne); // Add person twice
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    bParentOneGenoNodeFound = false;
    bParentOnePhenoNodeFound = false;
    bParentThreeGenoNodeFound = false;
    bParentThreePhenoNodeFound = false;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) bParentOneGenoNodeFound = true;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == pParentThree.getGenotypeNode()) bParentOneGenoNodeFound = true;
      else if (gjbnnNextNode == pParentThree.getPhenotypeNode()) bParentOnePhenoNodeFound = true;
      else bExtraNodeFound = true;
    }
    assert("Person has not been added.", (bParentOneGenoNodeFound && bParentOnePhenoNodeFound && bParentThreeGenoNodeFound && bParentThreePhenoNodeFound));
    assert("Extra elements have been added.", !bExtraNodeFound);
    
    // Test removePerson
    adDiseaseOne.removePerson(gjbnBayesNetOne, pParentOne);
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    bParentThreeGenoNodeFound = false;
    bParentThreePhenoNodeFound = false;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentThree.getGenotypeNode()) bParentThreeGenoNodeFound = true;
      else if (gjbnnNextNode == pParentThree.getPhenotypeNode()) bParentThreePhenoNodeFound = true;
      else bExtraNodeFound = true;
    }
    assert("Null has been removed.", (bParentThreeGenoNodeFound && bParentThreePhenoNodeFound));
    assert("Extra elements are present", !bExtraNodeFound);
      
    adDiseaseOne.removePerson(gjbnBayesNetOne, pParentOne); // Remove a person twice
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    bParentThreeGenoNodeFound = false;
    bParentThreePhenoNodeFound = false;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentThree.getGenotypeNode()) bParentThreeGenoNodeFound = true;
      else if (gjbnnNextNode == pParentThree.getPhenotypeNode()) bParentThreePhenoNodeFound = true;
      else bExtraNodeFound = true;
    }
    assert("Null has been removed.", (bParentThreeGenoNodeFound && bParentThreePhenoNodeFound));
    assert("Extra elements are present.", !(bExtraNodeFound));
      
    adDiseaseOne.removePerson(gjbnBayesNetOne, pParentThree);
    assert("Person has not been removed.", !(gjbnBayesNetOne.getNodes().hasMoreElements())); // No more elements should be remaining in the BayesNet
      
    // Test addPerson and removePerson together
    adDiseaseOne.addPerson(gjbnBayesNetOne, pParentThree); // Add a person we've just removed
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    bParentThreeGenoNodeFound = false;
    bParentThreePhenoNodeFound = false;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
	if (gjbnnNextNode == pParentThree.getGenotypeNode()) bParentThreeGenoNodeFound = true;
	else if (gjbnnNextNode == pParentThree.getGenotypeNode()) bParentThreePhenoNodeFound = true;
	else bExtraNodeFound = true;
      }
    assert("Person has not been added.", (bParentThreeGenoNodeFound && bParentThreePhenoNodeFound));
    assert("Extra elements have been added.", !bExtraNodeFound);
    adDiseaseOne.removePerson(gjbnBayesNetOne, pParentThree);
    assert("Person has not been removed.", !(gjbnBayesNetOne.getNodes().hasMoreElements())); // No more elements should be remaining in the BayesNet
      
    // Test addChild
    adDiseaseOne.addPerson(gjbnBayesNetOne, pChildOne); // addChild requires that all the people are already added to the BayesNet.
    adDiseaseOne.addPerson(gjbnBayesNetOne, pParentOne);
    adDiseaseOne.addPerson(gjbnBayesNetOne, pParentTwo);
    adDiseaseOne.addChild(gjbnBayesNetOne, pChildOne, pParentOne, pParentTwo);
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    GNUBayesNetNode gjbnnParentOneGenoNode = null;
    GNUBayesNetNode gjbnnParentOnePhenoNode = null;
    GNUBayesNetNode gjbnnParentTwoGenoNode = null;
    GNUBayesNetNode gjbnnParentTwoPhenoNode = null;
    GNUBayesNetNode gjbnnChildOneGenoNode = null;
    GNUBayesNetNode gjbnnChildOnePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
	if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode ==  pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode ==  pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode ==  pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode ==  pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
	else bExtraNodeFound = true;
    }
    assert("An extra element has been created.", !bExtraNodeFound);
    Enumeration eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentOnePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
	if (gjbnnNextNode == gjbnnParentOnePhenoNode) bParentOnePhenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
	else bExtraNodeFound = true;
    }
    assert("First parent relationship has not been created.", (bParentOnePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentTwoPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
	if (gjbnnNextNode ==  gjbnnParentTwoPhenoNode) bParentTwoPhenoNodeFound = true;
	else if (gjbnnNextNode ==  gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
	else bExtraNodeFound = true;
    }
    assert("Second parent relationship has not been created.", (bParentTwoPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    Enumeration eChildsParents = gjbnnChildOneGenoNode.getParents();  // Should have two: Parent1 and Parent2 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
	if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("Child relationship has not been created.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
      
    adDiseaseOne.addChild(gjbnBayesNetOne, pChildOne, pParentOne, pParentTwo); // addChild twice
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
	if (gjbnnNextNode ==  pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode ==  pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode ==  pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode ==  pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
	else bExtraNodeFound = true;
	}
    assert("An extra element has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentOnePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode =(GNUBayesNetNode) eParentsChildren.nextElement();
	if (gjbnnNextNode == gjbnnParentOnePhenoNode) bParentOnePhenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("First parent relationship has not been created.", (bParentOnePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentTwoPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
	if (gjbnnNextNode == gjbnnParentTwoPhenoNode) bParentTwoPhenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("Second parent relationship has not been created.", (bParentTwoPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildOneGenoNode.getParents();  // Should have two: Parent1 and Parent2 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
	if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("Child relationship has not been created.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
      
    adDiseaseOne.addChild(gjbnBayesNetOne, pChildOne, pParentTwo, pParentOne); // Add child again with same parents in different order
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
	if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
	else bExtraNodeFound = true;
    }
    assert("An extra element has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentOnePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
	if (gjbnnNextNode == gjbnnParentOnePhenoNode) bParentOnePhenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("First parent relationship has not been created.", (bParentOnePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentTwoPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
	if (gjbnnNextNode == gjbnnParentTwoPhenoNode) bParentTwoPhenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("Second parent relationship has not been created.", (bParentTwoPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildOneGenoNode.getParents();  // Should have two: Parent1 and Parent2 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
	if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("Child relationship has not been created.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
      
      // Test removeChild
    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildOne, pParentOne, pParentTwo);
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
	if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
	else bExtraNodeFound = true;
	}
    assert("An extra element has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentOnePhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !(eParentsChildren.hasMoreElements()));
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentTwoPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship has been created.", !(eParentsChildren.hasMoreElements()));
    eChildsParents = gjbnnChildOneGenoNode.getParents();  // Should have none
    assert("Child relationship has not been removed.", eChildsParents.hasMoreElements());
    
    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildOne, pParentOne, pParentTwo); // removeChild twice
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
	if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
	else bExtraNodeFound = true;
	}
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentOnePhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !(eParentsChildren.hasMoreElements()));
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentTwoPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship has been created.", !(eParentsChildren.hasMoreElements()));
    eChildsParents = gjbnnChildOneGenoNode.getParents();  // Should have none
    assert("Child relationship has not been removed.", eChildsParents.hasMoreElements());
      
    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildOne, pParentTwo, pParentOne); // removeChild again with same parents in different order
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
	if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
	else bExtraNodeFound = true;
	}
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentOnePhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !(eParentsChildren.hasMoreElements()));
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentTwoPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship has been created.", !(eParentsChildren.hasMoreElements()));
    eChildsParents = gjbnnChildOneGenoNode.getParents();  // Should have none
    assert("Child relationship has not been removed.", eChildsParents.hasMoreElements());
    
    // Complex cases using add and remove child.
    // Add another child while old child is still in BayesNet
    adDiseaseOne.addPerson(gjbnBayesNetOne, pChildTwo);
    adDiseaseOne.addChild(gjbnBayesNetOne, pChildTwo, pParentOne, pParentTwo);
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    GNUBayesNetNode gjbnnChildTwoGenoNode = null;
    GNUBayesNetNode gjbnnChildTwoPhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
	if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
	else bExtraNodeFound = true;
	}
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildTwo GenotypeNode
    bParentOnePhenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
	if (gjbnnNextNode == gjbnnParentOnePhenoNode) bParentOnePhenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildOneGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("First parent relationship has not been created.", (bParentOnePhenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildTwo GenotypeNode
    bParentTwoPhenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
	if (gjbnnNextNode == gjbnnParentTwoPhenoNode) bParentTwoPhenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildOneGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("Second parent relationship has not been created.", (bParentTwoPhenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have none.
    assert("Extra parent relationship is present.", !eChildsParents.hasMoreElements());
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have two: Parent1 and Parent2 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
	if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("Child relationship has not been created.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
      
    adDiseaseOne.addChild(gjbnBayesNetOne, pChildOne, pParentOne, pParentTwo); // Add another child to the same parents
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
	if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
	else bExtraNodeFound = true;
	}
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have three children: PhenotypeNode, ChildOne and ChildTwo GenotypeNode
    bParentOnePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentOnePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
	}
    assert("First parent relationship has not been created.", (bParentOnePhenoNodeFound && bChildOneGenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have three children: PhenotypeNode, ChildOne and ChildTwo GenotypeNode
    bParentTwoPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
	if (gjbnnNextNode == gjbnnParentTwoPhenoNode) bParentTwoPhenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildTwoGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("Second parent relationship has not been created.", (bParentTwoPhenoNodeFound && bChildOneGenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have two: Parent1 and Parent2 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has not been created.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have two: Parent1 and Parent2 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
	GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
	if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
	else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
	else bExtraNodeFound = true;
	}
    assert("Second child relationship has not been created.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
      
    // Add same child with different parents - possible according to specs
    adDiseaseOne.addPerson(gjbnBayesNetOne, pParentFour);
    adDiseaseOne.addPerson(gjbnBayesNetOne, pParentFive);
    adDiseaseOne.addChild(gjbnBayesNetOne, pChildOne, pParentFour, pParentFive);
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    GNUBayesNetNode gjbnnParentFourGenoNode = null;
    GNUBayesNetNode gjbnnParentFourPhenoNode = null;
    GNUBayesNetNode gjbnnParentFiveGenoNode = null;
    GNUBayesNetNode gjbnnParentFivePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have three children: PhenotypeNode, ChildOne and ChildTwo GenotypeNode
    bParentOnePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentOnePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
    }
    assert("First parent relationship has not been created.", (bParentOnePhenoNodeFound && bChildOneGenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have three children: PhenotypeNode, ChildOne and ChildTwo GenotypeNode
    bParentTwoPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentTwoPhenoNode) bParentTwoPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildTwoGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Second parent relationship has not been created.", (bParentTwoPhenoNodeFound && bChildOneGenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third parent relationship has not been created.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has not been created.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have four: Parent1, Parent2, Parent4, Parent5 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has not been created.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound && bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have two: Parent1 and Parent2 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
	if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
    else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
    else bExtraNodeFound = true;
      }
    assert("Second child relationship has not been created.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);

    // Add unconnected family
    adDiseaseOne.addPerson(gjbnBayesNetOne, pParentSix);
    adDiseaseOne.addPerson(gjbnBayesNetOne, pParentSeven);
    adDiseaseOne.addPerson(gjbnBayesNetOne, pChildThree);
    adDiseaseOne.addChild(gjbnBayesNetOne, pChildThree, pParentSix, pParentSeven);
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    gjbnnParentFourGenoNode = null;
    gjbnnParentFourPhenoNode = null;
    gjbnnParentFiveGenoNode = null;
    gjbnnParentFivePhenoNode = null;
    GNUBayesNetNode gjbnnParentSixGenoNode = null;
    GNUBayesNetNode gjbnnParentSixPhenoNode = null;
    GNUBayesNetNode gjbnnParentSevenGenoNode = null;
    GNUBayesNetNode gjbnnParentSevenPhenoNode = null;
    GNUBayesNetNode gjbnnChildThreeGenoNode = null;
    GNUBayesNetNode gjbnnChildThreePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getGenotypeNode()) gjbnnParentSixGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getPhenotypeNode()) gjbnnParentSixPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getGenotypeNode()) gjbnnParentSevenGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getPhenotypeNode()) gjbnnParentSevenPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have three children: PhenotypeNode, ChildOne and ChildTwo GenotypeNode
    bParentOnePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentOnePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First parent relationship has not been created.", (bParentOnePhenoNodeFound && bChildOneGenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have three children: PhenotypeNode, ChildOne and ChildTwo GenotypeNode
    bParentTwoPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentTwoPhenoNode) bParentTwoPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildTwoGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Second parent relationship has not been created.", (bParentTwoPhenoNodeFound && bChildOneGenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third parent relationship has not been created.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has not been created.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSixGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    bParentSixPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSixPhenoNode) bParentSixPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
    }
    assert("Fifth parent relationship has not been created.", (bParentSixPhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSevenGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    bParentSevenPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSevenPhenoNode) bParentSevenPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has not been created.", (bParentSevenPhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have four: Parent1, Parent2, Parent4, Parent5 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has not been created.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound && bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have two: Parent1 and Parent2 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Second child relationship has not been created.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildThreeGenoNode.getParents();  // Should have two: Parent6 and Parent7 GenotypeNode
    bParentSixGenoNodeFound = false;
    bParentSevenGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentSixGenoNode) bParentSixGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentSevenGenoNode) bParentSevenGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third child relationship has not been created.", (bParentSixGenoNodeFound && bParentSevenGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
    
    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildOne, pParentOne, pParentTwo); // Child1 should still be the child of parent4 and 5.  Parent1 and 2 should still be the parents of child2.
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    gjbnnParentFourGenoNode = null;
    gjbnnParentFourPhenoNode = null;
    gjbnnParentFiveGenoNode = null;
    gjbnnParentFivePhenoNode = null;
    gjbnnParentSixGenoNode = null;
    gjbnnParentSixPhenoNode = null;
    gjbnnParentSevenGenoNode = null;
    gjbnnParentSevenPhenoNode = null;
    gjbnnChildThreeGenoNode = null;
    gjbnnChildThreePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getGenotypeNode()) gjbnnParentSixGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getPhenotypeNode()) gjbnnParentSixPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getGenotypeNode()) gjbnnParentSevenGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getPhenotypeNode()) gjbnnParentSevenPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildTwo GenotypeNode
    bParentOnePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentOnePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildTwoGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First parent relationship has been removed.", (bParentOnePhenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildTwo GenotypeNode
    bParentTwoPhenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentTwoPhenoNode) bParentTwoPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildTwoGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Second parent relationship has been removed.", (bParentTwoPhenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third parent relationship has been removed.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSixGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    bParentSixPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSixPhenoNode) bParentSixPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fifth parent relationship has been removed.", (bParentSixPhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSevenGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    bParentSevenPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSevenPhenoNode) bParentSevenPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
    }
    assert("Fourth parent relationship has been removed.", (bParentSevenPhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have two: Parent4, Parent5 GenotypeNode
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has been removed.", (bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have two: Parent1 and Parent2 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Second child relationship has been removed.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);
    eChildsParents = gjbnnChildThreeGenoNode.getParents();  // Should have two: Parent6 and Parent7 GenotypeNode
    bParentSixGenoNodeFound = false;
    bParentSevenGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentSixGenoNode) bParentSixGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentSevenGenoNode) bParentSevenGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third child relationship has been removed.", (bParentSixGenoNodeFound && bParentSevenGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);

    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildTwo, pParentFour, pParentFive); // Remove relationship that never existed
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    gjbnnParentFourGenoNode = null;
    gjbnnParentFourPhenoNode = null;
    gjbnnParentFiveGenoNode = null;
    gjbnnParentFivePhenoNode = null;
    gjbnnParentSixGenoNode = null;
    gjbnnParentSixPhenoNode = null;
    gjbnnParentSevenGenoNode = null;
    gjbnnParentSevenPhenoNode = null;
    gjbnnChildThreeGenoNode = null;
    gjbnnChildThreePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
	 else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getGenotypeNode()) gjbnnParentSixGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pParentSix.getPhenotypeNode()) gjbnnParentSixPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getGenotypeNode()) gjbnnParentSevenGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getPhenotypeNode()) gjbnnParentSevenPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
	else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildTwo GenotypeNode
    bParentOnePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentOnePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildTwoGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First parent relationship has been removed.", (bParentOnePhenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildTwo GenotypeNode
    bParentTwoPhenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentTwoPhenoNode) bParentTwoPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildTwoGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Second parent relationship has been removed.", (bParentTwoPhenoNodeFound && bChildTwoGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third parent relationship has been removed.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSixGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    bParentSixPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSixPhenoNode) bParentSixPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
    }
    assert("Fifth parent relationship has been removed.", (bParentSixPhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSevenGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    bParentSevenPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSevenPhenoNode) bParentSevenPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentSevenPhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have two: Parent4, Parent5 GenotypeNode
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has been removed.", (bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have two: Parent1 and Parent2 GenotypeNode
    bParentOneGenoNodeFound = false;
    bParentTwoGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentOneGenoNode) bParentOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentTwoGenoNode) bParentTwoGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Second child relationship has been removed.", (bParentOneGenoNodeFound && bParentTwoGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);
    eChildsParents = gjbnnChildThreeGenoNode.getParents();  // Should have two: Parent6 and Parent7 GenotypeNode
    bParentSixGenoNodeFound = false;
    bParentSevenGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentSixGenoNode) bParentSixGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentSevenGenoNode) bParentSevenGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third child relationship has been removed.", (bParentSixGenoNodeFound && bParentSevenGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);

    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildTwo, pParentOne, pParentTwo); // Parent1 and 2 have no more children.  Child2 has no more parents.
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    gjbnnParentFourGenoNode = null;
    gjbnnParentFourPhenoNode = null;
    gjbnnParentFiveGenoNode = null;
    gjbnnParentFivePhenoNode = null;
    gjbnnParentSixGenoNode = null;
    gjbnnParentSixPhenoNode = null;
    gjbnnParentSevenGenoNode = null;
    gjbnnParentSevenPhenoNode = null;
    gjbnnChildThreeGenoNode = null;
    gjbnnChildThreePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getGenotypeNode()) gjbnnParentSixGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getPhenotypeNode()) gjbnnParentSixPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getGenotypeNode()) gjbnnParentSevenGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getPhenotypeNode()) gjbnnParentSevenPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentOnePhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Second parent relationship has been removed.", gjbnnParentTwoPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third parent relationship has been removed.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSixGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    bParentSixPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSixPhenoNode) bParentSixPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fifth parent relationship has been removed.", (bParentSixPhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSevenGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    bParentSevenPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSevenPhenoNode) bParentSevenPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentSevenPhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have two: Parent4, Parent5 GenotypeNode
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has not been created.", (bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have none.
    assert("Extra child relationship is present.", !eChildsParents.hasMoreElements());
    eChildsParents = gjbnnChildThreeGenoNode.getParents();  // Should have two: Parent6 and Parent7 GenotypeNode
    bParentSixGenoNodeFound = false;
    bParentSevenGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentSixGenoNode) bParentSixGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentSevenGenoNode) bParentSevenGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third child relationship has been removed.", (bParentSixGenoNodeFound && bParentSevenGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);

    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildThree, pParentSeven, pParentSix); // Only Parent4 and 5 with child1 should remain.
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    gjbnnParentFourGenoNode = null;
    gjbnnParentFourPhenoNode = null;
    gjbnnParentFiveGenoNode = null;
    gjbnnParentFivePhenoNode = null;
    gjbnnParentSixGenoNode = null;
    gjbnnParentSixPhenoNode = null;
    gjbnnParentSevenGenoNode = null;
    gjbnnParentSevenPhenoNode = null;
    gjbnnChildThreeGenoNode = null;
    gjbnnChildThreePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode ==  pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode ==  pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getGenotypeNode()) gjbnnParentSixGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getPhenotypeNode()) gjbnnParentSixPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getGenotypeNode()) gjbnnParentSevenGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getPhenotypeNode()) gjbnnParentSevenPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentOnePhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Second parent relationship has been removed.", gjbnnParentTwoPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third parent relationship has been removed.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSixGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentSixGenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentSevenGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentSevenPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have two: Parent4, Parent5 GenotypeNode
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has not been created.", (bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have none.
    assert("Extra child relationship is present.", !eChildsParents.hasMoreElements());
    eChildsParents = gjbnnChildThreeGenoNode.getParents();  // Should have none.
    assert("Extra chld relationship is present.", !eChildsParents.hasMoreElements());

    adDiseaseOne.addChild(gjbnBayesNetOne, pChildThree, pChildOne, pChildTwo); // Make a former child a parent.
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    gjbnnParentFourGenoNode = null;
    gjbnnParentFourPhenoNode = null;
    gjbnnParentFiveGenoNode = null;
    gjbnnParentFivePhenoNode = null;
    gjbnnParentSixGenoNode = null;
    gjbnnParentSixPhenoNode = null;
    gjbnnParentSevenGenoNode = null;
    gjbnnParentSevenPhenoNode = null;
    gjbnnChildThreeGenoNode = null;
    gjbnnChildThreePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getGenotypeNode()) gjbnnParentSixGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getPhenotypeNode()) gjbnnParentSixPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getGenotypeNode()) gjbnnParentSevenGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getPhenotypeNode()) gjbnnParentSevenPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentOnePhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Second parent relationship has been removed.", gjbnnParentTwoPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third parent relationship has been removed.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSixGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    bParentSixPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSixPhenoNode) bParentSixPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fifth parent relationship has been removed.", (bParentSixPhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSevenGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    bParentSevenPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSevenPhenoNode) bParentSevenPhenoNodeFound = true;
      else if(gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Sixth parent relationship has been removed.", (bParentSevenPhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnChildOneGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    boolean bChildOnePhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnChildOnePhenoNode) bChildOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Seventh parent relationship has been removed.", (bChildOnePhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnChildTwoGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildThree GenotypeNode
    boolean bChildTwoPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnChildTwoPhenoNode) bChildTwoPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
    }
    assert("Eigth parent relationship has been removed.", (bChildTwoPhenoNodeFound && bChildThreeGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have two: Parent4, Parent5 GenotypeNode
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has not been created.", (bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have none.
    assert("Extra child relationship is present.", !eChildsParents.hasMoreElements());
    eChildsParents = gjbnnChildThreeGenoNode.getParents();  // Should have four: Child1, Child2, Parent6 and Parent7 GenotypeNode
    bChildOneGenoNodeFound = false;
    bChildTwoGenoNodeFound = false;
    bParentSixGenoNodeFound = false;
    bParentSevenGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildTwoGenoNode) bChildTwoGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentSixGenoNode) bParentSixGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentSevenGenoNode) bParentSevenGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third child relationship has been removed.", (bChildOneGenoNodeFound && bChildTwoGenoNodeFound && bParentSixGenoNodeFound && bParentSevenGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);

    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildThree, pChildOne, pChildTwo); // Only Parent4 and 5 with child1 should remain.
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    gjbnnParentFourGenoNode = null;
    gjbnnParentFourPhenoNode = null;
    gjbnnParentFiveGenoNode = null;
    gjbnnParentFivePhenoNode = null;
    gjbnnParentSixGenoNode = null;
    gjbnnParentSixPhenoNode = null;
    gjbnnParentSevenGenoNode = null;
    gjbnnParentSevenPhenoNode = null;
    gjbnnChildThreeGenoNode = null;
    gjbnnChildThreePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getGenotypeNode()) gjbnnParentSixGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getPhenotypeNode()) gjbnnParentSixPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getGenotypeNode()) gjbnnParentSevenGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getPhenotypeNode()) gjbnnParentSevenPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    assert("An extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentOnePhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Second parent relationship has been removed.", gjbnnParentTwoPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third parent relationship has been removed.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSixGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Fifth parent relationship has been removed.", gjbnnParentSixPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentSevenGenoNode.getChildren(); // Should have one child: PhenotypeNode
    bParentSevenPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSevenPhenoNode) bParentSevenPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assertEquals("Fourth parent relationship has been removed.", gjbnnParentSevenPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have two: Parent4, Parent5 GenotypeNode
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has not been created.", (bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have none.
    assert("Extra child relationship is present.", !eChildsParents.hasMoreElements());
    eChildsParents = gjbnnChildThreeGenoNode.getParents();  // Should have two: Parent6 and Parent7 GenotypeNode
    bParentSixGenoNodeFound = false;
    bParentSevenGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentSixGenoNode) bParentSixGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentSevenGenoNode) bParentSevenGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third child relationship has been removed.", (bParentSixGenoNodeFound && bParentSevenGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);
    
    adDiseaseOne.removePerson(gjbnBayesNetOne, pChildOne); // Should do nothing since child is connected.
    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildThree, pChildOne, pChildTwo); // Only Parent4 and 5 with child1 should remain.
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    gjbnnParentFourGenoNode = null;
    gjbnnParentFourPhenoNode = null;
    gjbnnParentFiveGenoNode = null;
    gjbnnParentFivePhenoNode = null;
    gjbnnParentSixGenoNode = null;
    gjbnnParentSixPhenoNode = null;
    gjbnnParentSevenGenoNode = null;
    gjbnnParentSevenPhenoNode = null;
    gjbnnChildThreeGenoNode = null;
    gjbnnChildThreePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getGenotypeNode()) gjbnnParentSixGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getPhenotypeNode()) gjbnnParentSixPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getGenotypeNode()) gjbnnParentSevenGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getPhenotypeNode()) gjbnnParentSevenPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    assert("An Extra element is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentOnePhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Second parent relationship has been removed.", gjbnnParentTwoPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third parent relationship has been removed.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSixGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Fifth parent relationship has been removed.", gjbnnParentSixPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentSevenGenoNode.getChildren(); // Should have one child: PhenotypeNode
    bParentSevenPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSevenPhenoNode) bParentSevenPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assertEquals("Fourth parent relationship has been removed.", gjbnnParentSevenPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have two: Parent4, Parent5 GenotypeNode
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has not been created.", (bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have none.
    assert("Extra child relationship is present.", !eChildsParents.hasMoreElements());
    eChildsParents = gjbnnChildThreeGenoNode.getParents();  // Should have two: Parent6 and Parent7 GenotypeNode
    bParentSixGenoNodeFound = false;
    bParentSevenGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentSixGenoNode) bParentSixGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentSevenGenoNode) bParentSevenGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third child relationship has been removed.", (bParentSixGenoNodeFound && bParentSevenGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);

    adDiseaseOne.removePerson(gjbnBayesNetOne, pParentFour); // Should do nothing since parent4 is connected.
    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildThree, pChildOne, pChildTwo); // Only Parent4 and 5 with child1 should remain.
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    gjbnnParentFourGenoNode = null;
    gjbnnParentFourPhenoNode = null;
    gjbnnParentFiveGenoNode = null;
    gjbnnParentFivePhenoNode = null;
    gjbnnParentSixGenoNode = null;
    gjbnnParentSixPhenoNode = null;
    gjbnnParentSevenGenoNode = null;
    gjbnnParentSevenPhenoNode = null;
    gjbnnChildThreeGenoNode = null;
    gjbnnChildThreePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getGenotypeNode()) gjbnnParentSixGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getPhenotypeNode()) gjbnnParentSixPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getGenotypeNode()) gjbnnParentSevenGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getPhenotypeNode()) gjbnnParentSevenPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentOnePhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Second parent relationship has been removed.", gjbnnParentTwoPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third parent relationship has been removed.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSixGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Fifth parent relationship has been removed.", gjbnnParentSixPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentSevenGenoNode.getChildren(); // Should have one child: PhenotypeNode
    bParentSevenPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSevenPhenoNode) bParentSevenPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assertEquals("Fourth parent relationship has been removed.", gjbnnParentSevenPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have two: Parent4, Parent5 GenotypeNode
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has not been created.", (bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have none.
    assert("Extra child relationship is present.", !eChildsParents.hasMoreElements());
    eChildsParents = gjbnnChildThreeGenoNode.getParents();  // Should have two: Parent6 and Parent7 GenotypeNode
    bParentSixGenoNodeFound = false;
    bParentSevenGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentSixGenoNode) bParentSixGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentSevenGenoNode) bParentSevenGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third child relationship has been removed.", (bParentSixGenoNodeFound && bParentSevenGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);
    
    adDiseaseOne.addPerson(gjbnBayesNetOne, pChildOne); // Should do nothing since child1 is already in the BayesNet.
    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildThree, pChildOne, pChildTwo); // Only Parent4 and 5 with child1 should remain.
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    gjbnnParentFourGenoNode = null;
    gjbnnParentFourPhenoNode = null;
    gjbnnParentFiveGenoNode = null;
    gjbnnParentFivePhenoNode = null;
    gjbnnParentSixGenoNode = null;
    gjbnnParentSixPhenoNode = null;
    gjbnnParentSevenGenoNode = null;
    gjbnnParentSevenPhenoNode = null;
    gjbnnChildThreeGenoNode = null;
    gjbnnChildThreePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getGenotypeNode()) gjbnnParentSixGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getPhenotypeNode()) gjbnnParentSixPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getGenotypeNode()) gjbnnParentSevenGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getPhenotypeNode()) gjbnnParentSevenPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentOnePhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Second parent relationship has been removed.", gjbnnParentTwoPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third parent relationship has been removed.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSixGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Fifth parent relationship has been removed.", gjbnnParentSixPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentSevenGenoNode.getChildren(); // Should have one child: PhenotypeNode
    bParentSevenPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSevenPhenoNode) bParentSevenPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assertEquals("Fourth parent relationship has been removed.", gjbnnParentSevenPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have two: Parent4, Parent5 GenotypeNode
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has not been created.", (bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have none.
    assert("Extra child relationship is present.", !eChildsParents.hasMoreElements());
    eChildsParents = gjbnnChildThreeGenoNode.getParents();  // Should have two: Parent6 and Parent7 GenotypeNode
    bParentSixGenoNodeFound = false;
    bParentSevenGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentSixGenoNode) bParentSixGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentSevenGenoNode) bParentSevenGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third child relationship has been removed.", (bParentSixGenoNodeFound && bParentSevenGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);
    
    adDiseaseOne.removePerson(gjbnBayesNetOne, pParentOne);
    adDiseaseOne.removeChild(gjbnBayesNetOne, pChildThree, pChildOne, pChildTwo); // Only Parent4 and 5 with child1 should remain.
    eBayesNetNodes = gjbnBayesNetOne.getNodes();
    gjbnnParentOneGenoNode = null;
    gjbnnParentOnePhenoNode = null;
    gjbnnParentTwoGenoNode = null;
    gjbnnParentTwoPhenoNode = null;
    gjbnnChildOneGenoNode = null;
    gjbnnChildOnePhenoNode = null;
    gjbnnChildTwoGenoNode = null;
    gjbnnChildTwoPhenoNode = null;
    gjbnnParentFourGenoNode = null;
    gjbnnParentFourPhenoNode = null;
    gjbnnParentFiveGenoNode = null;
    gjbnnParentFivePhenoNode = null;
    gjbnnParentSixGenoNode = null;
    gjbnnParentSixPhenoNode = null;
    gjbnnParentSevenGenoNode = null;
    gjbnnParentSevenPhenoNode = null;
    gjbnnChildThreeGenoNode = null;
    gjbnnChildThreePhenoNode = null;
    bExtraNodeFound = false;
    while(eBayesNetNodes.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eBayesNetNodes.nextElement();
      if (gjbnnNextNode == pParentOne.getGenotypeNode()) gjbnnParentOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentOne.getPhenotypeNode()) gjbnnParentOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getGenotypeNode()) gjbnnParentTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentTwo.getPhenotypeNode()) gjbnnParentTwoPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getGenotypeNode()) gjbnnParentFourGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFour.getPhenotypeNode()) gjbnnParentFourPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getGenotypeNode()) gjbnnParentFiveGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentFive.getPhenotypeNode()) gjbnnParentFivePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getGenotypeNode()) gjbnnParentSixGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSix.getPhenotypeNode()) gjbnnParentSixPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getGenotypeNode()) gjbnnParentSevenGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pParentSeven.getPhenotypeNode()) gjbnnParentSevenPhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getGenotypeNode()) gjbnnChildOneGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildOne.getPhenotypeNode()) gjbnnChildOnePhenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getGenotypeNode()) gjbnnChildTwoGenoNode = gjbnnNextNode;
      else if (gjbnnNextNode == pChildTwo.getPhenotypeNode()) gjbnnChildTwoPhenoNode = gjbnnNextNode;
      else bExtraNodeFound = true;
      }
    eParentsChildren = gjbnnParentOneGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Genotype-Phenotype relationship has been removed.", gjbnnParentOnePhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentTwoGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Second parent relationship has been removed.", gjbnnParentTwoPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentFourGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFourPhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFourPhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
    }
    assert("Third parent relationship has been removed.", (bParentFourPhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentFiveGenoNode.getChildren(); // Should have two children: PhenotypeNode and ChildOne GenotypeNode
    bParentFivePhenoNodeFound = false;
    bChildOneGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentFivePhenoNode) bParentOnePhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildOneGenoNode) bChildOneGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Fourth parent relationship has been removed.", (bParentFivePhenoNodeFound && bChildOneGenoNodeFound));
    assert("Extra parent relationship is present.", !bExtraNodeFound);
    eParentsChildren = gjbnnParentSixGenoNode.getChildren(); // Should have one child: PhenotypeNode
    assertEquals("Fifth parent relationship has been removed.", gjbnnParentSixPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eParentsChildren = gjbnnParentSevenGenoNode.getChildren(); // Should have one child: PhenotypeNode
    bParentSevenPhenoNodeFound = false;
    bChildThreeGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eParentsChildren.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eParentsChildren.nextElement();
      if (gjbnnNextNode == gjbnnParentSevenPhenoNode) bParentSevenPhenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnChildThreeGenoNode) bChildThreeGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assertEquals("Fourth parent relationship has been removed.", gjbnnParentSevenPhenoNode, (GNUBayesNetNode) eParentsChildren.nextElement());
    assert("Extra parent relationship is present.", !eParentsChildren.hasMoreElements());
    eChildsParents = gjbnnChildOneGenoNode.getParents(); // Should have two: Parent4, Parent5 GenotypeNode
    bParentFourGenoNodeFound = false;
    bParentFiveGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentFourGenoNode) bParentFourGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentFiveGenoNode) bParentFiveGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("First child relationship has not been created.", (bParentFourGenoNodeFound && bParentFiveGenoNodeFound));
    assert("Extra child relationship has been created.", !bExtraNodeFound);
    eChildsParents = gjbnnChildTwoGenoNode.getParents();  // Should have none.
    assert("Extra child relationship is present.", !eChildsParents.hasMoreElements());
    eChildsParents = gjbnnChildThreeGenoNode.getParents();  // Should have two: Parent6 and Parent7 GenotypeNode
    bParentSixGenoNodeFound = false;
    bParentSevenGenoNodeFound = false;
    bExtraNodeFound = false;
    while(eChildsParents.hasMoreElements()){
      GNUBayesNetNode gjbnnNextNode = (GNUBayesNetNode) eChildsParents.nextElement();
      if (gjbnnNextNode == gjbnnParentSixGenoNode) bParentSixGenoNodeFound = true;
      else if (gjbnnNextNode == gjbnnParentSevenGenoNode) bParentSevenGenoNodeFound = true;
      else bExtraNodeFound = true;
      }
    assert("Third child relationship has been removed.", (bParentSixGenoNodeFound && bParentSevenGenoNodeFound));
    assert("Extra child relationship is present.", !bExtraNodeFound);
  } // testBayesNetworkInterface

  public void testToString(){
    try{
    String sDiseaseTwoToString = adDiseaseTwo.toString();
    assertEquals("The string representation of the disease should be: " + sDiseaseTwo, sDiseaseTwo, sDiseaseTwoToString);
    String sDiseaseOneToString = adDiseaseOne.toString();
    AutosomalDisease adNewDiseaseOne = new AutosomalDisease(sDiseaseOneToString);
    assertEquals("The string representation of the disease should create an exact clone of the disease.", adDiseaseOne, adNewDiseaseOne);
    assertEquals("The string representation of the disease should not create an exact clone of another disease.", adDiseaseThree, adNewDiseaseOne);
    assertEquals("Using the toString function twice should not change the string created.", sDiseaseOneToString, adNewDiseaseOne.toString());
    }
    catch (InvalidDiseaseException e){
      InvalidDiseaseException ide = new InvalidDiseaseException("Attempted to create an invalid disease.");
    }
  } // testToString

  public static Test suite(){
    TestSuite suite = new TestSuite();
    suite.addTest(new testAutosomalDisease("testObservers"));
    suite.addTest(new testAutosomalDisease("testBayesNetworkInterface"));
    suite.addTest(new testAutosomalDisease("testToString"));
    return suite;
  }
  
} // testAutosomalDisease




