//import Person.*;
//import Disease.*;
//import Pedigree.*;
//import BayesNetwork.*;
//import GenExceptions.*;

import java.util.Enumeration;

class testDiseases {

	static public void main (String[] args) {
		testAutosomalDisease();
	}

	static public void testAutosomalDisease() {

		// start with one bad allele, no mutations, no races, no age dependency
		String normalAllele = "Normal Allele";
		String[] badAlleles = {"Bad Allele 1"};
		double[][] expressionRates = {{0, 0, .5}};  // 50 percent chance of affected when you have both bad alleles
		double[][] freqRates = {{.5, .5}};

		NewAutosomalDisease ad = new NewAutosomalDisease("Disease1", normalAllele, badAlleles, null, expressionRates, null, freqRates);

		// add a person
		// remove a person
		// add them back, and create a family
		// remove the family

		Pedigree ped = new Pedigree(ad);
		Person p1 = new Person(0, "Person1", Person.MALE, ad);
		ped.addPerson(p1);
		displayPersonNetwork(p1);

		try {
		ped.removePerson(p1);
		}
		catch(InvalidRelationshipException e) {
			System.out.println("Unexpected exception in removePerson for p1");
		}
		
		Person p2 = new Person(1, "Person2", Person.FEMALE, ad);
		Person p3 = new Person(2, "Person3", Person.MALE, ad);

		try {
			ped.addPerson(p1);
			ped.addPerson(p2);
			ped.addPerson(p3);
			ped.makeMarriage(p1,p2);
			ped.makeChild(p3, p1, p2);
			displayPersonNetwork(p1);
			displayPersonNetwork(p3);

			ped.removeChild(p3);
			displayPersonNetwork(p3);
		}
		catch(InvalidRelationshipException e) {
			System.out.println("Unexpected exception in family tree of three person");
		}
	}

	static private void displayPersonNetwork(Person p) {
		BayesNetworkNode genotypeNode = p.getGenotypeNode();
		BayesNetworkNode phenotypeNode = p.getPhenotypeNode();

		try {
			System.out.println("Genotype Node:");
			String[] genotypes;
			printStringArray(genotypes = genotypeNode.getValues());
			// check for parents
			Enumeration enumParents = genotypeNode.getParents();
			if (enumParents.hasMoreElements()) {
				BayesNetworkNode node1 = (BayesNetworkNode)enumParents.nextElement();
				BayesNetworkNode node2 = (BayesNetworkNode)enumParents.nextElement();
				String[] node1Values = node1.getValues();
				String[] node2Values = node2.getValues();
				BayesNetworkNode[] parentNodes = {node1, node2};
				String[] parentValues = new String[2];
				for (int x = 0; x < node1Values.length; x++) {
					for (int y = 0; y < node2Values.length; y++) {
						parentValues[0] = node1Values[x];
						parentValues[1] = node2Values[y];
						System.out.println("Parent values: " + node1Values[x] + " " + node2Values[y]);
						printDoubleArray(genotypeNode.getProb(parentNodes, parentValues));
					}
				}
			}
			else {
				printDoubleArray(genotypeNode.getProb(null, null));
			}

			System.out.println("Phenotype Node:");
			printStringArray(phenotypeNode.getValues());
			BayesNetworkNode[] parentNodes = {genotypeNode};
			String[] parentValues = new String[1];
			for(int x = 0; x < genotypes.length; x++) {
				System.out.print(genotypes[x]);
				parentValues[0] = genotypes[x];
				printDoubleArray(phenotypeNode.getProb(parentNodes, parentValues));
			}
		}
		catch(ProbabilityException e) {
			System.out.println("unexpected probability exception");
		}
		catch(InvalidValueException e) {
			System.out.println("unepected invalid value exception");
		}
	}

	static private void printStringArray(String[] sArray) {
		for(int x=0; x < sArray.length; x++)
			System.out.print(sArray[x] + " ");
		System.out.println();
	}

	static private void printDoubleArray(double[] dArray) {
		for(int x=0; x < dArray.length; x ++)
			System.out.print(dArray[x] + " ");
		System.out.println();
	}
}

